#include "client.hpp"
#include <Graphics/Graphics.hpp>
#include <Graphics/Texture.hpp>
#include <Graphics/Font.hpp>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <iostream>
#include <iterator>
#include <stdlib.h>
#include "constants.hpp"
#include "common.hpp"
#include "csapp.h"

#define MAX(a, b) (((a) > (b)) ? (a) : (b))

using std::cout;
using std::endl;
using std::vector;
using std::string;
using std::iterator;

char *get_line (char *s, size_t n, FILE *f){
  char *p = fgets (s, n, f);
  if (p != NULL) {
    size_t last = strlen (s) - 1;
    if (s[last] == '\n') 
      s[last] = '\0';
  }
  return p;
}

client::client(char *hos, char *por) {
  host = hos;
  port = por;
  // not safe, need to check atoi and stuff, will do later.
  sock = Open_clientfd(host, atoi(port));
  if (sock == -1) {
    cout << "Connect error, so quitting" << endl;
    exit(1);
  }
  // make it nonblocking
  int opts;
  if ((opts = fcntl(sock, F_GETFL)) < 0)
    cout << "Error getting options" << endl;
  opts = (opts | O_NONBLOCK);
  if (fcntl(sock, F_SETFL, opts) < 0)
    cout << "Error setting options" << endl;
  
  // initiate left and right shift to off and last pm to nobody
  rshift = lshift = false;
  lastpm = string("");

  // load font for later
  font = Graphics::get_font("courier.txf");
  assert(font.ref);

  // scroll bars and screen to 0
  scrollbar = screen = 0;

  // initialize rio
  Rio_readinitb(&rio, sock);

  // set initial mouse_position to middle of the screen
  mouse_pos = make_vector(0.0, 0.0);
  mdown = false;

  // initialize gid so we don't get stuff like -663618176
  gid = 0;

  state.bulletList.reserve(200);
  state.mineList.reserve(2000);
  state.UFOList.reserve(2000);
  state.GunshipList.reserve(2000);
  state.MineshipList.reserve(2000);
  state.nukeList.reserve(2000);
  state.empList.reserve(2000);
  state.hexagonList.reserve(2000);
  state.removeList.reserve(2000);
  
  srand (time(NULL));
  ship o;
  o.position[0] = (rand()%200-100)/100.0f;
  o.position[1] = (rand()%200-100)/100.0f;
  o.orientation = rand()%360;
  o.life = rand()%MAX_HEALTH;
  o.attack = o.armor = o.speed = 0;
  o.player = 0;
  player = o.player;
  state.lifeList.push_back(o);
  
  
  for (int i=1; i<MAX_PLAYERS; i++){
    ship p;
    p.attack = p.armor = p.speed = 0;
    p.position[0] = p.position[1] = p.orientation = 0.0f;
    p.player = i;
    p.life = 100;
    state.lifeList.push_back(p);
  }

  weapon = 0;
  
  for(int i=0; i<MAX_PLAYERS; i++)
    state.flashScreen[i] = 0;
  
  for(int i=0; i<7; i++)
    nweapon[i] = 0;

  state.portalRotation = 0;
  selected = 0;
  playersIG = 0;
}

client::~client(){
}

void client::draw() {
  glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
  glClear(GL_COLOR_BUFFER_BIT);
  
  if (screen == 0){
    draw_login();
  }
  if (screen == 1 || screen == 2 || screen == 3){
    draw_chat();
  }
  if (screen == 2 || screen == 3){
    draw_gameswindow();
  }
  if (screen == 3){
    draw_creategame();
  }
  if (screen == 4){
    draw_lobby();
  }
  if (screen == 5){
    draw_game();
  }
  if (screen == 6)
    draw_losingscreen();
  if (screen == 7)
    draw_winningscreen();
  if (screen == -1){
    draw_background();
    font->draw_center(disc, make_vector(0.0f, 0.0f), 0.05f);
    draw_button( -0.20, -0.80, .4, 0.1f, string("Retry"), -1);  
  }
}

void client::update(float elapsed_time) {
  if (screen == 0 || screen == -1){
    
  }
  else{
    receive_chat();
  }
  if (screen == 5){
  	if(state.flashScreen[player]){
  	  bool nuke = state.flashScreen[player]<=1;
  	  state.flashScreen[player] -= elapsed_time;
  	  if((nuke && state.flashScreen[player] <= 0) || (!nuke && state.flashScreen[player] <= 1))
  	  	state.flashScreen[player] = 0;
  	}
  }
}

void client::handle_event(SDL_Event const &event) {
  if (screen == 5)
    game_handle_event(event);
  else
    chat_handle_event(event);
}


void client::send_chat(const char *p){
  char buf[256], name[256];
  if (p[0] == '/'){
    if (p[1] == 'w' && p[2] == ' '){
      sscanf(p+3, " %s",name);
      sprintf(buf, "%c%s%s", (char)strlen(name), name, p+4+strlen(name));
      send_message(MSG_CLIENT_PM, buf, strlen(buf), sock);
    }
    else if (p[1] == 'r' && p[2] == ' '){
      sprintf(buf, "%c%s%s", (char)(lastpm.length()), lastpm.c_str(), p+3);
      send_message(MSG_CLIENT_PM, buf, strlen(buf), sock);
    }
    else{
      //printf("Invalid command.\n");
    }
  }
  else{
    send_message(MSG_CLIENT_CHAT, p, strlen(p), sock);
  }
}

void client::send_username(const char* p){
  send_message(MSG_CLIENT_INIT, p, strlen(p), sock);
  screen = 1;
  buff.clear();
}

void client::receive_chat(){
  message_t message;
  int name_len, tgid;
  char name[256];
  char mid[4] = ": \0";
  char wf[7] = "From \0";
  char wt[5] = "To \0";
  struct chat temp;
  vector<string>::iterator itr;
  //int *intptr;
  // receieve any given message
  if (read_message(&rio, &message)){
    switch (message.type){
    case MSG_SERVER_PLAYER_STATUS:
      message.contents[message.length] = '\0';
      if (message.contents[0] == 1){
	players.push_back(string(message.contents+1));
      }
      else{
	for (itr = players.begin(); itr < players.end(); itr++){
	  if (*itr == string(message.contents+1))
	    players.erase(itr);
	}
      }
      break;
    case MSG_SERVER_CREATE_GAME:
      message.contents[message.length] = '\0';
      tgid = (unsigned int)message.contents[0];
      games.insert(make_pair(tgid, string(message.contents+1)));
      cout << "received game: " << tgid << endl;
      break;
    case MSG_SERVER_JOIN_GAME:
      message.contents[message.length] = '\0';
      tgid = message.contents[0];
      if (tgid == -1){
	cout << "error joining!" << endl;
      }
      else{
	gid = tgid;
	screen = 4;
      }
      break;
    case MSG_SERVER_LEAVE_GAME:
      message.contents[message.length] = '\0';
      tgid = (int)message.contents[0];
      if (tgid == -1){
	cout << "error leaving!" << endl;
      }
      else{
	screen = 1;
      }
      break;
    case MSG_SERVER_UPDATE_GAME:
      message.contents[message.length] = '\0';
      tgid = message.contents[0];
      //cout << (int)message.contents[1] << endl;
      if ((int)message.contents[1] == 0){
	games.erase(tgid);
      }	
      if (tgid == gid)
	playersIG = (int)message.contents[1];
      break;
    case MSG_SERVER_DISCONNECT:
      message.contents[message.length] = '\0';
      screen = -1;
      disc = string(message.contents);
      break;
    case MSG_SERVER_INFO:
      message.contents[message.length] = '\0';
      temp.s = string(message.contents);
      temp.red = 255;
      temp.green = 0;
      temp.blue = 0;
      chatlog.push_back(temp);
      break;
    case MSG_SERVER_GAME_PLAYERS: {
      cout << "Hello" << endl;
      int num_names = message.contents[0];
      int loc = 1;
      ingame_players.clear();
      for(int x=0; x<num_names; x++) {
        memcpy(name, message.contents+loc+1, message.contents[loc]);
        name[(int)message.contents[loc]] = '\0';
        ingame_players.push_back(name);
        loc += message.contents[loc]+1;  
      }
      break;
    }case MSG_SERVER_CHAT:
      message.contents[message.length] = '\0';
      name_len = message.contents[0];
      memcpy(name, message.contents+1, name_len);
      name[name_len] = '\0';
      temp.s = string(name)+string(mid)+string(message.contents+1+name_len);
      temp.red = 135;
      temp.green = 135;
      temp.blue = 135;
      chatlog.push_back(temp);
      break;
    case MSG_SERVER_PM:
      message.contents[message.length] = '\0';
      name_len = message.contents[0];
      memcpy(name, message.contents+1, name_len);
      name[name_len] = '\0';
      lastpm = string(name);
      temp.s = string(wf)+string(name)+string(mid)+string(message.contents+1+name_len);
      temp.red = 0;
      temp.green = 255;
      temp.blue = 0;
      chatlog.push_back(temp);
      break;
    case MSG_SERVER_PM_CONFIRM:
      message.contents[message.length] = '\0';
      name_len = message.contents[0];
      if(name_len == 0){
	temp.s = string(message.contents+1);
	temp.red = 0;
	temp.green = 255;
	temp.blue = 0;
	chatlog.push_back(temp);
      }
      else {
	memcpy(name, message.contents+1, name_len);
	name[name_len] = '\0';
	temp.s = string(wt)+string(name)+string(mid)+string(message.contents+1+name_len);
	temp.red = 0;
	temp.green = 255;
	temp.blue = 0;
	chatlog.push_back(temp);
      }
      break;
    default:
      receive_gamestate(message);
      break;
    }
  }
}

void client::receive_gamestate(message_t message){
  message.contents[message.length] = '\0';
  
  if(message.type == MSG_GSERVER_INIT){
    cout << "get MSG_GSERVER_INIT" << endl;
    player = message.contents[0];
    screen = 5;
  }      
  else if(message.type == MSG_GSERVER_SHIP_YOU){
    ship *s = NULL;;
    for(unsigned int i = 0; i < state.lifeList.size(); i++)
      if(state.lifeList[i].player == player)
	s=&state.lifeList[i];
    if(s == NULL)
      return;
    int *intptr = (int*)message.contents;
    float *floatptr;
    s->attack = *intptr;
    intptr++;
    s->armor = *intptr;
    intptr++;
    s->speed = *intptr;
    intptr++;
    s->life = *intptr;
    intptr++;
    floatptr= (float*)intptr;
    s->position[0]=*floatptr;
    floatptr++;
    s->position[1]=*floatptr;
    floatptr++;
    s->velocity[0]=*floatptr;
    floatptr++;
    s->velocity[1]=*floatptr;
    floatptr++;
    s->orientation=*floatptr;
    floatptr++;
  }
  else if(message.type == MSG_GSERVER_SHIP_OPP){
    int life;
    int id;
    int *intptr = (int*)message.contents;
    float *floatptr;
    intptr++;
    intptr++;
    intptr++;
    life = *intptr;
    intptr++;
    floatptr= (float*)intptr;
    floatptr++;
    floatptr++;
    floatptr++;
    floatptr++;
    floatptr++;
    intptr = (int*)floatptr;
    id = *intptr;
    
    //printf("Got opposing ship[%d] message: life is %d\n",id,life);
    for(unsigned int i = 0; i < state.lifeList.size(); i++)
      if(state.lifeList[i].player == id)
	state.lifeList[i].life = life;
    
  }
  
  //Case where we get a bullet message
  else if (message.type == MSG_GSERVER_BULLET)
    {
      //cout << "got MSG_GSERVER_BULLER" << endl;
      //Locals
      int id,life,intsize,floatsize,num_items,i,new_id,made_new;
      unsigned int j;
      char *temp;
      ship *s;
      bullet *b,new_bullet;
      mine *m,new_mine;
      ufo *u,new_ufo;
      
      gunship *gs,new_gunship;
      mineship *ms,new_mineship;
      nuke *n, new_nuke;
      emp *e, new_emp;
      hexagon *h, new_hexagon;
      powerup *l, new_local;
      
      
      // Temp pointer to the beginning of the transmission..
      temp = message.contents;
      
      //Set the size of incrementers.
      intsize   = sizeof(int);
      floatsize = sizeof(float);
      
      /************** Begin update your own ship **************/
      
      //Get the message about your own ship. 
      s = NULL;
      //cout << "player: " << player << endl;
      for(unsigned int i = 0; i < state.lifeList.size(); i++) {
        //cout << "lifelist[" << i << "]: " << state.lifeList[i].player << endl;
	if(state.lifeList[i].player == player) {
	  s=&state.lifeList[i];
	}
      }

      if(s == NULL)
        cout << "s is still NULL" << endl;
      
      //Get the updated attack
      s->attack = *((int *)temp);  
      temp += intsize;
      
      //Get the updated armor.
      s->armor  = *((int *)temp);
      temp += intsize;
      
      //Get the updated speed.
      s->speed = *((int *)temp);
      temp += intsize;
      
      //Get the updated life.
      s->life  = *((int *)temp);
      temp += intsize;
      
      //Get the updated position.
      s->position[0] = *((float *)temp);
      temp += floatsize;
      s->position[1] = *((float *)temp);
      temp += floatsize;
      
      //Get updated velocity
      s->velocity[0] = *((float *)temp);
      temp += floatsize;
      s->velocity[1] = *((float *)temp);
      temp += floatsize;
      
      //Get updated orientation
      s->orientation = *((float *)temp);
      temp += floatsize;
      
      //Set the current weapon for the player.
      weapon = *((int *)temp); 
      temp += intsize;

      // set the number of all weapons
      for (int i=0; i<7; i++){
	nweapon[i] = *((int*)temp);
	temp += intsize;
      }

      //cout << "s->attack: " << s->attack << endl;
      //cout << "s->armor: " << s->armor << endl;
      //cout << "s->speed: " << s->speed << endl;
      //cout << "s->life: " << s->life << endl;
      //cout << "s->position: " << s->position[0] << ", " << s->position[1] << endl;
      //cout << "s->velocity: " << s->velocity[0] << ", " << s->velocity[1] << endl;
      /************ End update your own ship ***********/
      
      
      
      /*********** Begin update other ships ***********/
      for(i = 0;i < MAX_PLAYERS;i++)//((int)state.lifeList.size());i++)
        {
          if(i==player)
            continue;
	  
          id = *((int *)temp);
          temp += intsize;
          life = *((int *)temp);
          temp += intsize;
	  
          for(j = 0; j < state.lifeList.size(); j++)
	    {
	      if(state.lifeList[j].player == id)
		{
		  state.lifeList[j].life = life;
		  break;
		}
	    }
        }
      /************End update other ships **************/       
      
      
      
      /*****************Begin Update bullets******************/
      
      //Find the number of bullets in the message and set the pointer.
      num_items = *((int *)temp);
      temp += intsize;
      
      //Now, read all the bullets.
      for(i=0;i<num_items;i++)
        {
          //Get the item id for this bullet.
          new_id = *((int *)temp);
          temp += intsize;
	  
          //Setup check for new bullet or not.
          made_new = 1;
          b = NULL;
	  
          //Check to see if we already have a bullet with this id.  
          for(j = 0; j < state.bulletList.size(); j++)
	    {
	      if(state.bulletList[j].id == new_id)
		{
		  b=&state.bulletList[j];
		  made_new = 0;
		  break;
		}
	    }
	  
          //If not, we need to create a new bullet.
          if(made_new)
	    {
	      made_new = 1;
	      b = &new_bullet;
	      b->id = new_id;
	    }
	  
          //Set the attack of the bullet.
          b->attack = *((int *)temp);
          temp += intsize;
	  
          //Update the position of this bullet.
          b->position[0] = *((float *)temp);
          temp += floatsize;
          b->position[1] = *((float *)temp);
          temp += floatsize;
	  
          //Get the orientation of the bullet.
          b->orientation = *((float *)temp);
          temp += floatsize;
	  
          //get the owner of this bullet. 
          b->player = *((int *)temp);
          temp += intsize;
	  
          //Get the screen of the bullet (should always be us)
          b->screen = *((int *)temp);
          temp += intsize;
	  
          //Get the distance this bullet has traveled.
          b->distance = *((float *)temp);
          temp += floatsize;
	  
          //If we made a new bullet push it onto the list.
          if(made_new)
            state.bulletList.push_back(new_bullet);
        } 
      /*********************** End update bullets ************/
      
      
      
      //Update portal rotation
      state.portalRotation = *((float *)temp);
      temp += floatsize;
      
      
      
      
      /*****************Begin Update mine******************/
      
      //Find the number of bullets in the message and set the pointer.
      num_items = *((int *)temp);
      temp += intsize;
      
      //Now, read all the bullets.
      for(i=0;i<num_items;i++)
        {
          //Get the item id for this bullet.
          new_id = *((int *)temp);
          temp += intsize;
	  
          //Setup check for new bullet or not.
          made_new = 1;
          m = NULL;
	  
          //Check to see if we already have a bullet with this id.  
          for(j = 0; j < state.mineList.size(); j++)
	    {
	      if(state.mineList[j].id == new_id)
		{
		  m=&state.mineList[j];
		  made_new = 0;
		  break;
		}
	    }
	  
          //If not, we need to create a new bullet.
          if(made_new)
	    {
	      made_new = 1;
	      m = &new_mine;
	      m->id = new_id;
	    }
	  
          //Update the position of this bullet.
          m->position[0] = *((float *)temp);
          temp += floatsize;
          m->position[1] = *((float *)temp);
          temp += floatsize;
	  
          //get the owner of this bullet. 
          m->player = *((int *)temp);
          temp += intsize;
	  
          //Get the screen of the bullet (should always be us)
          m->screen = *((int *)temp);
          temp += intsize;
	  
          //If we made a new bullet push it onto the list.
          if(made_new)
            state.mineList.push_back(new_mine);
        } 
      /*********************** End update mines ************/
      
      
      /*****************Begin Update ufos******************/
      
      //Find the number of ufos in the message and set the pointer.
      num_items = *((int *)temp);
      temp += intsize;
      
      //Now, read all the bullets.
      for(i=0;i<num_items;i++)
        {
          //Get the item id for this bullet.
          new_id = *((int *)temp);
          temp += intsize;
	  
          //Setup check for new bullet or not.
          made_new = 1;
          u = NULL;
	  
          //Check to see if we already have a bullet with this id.  
          for(j = 0; j < state.UFOList.size(); j++)
	    {
	      if(state.UFOList[j].id == new_id)
		{
		  u=&state.UFOList[j];
		  made_new = 0;
		  break;
		}
	    }
	  
          //If not, we need to create a new bullet.
          if(made_new)
	    {
	      made_new = 1;
	      u = &new_ufo;
	      u->id = new_id;
	    }
	  
          //Update the position of this bullet.
          u->position[0] = *((float *)temp);
          temp += floatsize;
          u->position[1] = *((float *)temp);
          temp += floatsize;
	  
          //Update the orientation of the ufo
          u->orientation = *((float *)temp);
          temp += floatsize;
	  
          //get the owner of this bullet. 
          u->player = *((int *)temp);
          temp += intsize;
	  
          //Get the screen of the bullet (should always be us)
          u->screen = *((int *)temp);
          temp += intsize;
	  
          //If we made a new bullet push it onto the list.
          if(made_new)
            state.UFOList.push_back(new_ufo);
        } 
      /*********************** End update ufo ************/
      
      
      /*****************Begin Update gunships ******************/
      
      //Find the number of ufos in the message and set the pointer.
      num_items = *((int *)temp);
      temp += intsize;
      
      //Now, read all the bullets.
      for(i=0;i<num_items;i++)
        {
          //Get the item id for this bullet.
          new_id = *((int *)temp);
          temp += intsize;
	  
          //Setup check for new bullet or not.
          made_new = 1;
          gs = NULL;
	  
          //Check to see if we already have a bullet with this id.  
          for(j = 0; j < state.GunshipList.size(); j++)
	    {
	      if(state.GunshipList[j].id == new_id)
		{
		  gs=&state.GunshipList[j];
		  made_new = 0;
		  break;
		}
	    }
	  
          //If not, we need to create a new bullet.
          if(made_new)
	    {
	      made_new = 1;
	      gs = &new_gunship;
	      gs->id = new_id;
	    }
	  
          //Update the position of this bullet.
          gs->position[0] = *((float *)temp);
          temp += floatsize;
          gs->position[1] = *((float *)temp);
          temp += floatsize;
	  
          //Update the orientation of the ufo
          gs->orientation = *((float *)temp);
          temp += floatsize;
	  
          //get the owner of this bullet. 
          gs->player = *((int *)temp);
          temp += intsize;
	  
          //Get the screen of the bullet (should always be us)
          gs->screen = *((int *)temp);
          temp += intsize;
	  
          //If we made a new bullet push it onto the list.
          if(made_new)
            state.GunshipList.push_back(new_gunship);
        } 
      /*********************** End update gunship ************/
      
      
      
      /*****************Begin Update mineship ******************/
      
      //Find the number of ufos in the message and set the pointer.
      num_items = *((int *)temp);
      temp += intsize;
      
      //Now, read all the bullets.
      for(i=0;i<num_items;i++)
        {
          //Get the item id for this bullet.
          new_id = *((int *)temp);
          temp += intsize;
	  
          //Setup check for new bullet or not.
          made_new = 1;
          ms = NULL;
	  
          //Check to see if we already have a bullet with this id.  
          for(j = 0; j < state.MineshipList.size(); j++)
	    {
	      if(state.MineshipList[j].id == new_id)
		{
		  ms=&state.MineshipList[j];
		  made_new = 0;
		  break;
		}
	    }
	  
          //If not, we need to create a new bullet.
          if(made_new)
	    {
	      made_new = 1;
	      ms = &new_mineship;
	      ms->id = new_id;
	    }
	  
          //Update the position of this bullet.
          ms->position[0] = *((float *)temp);
          temp += floatsize;
          ms->position[1] = *((float *)temp);
          temp += floatsize;
	  
          //time until next mine
          ms->mineCD = *((int *)temp);
          temp += intsize;
	  
          //Update the orientation of the ufo
          ms->orientation = *((float *)temp);
          temp += floatsize;
	  
          //get the owner of this bullet. 
          ms->player = *((int *)temp);
          temp += intsize;
	  
          //Get the screen of the bullet (should always be us)
          ms->screen = *((int *)temp);
          temp += intsize;

          //If we made a new bullet push it onto the list.
          if(made_new)
            state.MineshipList.push_back(new_mineship);
        } 
      /*********************** End update mineship ************/
      
      
      /***************** Begin Update nukes ******************/
      
      //Find the number of ufos in the message and set the pointer.
      num_items = *((int *)temp);
      temp += intsize;
      
      //Now, read all the bullets.
      for(i=0;i<num_items;i++)
        {
          //Get the item id for this bullet.
          new_id = *((int *)temp);
          temp += intsize;
	  
          //Setup check for new bullet or not.
          made_new = 1;
          n = NULL;
	  
          //Check to see if we already have a bullet with this id.  
          for(j = 0; j < state.nukeList.size(); j++)
	    {
	      if(state.nukeList[j].id == new_id)
		{
		  n=&state.nukeList[j];
		  made_new = 0;
		  break;
		}
	    }
	  
          //If not, we need to create a new bullet.
          if(made_new)
	    {
	      made_new = 1;
	      n = &new_nuke;
	      n->id = new_id;
	    }
	  
          //Update the position of this bullet.
          n->position[0] = *((float *)temp);
          temp += floatsize;
          n->position[1] = *((float *)temp);
          temp += floatsize;
	  
          //copy the timer
          n->timer = *((int *)temp);
          temp += intsize;
	  
          //get the owner of this bullet. 
          n->player = *((int *)temp);
          temp += intsize;
          
          //Update the orientation of the ufo
          n->orientation = *((float *)temp);
          temp += floatsize;

          //Get the screen of the bullet (should always be us)
          n->screen = *((int *)temp);
          temp += intsize;

          //If we made a new bullet push it onto the list.
          if(made_new)
            state.nukeList.push_back(new_nuke);
        } 
      /*********************** End update nukes ************/
      
      /*****************Begin Update emps******************/
      
      //Find the number of ufos in the message and set the pointer.
      num_items = *((int *)temp);
      temp += intsize;
      
      //Now, read all the bullets.
      for(i=0;i<num_items;i++)
        {
          //Get the item id for this bullet.
          new_id = *((int *)temp);
          temp += intsize;
	  
          //Setup check for new bullet or not.
          made_new = 1;
          e = NULL;
	  
          //Check to see if we already have a bullet with this id.  
          for(j = 0; j < state.empList.size(); j++)
	    {
	      if(state.empList[j].id == new_id)
		{
		  e=&state.empList[j];
		  made_new = 0;
		  break;
		}
	    }
	  
          //If not, we need to create a new bullet.
          if(made_new)
	    {
	      made_new = 1;
	      e = &new_emp;
	      e->id = new_id;
	    }
	  
          //Update the position of this bullet.
          e->position[0] = *((float *)temp);
          temp += floatsize;
          e->position[1] = *((float *)temp);
          temp += floatsize;
	  
          //copy over the timer
          e->timer = *((int *)temp);
          temp += intsize;
	  
          //get the owner of this bullet. 
          e->player = *((int *)temp);
          temp += intsize;
	  
          //Update the orientation of the ufo
          e->orientation = *((float *)temp);
          temp += floatsize;
	  
          //Get the screen of the bullet (should always be us)
          e->screen = *((int *)temp);
          temp += intsize;
	  
          //If we made a new bullet push it onto the list.
          if(made_new)
            state.empList.push_back(new_emp);
        } 
      /*********************** End update emp ************/
      
      
      /*****************Begin Update hexagon ******************/
      
      //Find the number of ufos in the message and set the pointer.
      num_items = *((int *)temp);
      temp += intsize;
      
      //Now, read all the bullets.
      for(i=0;i<num_items;i++)
        {
          //Get the item id for this bullet.
          new_id = *((int *)temp);
          temp += intsize;
	  
          //Setup check for new bullet or not.
          made_new = 1;
          h = NULL;
	  
          //Check to see if we already have a bullet with this id.  
          for(j = 0; j < state.hexagonList.size(); j++)
	    {
	      if(state.hexagonList[j].id == new_id)
		{
		  h=&state.hexagonList[j];
		  made_new = 0;
		  break;
		}
	    }
	  
          //If not, we need to create a new bullet.
          if(made_new)
	    {
	      made_new = 1;
	      h = &new_hexagon;
	      h->id = new_id;
	    }
	  
          //Update the position of this bullet.
          h->position[0] = *((float *)temp);
          temp += floatsize;
          h->position[1] = *((float *)temp);
          temp += floatsize;
	  
          //copy the hexagon's size
          h->size = *((int *)temp);
          temp += intsize;
	  
          //get the owner of this bullet. 
          h->player = *((int *)temp);
          temp += intsize;
          
          //Update the orientation of the ufo
          h->orientation = *((float *)temp);
          temp += floatsize;
	  
          //Get the screen of the bullet (should always be us)
          h->screen = *((int *)temp);
          temp += intsize;
	  
          //If we made a new bullet push it onto the list.
          if(made_new)
            state.hexagonList.push_back(new_hexagon);
        } 
      /*********************** End update hexagon ************/
      
      /*********************** Begin update powerups *********/
      
      //Find the number of ufos in the message and set the pointer.
      num_items = *((int *)temp);
      temp += intsize;
      
      //Now, read all the bullets.
      for(i=0;i<num_items;i++)
        {
          //Get the item id for this bullet.
          new_id = *((int *)temp);
          temp += intsize;
	  
          //Setup check for new bullet or not.
          made_new = 1;
          l = NULL;
	  
          //Check to see if we already have a bullet with this id.  
          for(j = 0; j < state.powerupList.size(); j++)
	    {
	      if(state.powerupList[j].id == new_id)
		{
		  l=&state.powerupList[j];
		  made_new = 0;
		  break;
		}
	    }
	  
          //If not, we need to create a new bullet.
          if(made_new)
	    {
	      made_new = 1;
	      l = &new_local;
	      l->id = new_id;
	    }
	  
          //Update the position of this bullet.
          l->position[0] = *((float *)temp);
          temp += floatsize;
          l->position[1] = *((float *)temp);
          temp += floatsize;
	  
          //Get the screen of the bullet (should always be us)
          l->screen = *((int *)temp);
          temp += intsize;

	  //Copy the type
	  l->type = *((int *)temp);
	  temp += intsize;
	  
          //If we made a new bullet push it onto the list.
          if(made_new)
            state.powerupList.push_back(new_local);
        }     
      
      
      
      /************* Begin update removed items ***************/
      
      //Get the number of items we are removing from the screen
      num_items = *((int *)temp);
      temp += intsize;
      
      //Go through and remove each item.
      for(i=0;i<num_items;i++)
      {
	    bool removed = false;
        //Get the id of the item to remove and see if it should
        // die of just disappear.
        new_id = *((int *)temp);
        temp += intsize;
	  
        //Find the bullet we are removing....
        for(j = 0; j < state.bulletList.size(); j++)
	    {
	      if(state.bulletList[j].id == new_id){
		  	state.bulletList.erase(state.bulletList.begin()+j); 
		  	removed = true;
		  	break;
		  }
	    }
	    if(!removed){
	      for(j = 0; j < state.mineList.size(); j++)
	      {
	        if(state.mineList[j].id == new_id){
		  	  state.mineList.erase(state.mineList.begin()+j); 
		  	  removed = true;
		  	  break;
		    }
	      }
	    }
	    if(!removed){
	      for(j = 0; j < state.UFOList.size(); j++)
	      {
	        if(state.UFOList[j].id == new_id){
		  	  state.UFOList.erase(state.UFOList.begin()+j); 
		  	  removed = true;
		  	  break;
		    }
	      }
	    }
	    if(!removed){
	      for(j = 0; j < state.GunshipList.size(); j++)
	      {
	        if(state.GunshipList[j].id == new_id){
		  	  state.GunshipList.erase(state.GunshipList.begin()+j); 
		  	  removed = true;
		  	  break;
		    }
	      }
	    }
	    if(!removed){
	      for(j = 0; j < state.MineshipList.size(); j++)
	      {
	        if(state.MineshipList[j].id == new_id){
		  	  state.MineshipList.erase(state.MineshipList.begin()+j); 
		  	  removed = true;
		  	  break;
		    }
	      }
	    }
	    if(!removed){
	      for(j = 0; j < state.nukeList.size(); j++)
	      {
	        if(state.nukeList[j].id == new_id){
		  	  state.nukeList.erase(state.nukeList.begin()+j); 
		  	  removed = true;
		  	  break;
		    }
	      }
	    }
	    if(!removed){
	      for(j = 0; j < state.empList.size(); j++)
	      {
	        if(state.empList[j].id == new_id){
		  	  state.empList.erase(state.empList.begin()+j); 
		  	  removed = true;
		  	  break;
		    }
	      }
	    }
	    if(!removed){
	      for(j = 0; j < state.hexagonList.size(); j++)
	      {
	        if(state.hexagonList[j].id == new_id){
		  	  state.hexagonList.erase(state.hexagonList.begin()+j); 
		  	  removed = true;
		  	  break;
		    }
	      }
	    }
	    if (!removed){
	      for(j = 0; j < state.powerupList.size(); j++)
	      {
	        if(state.powerupList[j].id == new_id){
		  	  state.powerupList.erase(state.powerupList.begin()+j); 
		  	  removed = true;
		  	  break;
		    }
	      }
	    }
	  
      }
      
      /************* End update removed items ***************/
      
      //Check if we need to flash the screen and how to flash the screen
      float flash = *((float *)temp);
      temp += floatsize;
      if(flash != 0)
      	state.flashScreen[player] = flash;
    }
  else if (message.type == MSG_PORTAL_ROTATION){
    float *delta_rotation = (float *)message.contents;
    state.portalRotation = *(delta_rotation);
  }


  /********* check for winning and losing conditions and stuff accordingly *********/
  bool win = true;
  for(unsigned int i = 0; i < MAX_PLAYERS; i++) {
    //cout << "lifelist[" << i << "]: " << state.lifeList[i].player << endl;
    if(state.lifeList[i].player == player && state.lifeList[i].life <= 0) {
      screen = 6;
    }
    if(state.lifeList[i].player != player && state.lifeList[i].life > 0) {
      //cout << "gets invalidated by " <<  << endl;
      win = false;
    }
    //cout << "ID: " << state.lifeList[i].player << "Life: " << state.lifeList[i].life << endl;
  }
  if (win){
    //cout << "gets met" << endl;
    screen = 7;
  }
  if (screen == 6 || screen == 7){
    for(unsigned int i = 0; i < state.lifeList.size(); i++) {
      // reset all local client stuff 
      state.lifeList[i].life = 100;
      state.bulletList.clear();
      state.mineList.clear();
      state.UFOList.clear();
      state.GunshipList.clear();
      state.MineshipList.clear();
      state.nukeList.clear();
      state.empList.clear();
      state.hexagonList.clear();
      state.removeList.clear();
      state.powerupList.clear();
      state.portalRotation = 0.0;
      state.itemnumber = 0;
      state.flashScreen[player] = 0;
    }
  }
    
}

void client::draw_background(){
  // draw the background
  Graphics::TextureRef tex = Graphics::get_texture("Resources/stars.png");
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }

  glBegin(GL_QUADS);
  glColor3f(1.0f, 1.0f, 1.0f);
  glTexCoord2f(0.0f, 0.0f);
  glVertex2f(-1.0f, -1.0f);
  glTexCoord2f(0.0f, 1.0f);
  glVertex2f(-1.0f, 1.0f);
  glTexCoord2f(1.0f, 1.0f);
  glVertex2f(1.0f, 1.0f);
  glTexCoord2f(1.0f, 0.0f);
  glVertex2f(1.0f, -1.0f);
  glEnd();

  glDisable(GL_TEXTURE_2D);
}

void client::draw_login_splash(){
  glBegin(GL_QUADS);
  glColor3f(185.0/255, 185.0/255, 185.0/255);
  glVertex2f(-0.5f, -0.05f);
  glVertex2f(0.5f, -0.05f);
  glVertex2f(0.5f, 0.20f);
  glVertex2f(-0.5f, 0.20f);
  glColor3f(135.0/255, 135.0/255, 135.0/255);
  glVertex2f(-0.45f, 0.00f);
  glVertex2f(0.45f, 0.00f);
  glVertex2f(0.45f, 0.15f);
  glVertex2f(-0.45f, 0.15f);
  glEnd();
  
  glColor3f(0.0f, 0.0f, 0.0f);
  font->draw("Username:", make_vector(-0.425f, 0.05f), 0.05f);
  //font->draw("Password:", make_vector(-0.425f, -0.10f), 0.05f);
  
  glBegin(GL_QUADS);
  glColor3f(0.0f, 0.0f, 0.0f);
  glVertex2f(-0.11f, 0.04f);
  glVertex2f(0.41f, 0.04f);
  glVertex2f(0.41f, 0.11f);
  glVertex2f(-0.11f, 0.11f);
  glColor3f(1.0f, 1.0f, 1.0f);
  glVertex2f(-0.1f, 0.05f);
  glVertex2f(0.4f, 0.05f);
  glVertex2f(0.4f, 0.10f);
  glVertex2f(-0.1f, 0.10f);
  glEnd();

  glColor3f(0.0f, 0.0f, 0.0f);
  int k = buff.size()-16;
  if (k < 0) k = 0;
  font->draw(buff.substr(k, buff.size()), make_vector(-0.1f, 0.04f), 0.05f);
  
  /*
  glBegin(GL_QUADS);
  glColor3f(0.0f, 0.0f, 0.0f);
  glVertex2f(-0.11f, -0.04f);
  glVertex2f(0.41f, -0.04f);
  glVertex2f(0.41f, -0.11f);
  glVertex2f(-0.11f, -0.11f);
  glColor3f(1.0f, 1.0f, 1.0f);
  glVertex2f(-0.1f, -.05f);
  glVertex2f(0.4f, -.05f);
  glVertex2f(0.4f, -0.10f);
  glVertex2f(-0.1f, -0.10f);
  glEnd();
  */
}

void client::draw_button(float x, float y, float xsize, float ysize, string text, int activescreen){
  glBegin(GL_QUADS);
  glColor3f(40.0/255, 40.0/255, 40.0/255);
  glVertex2f(x, y);
  glVertex2f(x, y+ysize);
  glVertex2f(x+xsize, y+ysize);
  glVertex2f(x+xsize, y);
  glEnd();
  if (screen == activescreen && mdown == 1 && 
      mouse_pos.x > x && mouse_pos.y < y+ysize &&
      mouse_pos.x < x+xsize && mouse_pos.y > y){
    glBegin(GL_QUADS);
    glColor3f(85.0/255, 85.0/255, 85.0/255);
    glVertex2f(x+.01+(xsize-.02)/10, y+.01+(ysize-.02)/10);
    glVertex2f(x+.01+(xsize-.02)/10, y+ysize-.01-(ysize-.02)/10);
    glVertex2f(x-.01+xsize-(xsize-.02)/10, y+ysize-.01-(ysize-.02)/10);
    glVertex2f(x-.01+xsize-(xsize-.02)/10, y+.01+(ysize-.02)/10); 
    glEnd();
    glColor3f(0.0, 0.0, 0.0);
    font->draw_center(text, make_vector(x-0.02f+xsize/2, y-0.02f+ysize/2), ysize*.8/2); 
  }
  else{
    glBegin(GL_QUADS);
    glColor3f(85.0/255, 85.0/255, 85.0/255);
    glVertex2f(x+.01, y+.01);
    glVertex2f(x+.01, y+ysize-.01);
    glVertex2f(x+xsize-.01, y+ysize-.01);
    glVertex2f(x+xsize-.01, y+.01);  
    glEnd();
    glColor3f(0.0, 0.0, 0.0);
    font->draw_center(text, make_vector(x-0.02f+xsize/2, y-0.02f+ysize/2), ysize/2); 
  }    
}

void client::draw_login(){
  draw_background();
  draw_login_splash();    
}

void client::draw_chat(){
  glClearColor(0.5f, 0.5f, 0.5f, 0.5f);
  glClear(GL_COLOR_BUFFER_BIT);
  
  //draw the background
  draw_background();
  
  // draw the bars that seperate everything
  Graphics::TextureRef tex = Graphics::get_texture("Resources/pipe.png");
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }
  
  glBegin(GL_QUADS);
  glColor3f(1.0, 1.0, 1.0);
  glTexCoord2f(0.0f, 0.0f);
  glVertex2f(0.55, -1.0);
  glTexCoord2f(0.0f, 1.0f);
  glVertex2f(0.55, 1.0);
  glTexCoord2f(1.0f, 1.0f);
  glVertex2f(0.60, 1.0);
  glTexCoord2f(1.0f, 0.0f);
  glVertex2f(0.60, -1.0);
  glEnd();
  glBegin(GL_QUADS);
  glTexCoord2f(0.0f, 0.0f);
  glVertex2f(-1.0, -.85);
  glTexCoord2f(1.0f, 0.0f);
  glVertex2f(1.0, -.85);
  glTexCoord2f(1.0f, 1.0f);
  glVertex2f(1.0, -.90);
  glTexCoord2f(0.0f, 1.0f);
  glVertex2f(-1.0, -.90);  
  glEnd();
  
  glDisable(GL_TEXTURE_2D);
  
  // draw what the person has typed thus far
  if (screen == 1){
    int k = buff.length()-50;
    if (k < 0)
      k=0;
    glColor3f(135.0/255, 135.0/255, 135.0/255);
    font->draw(buff.substr(k,MAXLENGTH), make_vector(-1.0f, -0.975f), 0.05f);
  }
  
  // draw the chat lines
  vector<chat>::reverse_iterator itr;
  itr = chatlog.rbegin();  
  int lines = 0;
  while(itr != chatlog.rend()){
    if (lines > MAXCHATH)
      break;
    int currline = ((*itr).s.size() / MAXLENGTH)+1;
    for (int i=0; i < currline; i++){
      glColor3f((*itr).red/255.0, (*itr).green/255.0, (*itr).blue/255.0);
      font->draw((*itr).s.substr(MAXLENGTH*i, MAXLENGTH), make_vector(-1.0f, -0.9f + 0.05f*(lines+currline-i)), 0.05f); 
    }
    lines += currline;
    itr++;
  }
  
  // draw the player names
  vector<string>::reverse_iterator itr1;
  itr1 = players.rbegin();
  lines = 0;
  while(itr1 != players.rend()){
    if(lines > MAXCHATH)
      break;
    glColor3f(135.0/255, 135.0/255, 135.0/255);
    font->draw((*itr1), make_vector(.60f, -0.85f + 0.05f*lines), 0.05f);
    lines++;
    itr1++;
  }
  
  // draw the game button
  draw_button(0.6f, -1.0f, 0.4f, 0.1f, string("Games"), 1);
}

void client::draw_gameswindow(){
  if (screen == 2){
    glColor4f(0.0f, 0.0f, 0.0f, 0.5f);
    glBegin(GL_QUADS);
    glVertex2f(-1.0f, -1.0f);
    glVertex2f(-1.0f, 1.0f);
    glVertex2f(1.0f, 1.0f);
    glVertex2f(1.0f, -1.0f);
    glEnd();
  }
  
  glColor3f(135.0/255, 135.0/255, 135.0/255);
  glBegin(GL_QUADS);
  glVertex2f(-0.7f, -0.7f);
  glVertex2f(-0.7f, 0.7f);
  glVertex2f(0.7f, 0.7f);
  glVertex2f(0.7f, -0.7f);
  glEnd();
  
  glColor3f(0.0f, 0.0f, 0.0f);
  glBegin(GL_QUADS);
  glVertex2f(-0.69f, -0.69f);
  glVertex2f(-0.69f, 0.69f);
  glVertex2f(0.69f, 0.69f);
  glVertex2f(0.69f, -0.69f);
  glEnd();
  
  draw_button(-0.6f, -0.6f, 1.0/3, 0.1f, string("Join"), 2);
  draw_button(-0.5f+1.0/3, -0.6f, 1.0/3, 0.1f, string("Create"), 2);
  draw_button(-0.4f+2.0/3, -0.6f, 1.0/3, 0.1f, string("Cancel"), 2);
  
  
  // draw the scroll bar
  if (scrollbar > games.size() - 21)
    scrollbar = games.size()-21;
  float proportion = 21.0/games.size();
  if (proportion > 1.0) proportion = 1.0;
  
  glColor3f(85.0f/255, 85.0f/255, 85.0f/255);
  glBegin(GL_QUADS);
  glVertex2f(0.65f, -0.43f);
  glVertex2f(0.65f, 0.60f);
  glVertex2f(0.60f, 0.60f);
  glVertex2f(0.60f, -0.43f);
  
  glColor3f(135.0f/255, 135.0f/255, 135.0f/255);
  glEnd();
  glBegin(GL_QUADS);
  glVertex2f(0.65f, -0.43f);
  glVertex2f(0.65f, -0.48f);
  glVertex2f(0.60f, -0.48f);
  glVertex2f(0.60f, -0.43f);
  glEnd();
  glBegin(GL_QUADS);
  glVertex2f(0.65f, 0.65f);
  glVertex2f(0.65f, 0.60f);
  glVertex2f(0.60f, 0.60f);
  glVertex2f(0.60f, 0.65f);
  glEnd();
  
  glColor3f(0.0f, 0.0f, 0.0f);
  glBegin(GL_TRIANGLES);
  glVertex2f(0.625, 0.64);
  glVertex2f(0.610, 0.61);
  glVertex2f(0.640, 0.61);
  glEnd();
  glBegin(GL_TRIANGLES);
  glVertex2f(0.625, -0.47);
  glVertex2f(0.610, -0.44);
  glVertex2f(0.640, -0.44);
  glEnd();
  
  float dist = 0.60f+0.43f;
  glColor3f(135.0f/255, 135.0f/255, 135.0f/255);
  glBegin(GL_QUADS);
  glVertex2f(0.65f, -0.43f+(scrollbar+.5f)/(games.size()-21)*(1-proportion));
  glVertex2f(0.65f, -0.43f+dist*proportion+(scrollbar+.5f)/(games.size()-21)*(1-proportion));
  glVertex2f(0.60f, -0.43f+dist*proportion+(scrollbar+.5f)/(games.size()-21)*(1-proportion));
  glVertex2f(0.60f, -0.43f+(scrollbar+.5f)/(games.size()-21)*(1-proportion));
  glEnd();
  
  
  // draw game names
  map<int, string>::iterator itr;
  itr = games.begin();  
  int lines = 0;
  for (unsigned int i = 0; i < scrollbar; i++)
    itr++;
  while(itr != games.end()){
    if (lines > 21)
      break;
    glColor3f(135.0/255, 135.0/255, 135.0/255);	
    if (scrollbar + lines == selected){
      //cout << "gid set to: " << gid << endl;
      gid = itr->first;
      glBegin(GL_QUADS);
      glVertex2f(-0.6f, -0.45f+ 0.05f*(lines));
      glVertex2f(-0.6f, -0.45f+ 0.05f*(lines+1));
      glVertex2f(0.59f, -0.45f+ 0.05f*(lines+1));
      glVertex2f(0.59f, -0.45f+ 0.05f*(lines));
      glEnd();
      glColor3f(0.0f, 0.0f, 0.0f);
    }
    else
      glColor3f(135.0/255, 135.0/255, 135.0/255);	
    if (itr->second.length() > 37)
      font->draw(itr->second.substr(0, 37)+string("..."), make_vector(-0.6f, -0.45f + 0.05f*(lines)), 0.05f);
    else
      font->draw(itr->second, make_vector(-0.6f, -0.45f + 0.05f*(lines)), 0.05f);
    lines++;
    itr++;
  }
}

void client::draw_creategame(){
  glColor4f(0.0f, 0.0f, 0.0f, 0.5f);
  glBegin(GL_QUADS);
  glVertex2f(-1.0f, -1.0f);
  glVertex2f(-1.0f, 1.0f);
  glVertex2f(1.0f, 1.0f);
  glVertex2f(1.0f, -1.0f);
  glEnd();
  
  glColor3f(135.0/255, 135.0/255, 135.0/255);
  glBegin(GL_QUADS);
  glVertex2f(-0.5f, -0.2f);
  glVertex2f(-0.5f, 0.2f);
  glVertex2f(0.5f, 0.2f);
  glVertex2f(0.5f, -0.2f);
  glEnd();
  
  glColor3f(0.0f, 0.0f, 0.0f);
  glBegin(GL_QUADS);
  glVertex2f(-0.49f, -0.19f);
  glVertex2f(-0.49f, 0.19f);
  glVertex2f(0.49f, 0.19f);
  glVertex2f(0.49f, -0.19f);
  glEnd();
  
  draw_button(-0.40, -0.15f, 1.0/3, 0.1f, string("Create"), 3);
  draw_button(-0.30f+1.0/3, -0.15f, 1.0/3, 0.1f, string("Cancel"), 3);
  
  glColor3f(135.0/255, 135.0/255, 135.0/255);
  font->draw("Game name:", make_vector(-0.425f, 0.05f), 0.05f);
  
  glBegin(GL_QUADS);
  glColor3f(135.0f/255, 135.0f/255, 135.0f/255);
  glVertex2f(-0.11f, 0.04f);
  glVertex2f(0.41f, 0.04f);
  glVertex2f(0.41f, 0.11f);
  glVertex2f(-0.11f, 0.11f);
  glColor3f(0.0f, 0.0f, 0.0f);
  glVertex2f(-0.1f, 0.05f);
  glVertex2f(0.4f, 0.05f);
  glVertex2f(0.4f, 0.10f);
  glVertex2f(-0.1f, 0.10f);
  glEnd();
  
  glColor3f(135.0/255, 135.0/255, 135.0/255);
  int k = buff.size()-16;
  if (k < 0) k = 0;
  font->draw(buff.substr(k, buff.size()), make_vector(-0.1f, 0.04f), 0.05f);
}

void client::draw_lobby(){
  draw_background();
  
  // draw the bars that seperate everything
  Graphics::TextureRef tex = Graphics::get_texture("Resources/pipe.png");
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }
  
  glBegin(GL_QUADS);
  glColor3f(1.0, 1.0, 1.0);
  glTexCoord2f(0.0f, 0.0f);
  glVertex2f(0.55, -1.0);
  glTexCoord2f(0.0f, 1.0f);
  glVertex2f(0.55, 1.0);
  glTexCoord2f(1.0f, 1.0f);
  glVertex2f(0.60, 1.0);
  glTexCoord2f(1.0f, 0.0f);
  glVertex2f(0.60, -1.0);
  glEnd();
  glBegin(GL_QUADS);
  glTexCoord2f(0.0f, 0.0f);
  glVertex2f(-1.0, -.85);
  glTexCoord2f(1.0f, 0.0f);
  glVertex2f(1.0, -.85);
  glTexCoord2f(1.0f, 1.0f);
  glVertex2f(1.0, -.90);
  glTexCoord2f(0.0f, 1.0f);
  glVertex2f(-1.0, -.90);  
  glEnd();
  
  glDisable(GL_TEXTURE_2D);
  
  // draw what the person has typed thus far
  int k = buff.length()-50;
  if (k < 0)
    k=0;
  glColor3f(135.0/255, 135.0/255, 135.0/255);
  font->draw(buff.substr(k,MAXLENGTH), make_vector(-1.0f, -0.975f), 0.05f);
  
  // draw the chat lines
  vector<chat>::reverse_iterator itr;
  itr = chatlog.rbegin();  
  int lines = 0;
  while(itr != chatlog.rend()){
    if (lines > MAXCHATH)
      break;
    int currline = ((*itr).s.size() / MAXLENGTH)+1;
    for (int i=0; i < currline; i++){
      glColor3f((*itr).red/255.0, (*itr).green/255.0, (*itr).blue/255.0);
      font->draw((*itr).s.substr(MAXLENGTH*i, MAXLENGTH), make_vector(-1.0f, -0.9f + 0.05f*(lines+currline-i)), 0.05f); 
    }
    lines += currline;
    itr++;
  }
  
  // draw the buttons
  draw_button(0.60, -0.9, 0.4f, 0.1f, string("Start"), 4);
  draw_button(0.60, -1.0, 0.4f, 0.1f, string("Quit"), 4);
  
  glColor3f(0.0f, 1.0f, 0.0f);
  char buffer[256];
  sprintf(buffer, "%d/%d Players", playersIG, MAX_PLAYERS);
  font->draw(buffer, make_vector(0.6f, -0.8f), 0.05f);
  
  int size = ingame_players.size();
  for(int x=0; x<size; x++)
    font->draw(ingame_players[x], make_vector(0.6f, -0.5f-0.05f*x), 0.05f);
}

void client::draw_game(){
  glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
  glClear(GL_COLOR_BUFFER_BIT);
  draw_background();
  drawPortals();
  for(unsigned int i = 0; i < state.bulletList.size(); i++)
    drawBullet(state.bulletList[i]);
  for(unsigned int i = 0; i < state.mineList.size(); i++)
    drawMine(state.mineList[i]);
  for(unsigned int i = 0; i < state.UFOList.size(); i++)
    drawUFO(state.UFOList[i]);
  for(unsigned int i = 0; i < state.GunshipList.size(); i++)
    drawGunship(state.GunshipList[i]);
  for(unsigned int i = 0; i < state.MineshipList.size(); i++)
    drawMineship(state.MineshipList[i]);
  for(unsigned int i = 0; i < state.nukeList.size(); i++)
    drawNuke(state.nukeList[i]);
  for(unsigned int i = 0; i < state.empList.size(); i++)
    drawEMP(state.empList[i]);
  for(unsigned int i = 0; i < state.hexagonList.size(); i++)
    drawHexagon(state.hexagonList[i]);
  for(unsigned int i = 0; i < state.powerupList.size(); i++)
    drawPowerup(state.powerupList[i]);
  drawLocal(0.5, 0.9, POWERUP_POWER);
  drawLocal(0.7, 0.9, POWERUP_ARMOR);
  drawLocal(0.9, 0.9, POWERUP_SPEED);
  int row = 0;
  for(unsigned int i = 0; i < state.lifeList.size(); i++){
    if(state.lifeList[i].player != player)
      drawHealth(-0.98, 0.95-0.04*row++, state.lifeList[i].life, state.lifeList[i].player);
    else
      drawShip(state.lifeList[i]);
  }

  //  void client::drawCurrentWeapon(int player, float place, float which)
  for (int i = 0; i < 7; i++){
    //drawCurrentWeapon(player, .77, weapon);
    drawCurrentWeapon(player,-.99f+.297*i, i);
  }
  drawFlash(player);
}

void client::draw_losingscreen(){
  draw_background();
  glColor3f(135.0/255, 135.0/255, 135.0/255);
  font->draw_center(string("You have lost."), make_vector(0.0f, 0.0f), 0.1f);
  font->draw_center(string("Better luck next time."), make_vector(0.0f, -0.3f), 0.1f);
  draw_button( -0.20, -0.80, .4, 0.1f, string("Quit"), 6);  
}

void client::draw_winningscreen(){
  draw_background();
  font->draw_center(string("You have won."), make_vector(0.0f, 0.0f), 0.1f);
  font->draw_center(string("Congrats!"), make_vector(0.0f, -0.3f), 0.1f);
  draw_button( -0.20, -0.80, .4, 0.1f, string("Quit"), 7);  
}

void client::setColor(int player){
  switch (player){
  case -2:glColor4f(0.5f, 0.5f, 0.5f, 0.5f);
    break;
  case -1:glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
    break;
  case 0:glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
    break;
  case 1:glColor4f(0.0f, 0.5f, 1.0f, 1.0f);
    break;
  case 2:glColor4f(0.0f, 1.0f, 0.0f, 1.0f);
    break;
  case 3:glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
    break;
  case 4:glColor4f(0.0f, 1.0f, 1.0f, 1.0f);
    break;
  case 5:glColor4f(1.0f, 0.0f, 1.0f, 1.0f);
    break;
  case 6:glColor4f(1.0f, 1.0f, 0.0f, 1.0f);
    break;
  default:
    break;
  }
}

void client::drawPortals(){
  int portals = state.lifeList.size()-1;
  int count = 0;
  for(unsigned int i = 0; i < state.lifeList.size(); i++){
    if(state.lifeList[i].player == player)
      continue;
    Graphics::TextureRef tex = Graphics::get_texture("Resources/ufo.png");
    if (tex.ref) {
      glEnable(GL_TEXTURE_2D);
      glBindTexture(GL_TEXTURE_2D, tex->obj);
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    }
    glPushMatrix();
    glRotatef((int)(state.portalRotation + count++*360/portals)%360, 0, 0, -1);
    glTranslatef(0, 30.0f*portals/Graphics::screen.y, 0);
    glBegin(GL_QUADS);
    if(state.lifeList[i].life > 0)
    	setColor(state.lifeList[i].player);
    else
    	setColor(-2);
    float size = (PORTAL_SIZE/2.0)/Graphics::screen.y;
    glTexCoord2f(1.0f,0.0f);
    glVertex2f(size, -size/2);
    glTexCoord2f(0.0f,0.0f);
    glVertex2f(-size, -size/2);
    glTexCoord2f(0.0f,1.0f);
    glVertex2f(-size, size/2);
    glTexCoord2f(1.0f,1.0f);
    glVertex2f(size, size/2);
    glEnd();

    glPopMatrix();
    glDisable(GL_TEXTURE_2D);
  }
}

void client::drawShip(ship s){
  Graphics::TextureRef tex = Graphics::get_texture("Resources/ship.png");
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }
  glPushMatrix();
  glTranslatef(s.position[0], s.position[1], 0);
  glRotatef(s.orientation, 0, 0, -1);
  glBegin(GL_QUADS);
  setColor(s.player);
  float size = (SHIP_SIZE/2.0)/Graphics::screen.y;
  glTexCoord2f(1.0f,0.0f);
  glVertex2f(size, -size);
  glTexCoord2f(0.0f,0.0f);
  glVertex2f(-size, -size);
  glTexCoord2f(0.0f,1.0f);
  glVertex2f(-size, size);
  glTexCoord2f(1.0f,1.0f);
  glVertex2f(size, size);
  glEnd();
  glPopMatrix();
  glDisable(GL_TEXTURE_2D);
  drawHealth(0.48, 0.95, s.life, s.player);
}

void client::drawBullet(bullet b){
  glPushMatrix();
  glTranslatef(b.position[0], b.position[1], 0);
  glRotatef(b.orientation, 0, 0, -1);
  glBegin(GL_QUADS);
  setColor(b.player);
  float size = (BULLET_SIZE/2.0)/Graphics::screen.y;
  glVertex2f(size, -size);
  glVertex2f(-size, -size);
  glVertex2f(-size, size);
  glVertex2f(size, size);
  glEnd();
  glPopMatrix();
}

void client::drawMine(mine m){
  Graphics::TextureRef tex = NULL;
  if(m.player == -1)
  	tex = Graphics::get_texture("Resources/mine_powerup.png");
  else
    tex = Graphics::get_texture("Resources/mine.png");
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }
  glPushMatrix();
  glTranslatef(m.position[0], m.position[1], 0);
  glBegin(GL_QUADS);
  setColor(m.player);
  float size = 0;
  if(m.player == -1)
   	size = (POWERUP_SIZE/2.0)/Graphics::screen.y;
  else
   	size = (MINE_SIZE/2.0)/Graphics::screen.y;
  glTexCoord2f(1.0f,0.0f);
  glVertex2f(size, -size);
  glTexCoord2f(0.0f,0.0f);
  glVertex2f(-size, -size);
  glTexCoord2f(0.0f,1.0f);
  glVertex2f(-size, size);
  glTexCoord2f(1.0f,1.0f);
  glVertex2f(size, size);
  glEnd();
  glPopMatrix();
  glDisable(GL_TEXTURE_2D);
}

void client::drawUFO(ufo u){
  Graphics::TextureRef tex = NULL;
  if(u.player == -1)
  	tex = Graphics::get_texture("Resources/ufo_powerup.png");
  else
    tex = Graphics::get_texture("Resources/ufo.png");
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }
  glPushMatrix();
  glTranslatef(u.position[0], u.position[1], 0);
  glRotatef(u.orientation, 0, 0, -1);
  glBegin(GL_QUADS);
  setColor(u.player);
  float size = 0;
  if(u.player == -1)
   	size = (POWERUP_SIZE/2.0)/Graphics::screen.y;
  else
   	size = (UFO_SIZE/2.0)/Graphics::screen.y;
  glTexCoord2f(1.0f,0.0f);
  glVertex2f(size, -size);
  glTexCoord2f(0.0f,0.0f);
  glVertex2f(-size, -size);
  glTexCoord2f(0.0f,1.0f);
  glVertex2f(-size, size);
  glTexCoord2f(1.0f,1.0f);
  glVertex2f(size, size);
  glEnd();
  glPopMatrix();
  glDisable(GL_TEXTURE_2D);
}

void client::drawGunship(gunship gs){
  Graphics::TextureRef tex = NULL;
  if(gs.player == -1)
  	tex = Graphics::get_texture("Resources/gunship_powerup.png");
  else
    tex = Graphics::get_texture("Resources/gunship.png");
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }
  glPushMatrix();
  glTranslatef(gs.position[0], gs.position[1], 0);
  //glRotatef(gs.orientation, 0, 0, -1);
  glBegin(GL_QUADS);
  setColor(gs.player);
  float size = 0;
  if(gs.player == -1)
   	size = (POWERUP_SIZE/2.0)/Graphics::screen.y;
  else
   	size = (GUNSHIP_SIZE/2.0)/Graphics::screen.y;
  glTexCoord2f(1.0f,0.0f);
  glVertex2f(size, -size);
  glTexCoord2f(0.0f,0.0f);
  glVertex2f(-size, -size);
  glTexCoord2f(0.0f,1.0f);
  glVertex2f(-size, size);
  glTexCoord2f(1.0f,1.0f);
  glVertex2f(size, size);
  glEnd();
  glPopMatrix();
  glDisable(GL_TEXTURE_2D);
}

void client::drawMineship(mineship ms){
  Graphics::TextureRef tex = NULL;
  if(ms.player == -1)
  	tex = Graphics::get_texture("Resources/mineship_powerup.png");
  else
    tex = Graphics::get_texture("Resources/mineship.png");
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }
  glPushMatrix();
  glTranslatef(ms.position[0], ms.position[1], 0);
  glRotatef(ms.orientation, 0, 0, -1);
  glBegin(GL_QUADS);
  setColor(ms.player);
  float size = 0;
  if(ms.player == -1)
   	size = (POWERUP_SIZE/2.0)/Graphics::screen.y;
  else
   	size = (MINESHIP_SIZE/2.0)/Graphics::screen.y;
  glTexCoord2f(1.0f,0.0f);
  glVertex2f(size, -size);
  glTexCoord2f(0.0f,0.0f);
  glVertex2f(-size, -size);
  glTexCoord2f(0.0f,1.0f);
  glVertex2f(-size, size);
  glTexCoord2f(1.0f,1.0f);
  glVertex2f(size, size);
  glEnd();
  glPopMatrix();
  glDisable(GL_TEXTURE_2D);
}

void client::drawNuke(nuke n){
  Graphics::TextureRef tex = NULL;
  if(n.player == -1)
  	tex = Graphics::get_texture("Resources/nuke_powerup.png");
  else
    tex = Graphics::get_texture("Resources/nuke.png");
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }
  glPushMatrix();
  glTranslatef(n.position[0], n.position[1], 0);
  glRotatef(n.orientation, 0, 0, -1);
  glBegin(GL_QUADS);
  setColor(n.player);
  float size = 0;
  if(n.player == -1)
   	size = (POWERUP_SIZE/2.0)/Graphics::screen.y;
  else
   	size = (NUKE_SIZE/2.0)/Graphics::screen.y;
  glTexCoord2f(1.0f,0.0f);
  glVertex2f(size, -size);
  glTexCoord2f(0.0f,0.0f);
  glVertex2f(-size, -size);
  glTexCoord2f(0.0f,1.0f);
  glVertex2f(-size, size);
  glTexCoord2f(1.0f,1.0f);
  glVertex2f(size, size);
  glEnd();
  glPopMatrix();
  glDisable(GL_TEXTURE_2D);
}

void client::drawEMP(emp e){
  Graphics::TextureRef tex = NULL;
  if(e.player == -1)
  	tex = Graphics::get_texture("Resources/emp_powerup.png");
  else
    tex = Graphics::get_texture("Resources/emp.png");
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }
  glPushMatrix();
  glTranslatef(e.position[0], e.position[1], 0);
  glRotatef(e.orientation, 0, 0, -1);
  glBegin(GL_QUADS);
  setColor(e.player);
  float size = 0;
  if(e.player == -1)
   	size = (POWERUP_SIZE/2.0)/Graphics::screen.y;
  else
   	size = (EMP_SIZE/2.0)/Graphics::screen.y;
  glTexCoord2f(1.0f,0.0f);
  glVertex2f(size, -size);
  glTexCoord2f(0.0f,0.0f);
  glVertex2f(-size, -size);
  glTexCoord2f(0.0f,1.0f);
  glVertex2f(-size, size);
  glTexCoord2f(1.0f,1.0f);
  glVertex2f(size, size);
  glEnd();
  glPopMatrix();
  glDisable(GL_TEXTURE_2D);
}

void client::drawHexagon(hexagon h){
  Graphics::TextureRef tex = NULL;
  if(h.player == -1)
  	tex = Graphics::get_texture("Resources/hexagon_powerup.png");
  else
    tex = Graphics::get_texture("Resources/hexagon.png");
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }
  glPushMatrix();
  glTranslatef(h.position[0], h.position[1], 0);
  glBegin(GL_QUADS);
  setColor(h.player);
  float size = 0;
  if(h.player == -1)
   	size = (POWERUP_SIZE/2.0)/Graphics::screen.y;
  else
   	size = ((h.size)/2.0)/Graphics::screen.y;
  glTexCoord2f(1.0f,0.0f);
  glVertex2f(size, -size);
  glTexCoord2f(0.0f,0.0f);
  glVertex2f(-size, -size);
  glTexCoord2f(0.0f,1.0f);
  glVertex2f(-size, size);
  glTexCoord2f(1.0f,1.0f);
  glVertex2f(size, size);
  glEnd();
  glPopMatrix();
  glDisable(GL_TEXTURE_2D);
}

void client::drawPowerup(powerup p){
  Graphics::TextureRef tex = NULL;
  if (p.type == POWERUP_POWER)
    tex = Graphics::get_texture("Resources/gun_powerup.png");
  else if (p.type == POWERUP_HEALTH)
    tex = Graphics::get_texture("Resources/health_powerup.png");
  else if (p.type == POWERUP_POWER)
    tex = Graphics::get_texture("Resources/shield_powerup.png");
  else
    tex = Graphics::get_texture("Resources/speed_powerup.png");
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }
  glPushMatrix();
  glTranslatef(p.position[0], p.position[1], 0);
  glBegin(GL_QUADS);
  setColor(-1);
  float size = (POWERUP_SIZE/2.0)/Graphics::screen.y;
  glTexCoord2f(1.0f,0.0f);
  glVertex2f(size, -size);
  glTexCoord2f(0.0f,0.0f);
  glVertex2f(-size, -size);
  glTexCoord2f(0.0f,1.0f);
  glVertex2f(-size, size);
  glTexCoord2f(1.0f,1.0f);
  glVertex2f(size, size);
  glEnd();
  glPopMatrix();
  glDisable(GL_TEXTURE_2D);
}

void client::drawHealth(float x, float y, int health, int player){
  glBegin(GL_LINES);
  setColor(player);
  glVertex2f(x+0.5, y);
  glVertex2f(x, y);
  glVertex2f(x, y);
  glVertex2f(x, y+0.03);
  glVertex2f(x, y+0.03);
  glVertex2f(x+0.5, y+0.03);
  glVertex2f(x+0.5, y+0.03);
  glVertex2f(x+0.5, y);
  glEnd();

  glBegin(GL_QUADS);
  setColor(player);
  glVertex2f(x+0.495 - 0.49*((float)(MAX_HEALTH-health)/MAX_HEALTH), y+.005);
  glVertex2f(x+.005, y+.005);
  glVertex2f(x+.005, y+0.025);
  glVertex2f(x+0.495 - 0.49*((float)(MAX_HEALTH-health)/MAX_HEALTH), y+0.025);
  glEnd();
}

void client::drawLocal(float x, float y, int which){
  Graphics::TextureRef tex = NULL;
  char buf[256];
  if (which == POWERUP_POWER){
    tex = Graphics::get_texture("Resources/gun_powerup.png");
    sprintf(buf, "x%d", state.lifeList[player].attack);
  }
  else if (which == POWERUP_ARMOR){
    tex = Graphics::get_texture("Resources/shield_powerup.png");
    sprintf(buf, "x%d", state.lifeList[player].armor);
  }
  else{
    tex = Graphics::get_texture("Resources/speed_powerup.png");
    sprintf(buf, "x%d", state.lifeList[player].speed);
  }
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }
  glPushMatrix();
  glTranslatef(x, y, 0);
  glBegin(GL_QUADS);
  setColor(-1);
  float size = (POWERUP_SIZE/3.0)/Graphics::screen.y;
  glTexCoord2f(1.0f,0.0f);
  glVertex2f(size, -size);
  glTexCoord2f(0.0f,0.0f);
  glVertex2f(-size, -size);
  glTexCoord2f(0.0f,1.0f);
  glVertex2f(-size, size);
  glTexCoord2f(1.0f,1.0f);
  glVertex2f(size, size);
  glEnd();
  glPopMatrix();
  glDisable(GL_TEXTURE_2D);
  Vector2f pos;
  pos.x = x+(POWERUP_SIZE/3.0)/Graphics::screen.y;
  pos.y = y-.05;
  font->draw(buf, pos, 0.05f);
}

void client::drawCurrentWeapon(int player, float place, int which)
{
  int osize = 0;
  float size;
  char buf[10];
  Graphics::TextureRef tex; 
  if (which == weapon)
    setColor(player);
  else
    setColor(-2);
  
  Vector2f pos;
  pos.x = place + 0.045;
  pos.y = -0.98;
  sprintf(buf,"%d",nweapon[which]); 
  font->draw_center(buf,pos ,0.1f);

  glBegin(GL_LINE_LOOP);
  glVertex2f(place,-0.87f); 
  glVertex2f(place+.22,-0.87f); 
  glVertex2f(place+.22,-0.99f); 
  glVertex2f(place,-0.99f); 
  glEnd();

  switch(which)
    {
    case 0:
      tex = Graphics::get_texture("Resources/mine.png");
      osize = MINE_SIZE;
    break;
    case 1:
      tex = Graphics::get_texture("Resources/ufo.png");
      osize = UFO_SIZE;
      break;
    case 2:
      tex = Graphics::get_texture("Resources/gunship.png");
      osize = GUNSHIP_SIZE;
      break;
    case 3:
      tex = Graphics::get_texture("Resources/mineship.png");
      osize = MINESHIP_SIZE;
      break;
    case 4:
      tex = Graphics::get_texture("Resources/nuke.png");
      osize = NUKE_SIZE;
      break;
    case 5:
      tex = Graphics::get_texture("Resources/emp.png");
      osize = EMP_SIZE;
      break;
    case 6:
      tex = Graphics::get_texture("Resources/hexagon.png");
      osize = EMP_SIZE;
      break;
    default:
      printf("YOU ARE FUCKED  %d \n",which);

      break;
  }
  if (tex.ref) 
  {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, 
        GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, 
        GL_CLAMP_TO_EDGE);
  }

  glPushMatrix();
  glTranslatef(place+.16,-.93f, 0);
  glBegin(GL_QUADS);
  size = (80/2.0)/Graphics::screen.y;
  glTexCoord2f(1.0f,0.0f);
  glVertex2f(size, -size);
  glTexCoord2f(0.0f,0.0f);
  glVertex2f(-size, -size);
  glTexCoord2f(0.0f,1.0f);
  glVertex2f(-size, size);
  glTexCoord2f(1.0f,1.0f);
  glVertex2f(size, size);
  glEnd();
  glPopMatrix();
  glDisable(GL_TEXTURE_2D);

}

void client::drawFlash(int player){
  if(state.flashScreen[player]){
    bool nuke = state.flashScreen[player] <= 1;
	glBegin(GL_QUADS);
	if(nuke)
    	glColor4f(1.0f, 1.0f, 1.0f, state.flashScreen[player]);
    else
    	glColor4f(0.0f, 1.0f, 1.0f, state.flashScreen[player]-1);
    glVertex2f(1, -1);
    glVertex2f(-1, -1);
    glVertex2f(-1, 1);
    glVertex2f(1, 1);
    glEnd();
  }
}


void client::chat_handle_event(SDL_Event const &event) {
  //Handle events here:
  if (event.type == SDL_MOUSEMOTION) {
    mouse_pos.x = event.motion.x / float(Graphics::screen.x) * 2.0f * Graphics::aspectf() - Graphics::aspectf();
    mouse_pos.y = event.motion.y / float(Graphics::screen.y) *-2.0f + 1.0f;
  }
  if (event.type == SDL_MOUSEBUTTONDOWN) {
    mdown = true;
  }
  if (event.type == SDL_MOUSEBUTTONUP) {
    mdown = false;
    if (screen == 1){
      // games button
      if (mouse_pos.x > 0.6 && mouse_pos.y < -0.9 &&
	  mouse_pos.x < 1.0 && mouse_pos.y > -1.0){
	screen = 2;
	buff.clear();
      }
    }
    else if (screen == 2){
      // join game
      if (mouse_pos.x > -0.6 && mouse_pos.y < -0.5 &&
	  mouse_pos.x < -0.6+1.0/3 && mouse_pos.y > -0.6){
	char buffer[256];
	sprintf(buffer, "%c", gid);
	send_message(MSG_CLIENT_JOIN_GAME, buffer, 1, sock);
	chatlog.clear();
	buff.clear();
	cout << "gid=" << (int)buffer[1] << " len=" << 1 << endl;
      }
      // create game
      if (mouse_pos.x > -0.5+1.0/3 && mouse_pos.y < -0.5 &&
	  mouse_pos.x < -0.5+2.0/3 && mouse_pos.y > -0.6){
	screen = 3;
	buff.clear();
      }
      // cancel
      if (mouse_pos.x > -0.4+2.0/3 && mouse_pos.y < -0.5 &&
	  mouse_pos.x < -0.4+3.0/3 && mouse_pos.y > -0.6){
	screen = 1;
	buff.clear();
      }
      // scroll up and down
      if (mouse_pos.x > 0.60 && mouse_pos.x < 0.65){
	  if (mouse_pos.y > -0.48 && mouse_pos.y < -0.43){
	    if (scrollbar != 0)
	      scrollbar--;
	  }
	  if (mouse_pos.y > 0.60 && mouse_pos.y < 0.65){
	    scrollbar++;
	  }
      }
      // selecting game
      if (mouse_pos.x > -0.6 && mouse_pos.x < 0.6){
	if (mouse_pos.y > -0.45 && mouse_pos.y < 0.6){
	  float temp = fabs(-0.45-mouse_pos.y);
	  selected = int(temp / 0.05) + scrollbar;
	}
      }
    }
    else if (screen == 3){
      // create
      if (mouse_pos.x > -0.4 && mouse_pos.y < -0.05 &&
	  mouse_pos.x < -0.4+1.0/3 && mouse_pos.y > -0.15){
	send_message(MSG_CLIENT_CREATE_GAME, buff.substr(0, 63).c_str(), buff.size(), sock);
	buff.clear();
	chatlog.clear();
	cout << "chatlog better have gotten cleared" << endl;
      }
      // cancel
      if (mouse_pos.x > -0.3+1.0/3 && mouse_pos.y < -0.05 &&
	  mouse_pos.x < -0.3+2.0/3 && mouse_pos.y > -0.15){
	screen = 2;
	buff.clear();
      }
    }
    else if (screen == 4){
      // start game
      if (mouse_pos.x > 0.6 && mouse_pos.y < -0.8  &&
	  mouse_pos.x < 1.0 && mouse_pos.y > -0.9){
	if (playersIG == MAX_PLAYERS) {
	  send_message(MSG_CLIENT_START_GAME, "", 0, sock);
	  buff.clear();
	}
      }
      // quit game
      if (mouse_pos.x > 0.6 && mouse_pos.y < -0.9  &&
	  mouse_pos.x < 1.0 && mouse_pos.y > -1.0){
	send_message(MSG_CLIENT_LEAVE_GAME, "", 0, sock);
	chatlog.clear();
	buff.clear();
	cout << "leaving game" << endl;
      }
    }
    else if (screen == 6 || screen == 7){
      if (mouse_pos.x > -.20 && mouse_pos.y < -0.70 &&
	  mouse_pos.x <  .20 && mouse_pos.y > -0.80){
	screen = 1;
	chatlog.clear();
	buff.clear();
      }
    }
    else if (screen == -1){
      if (mouse_pos.x > -.20 && mouse_pos.y < -0.70 &&
	  mouse_pos.x <  .20 && mouse_pos.y > -0.80){
	//reconnect here
	// not safe, need to check atoi and stuff, will do later.
	sock = Open_clientfd(host, atoi(port));
	if (sock == -1) {
	  cout << "Connect error, so quitting" << endl;
	  exit(1);
	}
	// make it nonblocking
	int opts;
	if ((opts = fcntl(sock, F_GETFL)) < 0)
	  cout << "Error getting options" << endl;
	opts = (opts | O_NONBLOCK);
	if (fcntl(sock, F_SETFL, opts) < 0)
	  cout << "Error setting options" << endl;
	
	// initiate left and right shift to off and last pm to nobody
	rshift = lshift = false;
	lastpm = string("");
	
	// load font for later
	font = Graphics::get_font("courier.txf");
	assert(font.ref);
	
	// scroll bars and screen to 0
	scrollbar = screen = 0;
	
	// initialize rio
	Rio_readinitb(&rio, sock);
	screen = 0;
      }
    }
  }
  if (event.type == SDL_KEYDOWN) {
    switch(event.key.keysym.sym){
    case SDLK_ESCAPE:
      quit_screen(this);
      break;
    case SDLK_RSHIFT:
      rshift = true;
      break;
    case SDLK_LSHIFT:
      lshift = true;
      cout << "left shift down" << endl;
      break;
    case SDLK_a:
      if (rshift || lshift)
	buff += "A";
      else
	buff += "a";
      break;
    case SDLK_b:
      if (rshift || lshift)
	buff += "B";
      else
	buff += "b";
      break;
    case SDLK_c:
      if (rshift || lshift)
	buff += "C";
      else
	buff += "c";
      break;
    case SDLK_d:
      if (rshift || lshift)
	buff += "D";
      else
	buff += "d";
      break;
    case SDLK_e:
      if (rshift || lshift)
	buff += "E";
      else
	buff += "e";
      break;
    case SDLK_f:
      if (rshift || lshift)
	buff += "F";
      else
	buff += "f";
      break;
    case SDLK_g:
      if (rshift || lshift)
	buff += "G";
      else
	buff += "g";
      break;
    case SDLK_h:
      if (rshift || lshift)
	buff += "H";
      else
	buff += "h";
      break;
    case SDLK_i:
      if (rshift || lshift)
	buff += "I";
      else
	buff += "i";
      break;
    case SDLK_j:
      if (rshift || lshift)
	buff += "J";
      else
	buff += "j";
      break;
    case SDLK_k:
      if (rshift || lshift)
	buff += "K";
      else
	buff += "k";
      break;
    case SDLK_l:
      if (rshift || lshift)
	buff += "L";
      else
	buff += "l";
      break;
    case SDLK_m:
      if (rshift || lshift)
	buff += "M";
      else
	buff += "m";
      break;
    case SDLK_n:
      if (rshift || lshift)
	buff += "N";
      else
	buff += "n";
      break;
    case SDLK_o:
      if (rshift || lshift)
	buff += "O";
      else
	buff += "o";
      break;
    case SDLK_p:
      if (rshift || lshift)
	buff += "P";
      else
	buff += "p";
      break;
    case SDLK_q:
      if (rshift || lshift)
	buff += "Q";
      else
	buff += "q";
      break;
    case SDLK_r:
      if (rshift || lshift)
	buff += "R";
      else
	buff += "r";
      break;
    case SDLK_s:
      if (rshift || lshift)
	buff += "S";
      else
	buff += "s";
      break;
    case SDLK_t:
      if (rshift || lshift)
	buff += "T";
      else
	buff += "t";
      break;
    case SDLK_u:
      if (rshift || lshift)
	buff += "U";
      else
	buff += "u";
      break;
    case SDLK_v:
      if (rshift || lshift)
	buff += "V";
      else
	buff += "v";
      break;
    case SDLK_w:
      if (rshift || lshift)
	buff += "W";
      else
	buff += "w";
      break;
    case SDLK_x:
      if (rshift || lshift)
	buff += "X";
      else
	buff += "x";
      break;
    case SDLK_y:
      if (rshift || lshift)
	buff += "Y";
      else
	buff += "y";
      break;
    case SDLK_z:
      if (rshift || lshift)
	buff += "Z";
      else
	buff += "z";
      break;
    case SDLK_1:
      if (rshift || lshift)
	buff += "!";
      else
	buff += "1";
      break;
    case SDLK_2:
      if (rshift || lshift)
	buff += "@";
      else
	buff += "2";
      break;
    case SDLK_3:
      if (rshift || lshift)
	buff += "#";
      else
	buff += "3";
      break;
    case SDLK_4:
      if (rshift || lshift)
	buff += "$";
      else
	buff += "4";
      break;
    case SDLK_5:
      if (rshift || lshift)
	buff += "%";
      else
	buff += "5";
      break;
    case SDLK_6:
      if (rshift || lshift)
	buff += "^";
      else
	buff += "6";
      break;
    case SDLK_7:
      if (rshift || lshift)
	buff += "&";
      else
	buff += "7";
      break;
    case SDLK_8:
      if (rshift || lshift)
	buff += "*";
      else
	buff += "8";
      break;
    case SDLK_9:
      if (rshift || lshift)
	buff += "(";
      else
	buff += "9";
      break;
    case SDLK_0:
      if (rshift || lshift)
	buff += ")";
      else
	buff += "0";
      break;
    case SDLK_MINUS:
      if (rshift || lshift)
	buff += "_";
      else
	buff += "-";
      break;
    case SDLK_EQUALS:
      if (rshift || lshift)
	buff += "+";
      else
	buff += "=";
      break;
    case SDLK_LEFTBRACKET:
      if (rshift || lshift)
	buff += "{";
      else
	buff += "[";
      break;
    case SDLK_RIGHTBRACKET:
      if (rshift || lshift)
	buff += "}";
      else
	buff += "]";
      break;
    case SDLK_BACKSLASH:
      if (rshift || lshift)
	buff += "|";
      else
	buff += "\\";
      break;
    case SDLK_SEMICOLON:
      if (rshift || lshift)
	buff += ":";
      else
	buff += ";";
      break;
    case SDLK_QUOTE:
      if (rshift || lshift)
	buff += "\"";
      else
	buff += "\'";
      break;
    case SDLK_COMMA:
      if (rshift || lshift)
	buff += "<";
      else
	buff += ",";
      break;
    case SDLK_PERIOD:
      if (rshift || lshift)
	buff += ">";
      else
	buff += ".";
      break;
    case SDLK_SLASH:
      if (rshift || lshift)
	buff += "?";
      else
	buff += "/";
      break;
    case SDLK_SPACE:
      buff += " ";
      break;
    case SDLK_RETURN:
    case SDLK_KP_ENTER:
      if (buff.length() != 0){
	if (screen == 0){
	  send_username(buff.substr(0, 64).c_str());
	  buff.clear();
	}
	else if (screen == 1 || screen == 4){
	  buff += '\0';
	  send_chat(buff.c_str());
	  buff.clear();
	}
	else if (screen == 2){
	  //join currently selected game
	  char buffer[256];
	  sprintf(buffer, "%c", gid);
	  send_message(MSG_CLIENT_JOIN_GAME, buffer, 1, sock);
	  cout << "gid=" << (int)buffer[1] << " len=" << 1 << endl;
	}
	else if (screen == 3){
	  send_message(MSG_CLIENT_CREATE_GAME, buff.substr(0, 63).c_str(), buff.size(), sock);
	  chatlog.clear();
	  buff.clear();
	}
	if (screen == -1){
	  //reconnect here
	  // not safe, need to check atoi and stuff, will do later.
	  sock = Open_clientfd(host, atoi(port));
	  if (sock == -1) {
	    cout << "Connect error, so quitting" << endl;
	    exit(1);
	  }
	  // make it nonblocking
	  int opts;
	  if ((opts = fcntl(sock, F_GETFL)) < 0)
	    cout << "Error getting options" << endl;
	  opts = (opts | O_NONBLOCK);
	  if (fcntl(sock, F_SETFL, opts) < 0)
	    cout << "Error setting options" << endl;
	  
	  // initiate left and right shift to off and last pm to nobody
	  rshift = lshift = false;
	  lastpm = string("");
	  
	  // load font for later
	  font = Graphics::get_font("courier.txf");
	  assert(font.ref);
	  
	  // scroll bars and screen to 0
	  scrollbar = screen = 0;
	  
	  // initialize rio
	  Rio_readinitb(&rio, sock);
	  screen = 0;
	}
      }
      break;
    case SDLK_BACKSPACE:
      if (buff.length() != 0)
	buff.erase(buff.end()-1);
      break;
    case SDLK_LEFT:
      
      break;
    case SDLK_RIGHT:

      break;
    case SDLK_DOWN:
      if (screen == 2)
	if (scrollbar!= 0)
	  scrollbar--;
      break;
    case SDLK_UP:
      if (screen == 2)
	  scrollbar++;
      break;
    default:
      //cout << event.key.keysym.sym << endl;
      break;
    }
  }
  if (event.type == SDL_KEYUP){
    switch(event.key.keysym.sym){
    case SDLK_RSHIFT:
      rshift = false;
      break;
    case SDLK_LSHIFT:
      lshift = false;
      break;
    default:
      break;
    }
  }
}
void client::game_handle_event(SDL_Event const &event) {
  //Handle events here:
  if (event.type == SDL_KEYDOWN) {
    if (event.key.keysym.sym == SDLK_RSHIFT)
      rshift = true;
    else if (event.key.keysym.sym == SDLK_LSHIFT)
      lshift = true;
    else if (event.key.keysym.sym == SDLK_ESCAPE) {
      //escape quits.
      quit_screen(this);
    }
    else if(event.key.keysym.sym == SDLK_UP || event.key.keysym.sym == SDLK_w){
      char dir[1];
      dir[0]='U';
      send_message(MSG_GCLIENT_KEYDOWN,dir,1,sock);
    }
    else if(event.key.keysym.sym == SDLK_DOWN || event.key.keysym.sym == SDLK_s){
      char dir[1];
      dir[0]='D';
      send_message(MSG_GCLIENT_KEYDOWN,dir,1,sock);
    }
    else if(event.key.keysym.sym == SDLK_LEFT || event.key.keysym.sym == SDLK_a){
      char dir[1];
      dir[0]='L';
      send_message(MSG_GCLIENT_KEYDOWN,dir,1,sock);
    }
    else if(event.key.keysym.sym == SDLK_RIGHT || event.key.keysym.sym == SDLK_d){
      char dir[1];
      dir[0]='R';
      send_message(MSG_GCLIENT_KEYDOWN,dir,1,sock);
    }
    else if(event.key.keysym.sym == SDLK_SPACE)
    {
      char action[1];
      action[0] = 'S';
      send_message(MSG_GCLIENT_KEYDOWN,action,1,sock);
    }
  }
  else if (event.type == SDL_KEYUP) {
    if (event.key.keysym.sym == SDLK_RSHIFT)
      rshift = false;
    else if (event.key.keysym.sym == SDLK_LSHIFT)
      lshift = false;
    else if(event.key.keysym.sym == SDLK_UP || event.key.keysym.sym == SDLK_w){
      char dir[1];
      dir[0]='U';
      send_message(MSG_GCLIENT_KEYUP,dir,1,sock);
    }
    else if(event.key.keysym.sym == SDLK_DOWN || event.key.keysym.sym == SDLK_s){
      char dir[1];
      dir[0]='D';
      send_message(MSG_GCLIENT_KEYUP,dir,1,sock);
    }
    else if(event.key.keysym.sym == SDLK_LEFT || event.key.keysym.sym == SDLK_a){
      char dir[1];
      dir[0]='L';
      send_message(MSG_GCLIENT_KEYUP,dir,1,sock);
    }
    else if(event.key.keysym.sym == SDLK_RIGHT || event.key.keysym.sym == SDLK_d){
      char dir[1];
      dir[0]='R';
      send_message(MSG_GCLIENT_KEYUP,dir,1,sock);
    }
    else if(event.key.keysym.sym == SDLK_TAB)
    {  
      if (lshift || rshift){
	char dir[2];
	dir[0]='T';
	//cout << weapon << endl;
	if((weapon-1)%7 == -1)
	  dir[1] = 6;
	else
	  dir[1] = (weapon-1)%7;
    
	send_message(MSG_GCLIENT_KEYUP,dir,2,sock);
      }
      else{
	char dir[2];
	dir[0]='T';
	//cout << weapon << endl;
	dir[1]=((weapon+1)%7);
	send_message(MSG_GCLIENT_KEYUP,dir,2,sock);
      }
    }
    else if(event.key.keysym.sym == SDLK_1){
      char dir[2];
      dir[0]='T';
      dir[1]=0;
      send_message(MSG_GCLIENT_KEYUP,dir,2,sock);
    }
    else if(event.key.keysym.sym == SDLK_2){
      char dir[2];
      dir[0]='T';
      dir[1]=1;
      send_message(MSG_GCLIENT_KEYUP,dir,2,sock);
    }
    else if(event.key.keysym.sym == SDLK_3){
      char dir[2];
      dir[0]='T';
      dir[1]=2;
      send_message(MSG_GCLIENT_KEYUP,dir,2,sock);
    }
    else if(event.key.keysym.sym == SDLK_4){
      char dir[2];
      dir[0]='T';
      dir[1]=3;
      send_message(MSG_GCLIENT_KEYUP,dir,2,sock);
    }
    else if(event.key.keysym.sym == SDLK_5){
      char dir[2];
      dir[0]='T';
      dir[1]=4;
      send_message(MSG_GCLIENT_KEYUP,dir,2,sock);
    }
    else if(event.key.keysym.sym == SDLK_6){
      char dir[2];
      dir[0]='T';
      dir[1]=5;
      send_message(MSG_GCLIENT_KEYUP,dir,2,sock);
    }
    else if(event.key.keysym.sym == SDLK_7){
      char dir[2];
      dir[0]='T';
      dir[1]=6;
      send_message(MSG_GCLIENT_KEYUP,dir,2,sock);
    }
    
    else if(event.key.keysym.sym == SDLK_f)
    {  
      char dir[1];
      dir[0]='A';
      send_message(MSG_GCLIENT_KEYUP,dir,1,sock);
    }
  }
}
