#ifndef CLIENT_HPP
#define CLIENT_HPP

#include <Basis/Screen.hpp>
#include <Graphics/Font.hpp>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <iostream>
#include <vector>
#include <string>
#include <iterator>
#include <map>
#include "csapp.h"
#include "gamestate.hpp"
#include "common.hpp"

#define MAXLENGTH 51
#define MAXCHATH 37
#define MAXTYPEH 4

using std::vector;
using std::string;
using std::map;

struct chat{
  string s;
  int red;
  int green;
  int blue;
};

//This is a basic, empty screen. Fill at your leisure.

class client : public Screen {
public:
  virtual void draw();
  virtual void update(float elapsed_time);
  virtual void handle_event(SDL_Event const &event);


  char *host, *port;            // post and port
  bool rshift, lshift;         // booleans that keep track lshift and rshift
  vector <chat> chatlog;       // a vector that stores the chat log
  vector <string> players;     // a vector of all players
  vector <string> ingame_players;
  map <int, string> games;     // a map of all games -- gameID to gameName
  string buff, lastpm, disc;   // buffer stores what we typed
                               // lastpm stores the name of last person to pm us
                               // disc stores disc message
  Graphics::FontRef font;      // stores the font we loaded

  int sock;                    // stores the socket we connected
  rio_t rio;                   // stores the rio number for RIO_INIT
  int screen;                  // a number that indicates which screen to display

  Vector2f mouse_pos;          // position of the mouse
  bool mdown;                  // whether mouse is down
  unsigned int scrollbar;      // scroll bar
  unsigned int selected;       // int which shows what game is selected
  int gid;                     // thing selected
  int playersIG;               // players in your game
  
  client();
  client (char *host, char *port);
  virtual ~client();
  virtual void send_chat(const char *message);
  virtual void send_username(const char *username);
  virtual void receive_chat();
  virtual void receive_gamestate(message_t message);
  virtual void draw_background();      // draw the background
  virtual void draw_login_splash();    // draw the login splash
  virtual void draw_button(float x,    // draw a button with specified
			   float y,    // parameters
			   float xsize, 
			   float ysize, 
			   string text,
			   int activescreen);
  virtual void draw_login();                   // draw login stuff (screen == 0)
  virtual void draw_chat();                    // draw chat stuff (screen == 1)
  virtual void draw_gameswindow();             // draw games stuff (screen == 2)
  virtual void draw_creategame();              // draw create game stuff (screen == 3)
  virtual void draw_lobby();                   // draw lobby (screen == 4)
  virtual void draw_game();                    // draw game (screen == 5)
  virtual void draw_losingscreen();            // draw losing screen (screen == 6)
  virtual void draw_winningscreen();           // draw winning screen (screen == 7)

  gamestate state;
  int player;
  int weapon;
  int nweapon[7];
  
  virtual void drawPortals();
  virtual void drawShip(ship s);
  virtual void drawBullet(bullet b);
  virtual void drawMine(mine m);
  virtual void drawUFO(ufo u);
  virtual void drawGunship(gunship gs);
  virtual void drawMineship(mineship ms);
  virtual void drawNuke(nuke n);
  virtual void drawEMP(emp e);
  virtual void drawHexagon(hexagon h);
  virtual void drawPowerup(powerup p);
  virtual void drawHealth(float x, float y, int health, int player);
  virtual void drawLocal(float x, float y, int which);
  virtual void drawCurrentWeapon(int player, float place, int which);
  virtual void drawFlash(int player);
  virtual void setColor(int player);

  virtual void game_handle_event(SDL_Event const &event) ;
  virtual void chat_handle_event(SDL_Event const &event) ;
};

#endif //CLIENT_HPP
