#include "client_chat.hpp"

#include <Graphics/Graphics.hpp>
#include <Graphics/Texture.hpp>
#include <Graphics/Font.hpp>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <iostream>
#include "constants.hpp"
#include "common.hpp"
#include "csapp.h"

#define MAX(a, b) (((a) > (b)) ? (a) : (b))

using std::cout;
using std::endl;
using std::vector;
using std::string;


char *get_line (char *s, size_t n, FILE *f){
  char *p = fgets (s, n, f);
  if (p != NULL) {
    size_t last = strlen (s) - 1;
    if (s[last] == '\n') 
      s[last] = '\0';
  }
  return p;
}

client_chat::client_chat(char *host, char *port, char *name) {
  // not safe, need to check atoi and stuff, will do later.
  sock = Open_clientfd(host, atoi(port));
  if (sock == -1) {
    cout << "Connect error, so quitting" << endl;
    exit(1);
  }
  int opts;
  if ((opts = fcntl(sock, F_GETFL)) < 0)
    cout << "Error getting options" << endl;
  opts = (opts | O_NONBLOCK);
  if (fcntl(sock, F_SETFL, opts) < 0)
    cout << "Error setting options" << endl;
  
  rshift = lshift = false;
  lastpm = string("");

  // load font for later
  font = Graphics::get_font("courier.txf");
  assert(font.ref);

  send_message(MSG_CLIENT_INIT, name, strlen(name), sock);
}

client_chat::~client_chat(){

}

void client_chat::send_chat(const char *p){
  char buf[256], name[256];
  if (p[0] == '/'){
    if (p[1] == 'w' && p[2] == ' '){
      sscanf(p+3, " %s",name);
      sprintf(buf, "%c%s%s", (char)strlen(name), name, p+4+strlen(name));
      cout << buf << endl;
      send_message(MSG_CLIENT_PM, buf, strlen(buf), sock);
    }
    else if (p[1] == 'r' && p[2] == ' '){
      sprintf(buf, "%c%s%s", (char)(lastpm.length()), lastpm.c_str(), p+3);
      send_message(MSG_CLIENT_PM, buf, strlen(buf), sock);
    }
    else{
      printf("Invalid command.\n");
    }
  }
  else{
    send_message(MSG_CLIENT_CHAT, p, strlen(p), sock);
  }
}

void client_chat::receive_chat(){
  rio_t rio;
  message_t message;
  int name_len;
  char name[256];
  char mid[4] = ": \0";
  char wf[7] = "From \0";
  char wt[5] = "To \0";
  struct chat temp;
  Rio_readinitb(&rio, sock);
  if (read_message(&rio, &message)){
    switch (message.type){
    case MSG_SERVER_INFO:
      message.contents[message.length] = '\0';
      temp.s = string(message.contents);
      temp.red = 255;
      temp.green = 0;
      temp.blue = 0;
      chatlog.push_back(temp);
      break;
    case MSG_SERVER_CHAT:
      message.contents[message.length] = '\0';
      name_len = message.contents[0];
      memcpy(name, message.contents+1, name_len);
      name[name_len] = '\0';
      temp.s = string(name)+string(mid)+string(message.contents+1+name_len);
      temp.red = 135;
      temp.green = 135;
      temp.blue = 135;
      chatlog.push_back(temp);
      break;
    case MSG_SERVER_PM:
      message.contents[message.length] = '\0';
      name_len = message.contents[0];
      memcpy(name, message.contents+1, name_len);
      name[name_len] = '\0';
      lastpm = string(name);
      temp.s = string(wf)+string(name)+string(mid)+string(message.contents+1+name_len);
      temp.red = 0;
      temp.green = 255;
      temp.blue = 0;
      chatlog.push_back(temp);
      break;
    case MSG_SERVER_PM_CONFIRM:
      message.contents[message.length] = '\0';
      name_len = message.contents[0];
      if(name_len == 0){
	temp.s = string(message.contents+1);
	temp.red = 0;
	temp.green = 255;
	temp.blue = 0;
	chatlog.push_back(temp);
      }
      else {
	memcpy(name, message.contents+1, name_len);
	name[name_len] = '\0';
	temp.s = string(wt)+string(name)+string(mid)+string(message.contents+1+name_len);
	temp.red = 0;
	temp.green = 255;
	temp.blue = 0;
	chatlog.push_back(temp);
      }
      break;
    }
  } 
}


void client_chat::draw() {
  glClearColor(0.5f, 0.5f, 0.5f, 0.5f);
  glClear(GL_COLOR_BUFFER_BIT);

  Graphics::TextureRef tex = Graphics::get_texture("Resources/stars.png");
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }

  glBegin(GL_QUADS);
  glColor3f(1.0f, 1.0f, 1.0f);
  glTexCoord2f(0.0f, 0.0f);
  glVertex2f(-1.0f, -1.0f);
  glTexCoord2f(0.0f, 1.0f);
  glVertex2f(-1.0f, 1.0f);
  glTexCoord2f(1.0f, 1.0f);
  glVertex2f(1.0f, 1.0f);
  glTexCoord2f(1.0f, 0.0f);
  glVertex2f(1.0f, -1.0f);
  glEnd();

  tex = Graphics::get_texture("Resources/pipe.png");
  if (tex.ref) {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex->obj);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  }

  glBegin(GL_QUADS);
  glColor3f(1.0, 1.0, 1.0);
  glTexCoord2f(0.0f, 0.0f);
  glVertex2f(0.55, -1.0);
  glTexCoord2f(0.0f, 1.0f);
  glVertex2f(0.55, 1.0);
  glTexCoord2f(1.0f, 1.0f);
  glVertex2f(0.60, 1.0);
  glTexCoord2f(1.0f, 0.0f);
  glVertex2f(0.60, -1.0);
  glEnd();
  glBegin(GL_QUADS);
  glTexCoord2f(0.0f, 0.0f);
  glVertex2f(-1.0, -.85);
  glTexCoord2f(1.0f, 0.0f);
  glVertex2f(1.0, -.85);
  glTexCoord2f(1.0f, 1.0f);
  glVertex2f(1.0, -.90);
  glTexCoord2f(0.0f, 1.0f);
  glVertex2f(-1.0, -.90);  
  glEnd();

  glDisable(GL_TEXTURE_2D);

  int k = buff.length()-50;
  if (k < 0)
    k=0;
  glColor3f(135.0/255, 135.0/255, 135.0/255);
  font->draw(buff.substr(k,MAXLENGTH), make_vector(-1.0f, -0.975f), 0.05f);
 
  vector<chat>::reverse_iterator itr;
  itr = chatlog.rbegin();  
  int lines = 0;
  while(itr != chatlog.rend()){
    if (lines > MAXCHATH)
      break;
    int currline = ((*itr).s.size() / MAXLENGTH)+1;
    for (int i=0; i < currline; i++){
      glColor3f((*itr).red/255.0, (*itr).green/255.0, (*itr).blue/255.0);
      font->draw((*itr).s.substr(MAXLENGTH*i, MAXLENGTH), make_vector(-1.0f, -0.9f + 0.05f*(lines+currline-i)), 0.05f); 
    }
    lines += currline;
    itr++;
  }
}

void client_chat::update(float elapsed_time) {
  receive_chat();
}

void client_chat::handle_event(SDL_Event const &event) {
  //Handle events here:
  if (event.type == SDL_KEYDOWN) {
    switch(event.key.keysym.sym){
    case SDLK_ESCAPE:
      quit_screen(this);
      break;
    case SDLK_RSHIFT:
      rshift = true;
      break;
    case SDLK_LSHIFT:
      lshift = true;
      break;
    case SDLK_a:
      if (rshift || lshift)
	buff += "A";
      else
	buff += "a";
      break;
    case SDLK_b:
      if (rshift || lshift)
	buff += "B";
      else
	buff += "b";
      break;
    case SDLK_c:
      if (rshift || lshift)
	buff += "C";
      else
	buff += "c";
      break;
    case SDLK_d:
      if (rshift || lshift)
	buff += "D";
      else
	buff += "d";
      break;
    case SDLK_e:
      if (rshift || lshift)
	buff += "E";
      else
	buff += "e";
      break;
    case SDLK_f:
      if (rshift || lshift)
	buff += "F";
      else
	buff += "f";
      break;
    case SDLK_g:
      if (rshift || lshift)
	buff += "G";
      else
	buff += "g";
      break;
    case SDLK_h:
      if (rshift || lshift)
	buff += "H";
      else
	buff += "h";
      break;
    case SDLK_i:
      if (rshift || lshift)
	buff += "I";
      else
	buff += "i";
      break;
    case SDLK_j:
      if (rshift || lshift)
	buff += "J";
      else
	buff += "j";
      break;
    case SDLK_k:
      if (rshift || lshift)
	buff += "K";
      else
	buff += "k";
      break;
    case SDLK_l:
      if (rshift || lshift)
	buff += "L";
      else
	buff += "l";
      break;
    case SDLK_m:
      if (rshift || lshift)
	buff += "M";
      else
	buff += "m";
      break;
    case SDLK_n:
      if (rshift || lshift)
	buff += "N";
      else
	buff += "n";
      break;
    case SDLK_o:
      if (rshift || lshift)
	buff += "O";
      else
	buff += "o";
      break;
    case SDLK_p:
      if (rshift || lshift)
	buff += "P";
      else
	buff += "p";
      break;
    case SDLK_q:
      if (rshift || lshift)
	buff += "Q";
      else
	buff += "q";
      break;
    case SDLK_r:
      if (rshift || lshift)
	buff += "R";
      else
	buff += "r";
      break;
    case SDLK_s:
      if (rshift || lshift)
	buff += "S";
      else
	buff += "s";
      break;
    case SDLK_t:
      if (rshift || lshift)
	buff += "T";
      else
	buff += "t";
      break;
    case SDLK_u:
      if (rshift || lshift)
	buff += "U";
      else
	buff += "u";
      break;
    case SDLK_v:
      if (rshift || lshift)
	buff += "V";
      else
	buff += "v";
      break;
    case SDLK_w:
      if (rshift || lshift)
	buff += "W";
      else
	buff += "w";
      break;
    case SDLK_x:
      if (rshift || lshift)
	buff += "X";
      else
	buff += "x";
      break;
    case SDLK_y:
      if (rshift || lshift)
	buff += "Y";
      else
	buff += "y";
      break;
    case SDLK_z:
      if (rshift || lshift)
	buff += "Z";
      else
	buff += "z";
      break;
    case SDLK_1:
      if (rshift || lshift)
	buff += "!";
      else
	buff += "1";
      break;
    case SDLK_2:
      if (rshift || lshift)
	buff += "@";
      else
	buff += "2";
      break;
    case SDLK_3:
      if (rshift || lshift)
	buff += "#";
      else
	buff += "3";
      break;
    case SDLK_4:
      if (rshift || lshift)
	buff += "$";
      else
	buff += "4";
      break;
    case SDLK_5:
      if (rshift || lshift)
	buff += "%";
      else
	buff += "5";
      break;
    case SDLK_6:
      if (rshift || lshift)
	buff += "^";
      else
	buff += "6";
      break;
    case SDLK_7:
      if (rshift || lshift)
	buff += "&";
      else
	buff += "7";
      break;
    case SDLK_8:
      if (rshift || lshift)
	buff += "*";
      else
	buff += "8";
      break;
    case SDLK_9:
      if (rshift || lshift)
	buff += "(";
      else
	buff += "9";
      break;
    case SDLK_0:
      if (rshift || lshift)
	buff += ")";
      else
	buff += "0";
      break;
    case SDLK_MINUS:
      if (rshift || lshift)
	buff += "_";
      else
	buff += "-";
      break;
    case SDLK_EQUALS:
      if (rshift || lshift)
	buff += "+";
      else
	buff += "=";
      break;
    case SDLK_LEFTBRACKET:
      if (rshift || lshift)
	buff += "{";
      else
	buff += "[";
      break;
    case SDLK_RIGHTBRACKET:
      if (rshift || lshift)
	buff += "}";
      else
	buff += "]";
      break;
    case SDLK_BACKSLASH:
      if (rshift || lshift)
	buff += "|";
      else
	buff += "\\";
      break;
    case SDLK_SEMICOLON:
      if (rshift || lshift)
	buff += ";";
      else
	buff += ":";
      break;
    case SDLK_QUOTE:
      if (rshift || lshift)
	buff += "\"";
      else
	buff += "\'";
      break;
    case SDLK_COMMA:
      if (rshift || lshift)
	buff += "<";
      else
	buff += ",";
      break;
    case SDLK_PERIOD:
      if (rshift || lshift)
	buff += ">";
      else
	buff += ".";
      break;
    case SDLK_SLASH:
      if (rshift || lshift)
	buff += "?";
      else
	buff += "/";
      break;
    case SDLK_SPACE:
      buff += " ";
      break;
    case SDLK_RETURN:
      if (buff.length() != 0){
	std::cout<<buff<<std::endl;
	send_chat(buff.c_str());
	buff.clear();
      }
      break;
    case SDLK_BACKSPACE:
      if (buff.length() != 0)
	buff.erase(buff.end()-1);
      break;
    default:
      break;
    }
  }
  if (event.type == SDL_KEYUP){
    switch(event.key.keysym.sym){
    case SDLK_RSHIFT:
      rshift = false;
      break;
    case SDLK_LSHIFT:
      lshift = false;
      break;
    default:
      break;
    }
  }
}
