#ifndef CONSTANTS_HPP
#define CONSTANTS_HPP

#define CLOCKWISE 0
#define COUNTERCLOCKWISE 1
#define KEYPRESS_UP   0
#define KEYPRESS_DOWN 1
#define NAMELEN 64
#define ACCELERATION 0.25
#define ROTATION_RATE 135
#define UPDATE_RATE 1/30.0f
#define MAX_SHIP_VELOCITY 0.8f
#define PORTAL_RADIUS 0.25f
#define NUMBER_OF_PORTALS 5 
#define MAX_PLAYERS 6
#define BULLET_SPEED 0.25
#define BULLET_EFFECT 0.2
#define NUKE_SPEED 0.25
#define NUKE_DAMAGE 25
#define NUKE_LIFE 10
#define NUKE_DECELERATION .01
#define MINE_SPEED 0.25
#define MINE_DAMAGE 10
#define MINES_SPAWNED 4
#define MINE_LIFE 5
#define UFO_SPEED 0.25
#define UFO_DAMAGE 20
#define UFO_LIFE 1
#define MINESHIP_SPEED 0.25
#define MINESHIP_TIME 4
#define MINESHIP_LIFE 15
#define GUNSHIP_SPEED 0.25
#define GUNSHIP_SHOT_TIME 4
#define GUNSHIP_LIFE 20
#define GUNSHIP_ATTACK 10
#define EMP_SPEED 0.25
#define EMP_TIME 10
#define EMP_LIFE 10
#define EMP_DECELERATION .01
#define HEXAGON_SPEED 0.25
#define COLLISION_DAMAGE 15
#define PI 3.141592654
#define PORTAL_MAX_HITPOINTS 5
#define NO_FLASH 0
#define NUKE_FLASH 1
#define EMP_FLASH 2
#define POWERUP_SPAWN_CHANCE (0.8f)
#define MINE_SPAWN_CHANCE (0.2f)
#define MAX_POWER 10
#define MAX_ARMOR 10
#define MAX_SPEED 10
#define LOCAL_POWERUP_SPEED 0.25

//#define DEBUG
#ifdef DEBUG
  #undef PORTAL_MAX_HITPOINTS
  #define PORTAL_MAX_HITPOINTS 1
  #undef MAX_PLAYERS
  #define MAX_PLAYERS 2
#endif

enum message_e {
  MSG_CLIENT_INIT,            //for now, the client just sends a username
  MSG_CLIENT_CHAT,            //message to everyone
  MSG_CLIENT_PM,              //message to a specific player
  MSG_CLIENT_CREATE_GAME,     //client wants to create a new game
  MSG_CLIENT_JOIN_GAME,       //client wants to join an existing game
  MSG_CLIENT_LEAVE_GAME,      //client wants to leave the game he's currently in
  MSG_CLIENT_START_GAME,

  MSG_SERVER_INFO,            //info that the client should display to the user
  MSG_SERVER_CHAT,            //message to everyone
  MSG_SERVER_PM,              //message to a specific player
  MSG_SERVER_PM_CONFIRM,      //sent back to sending player to confirm that pm was redirected by the server
  MSG_SERVER_CREATE_GAME,     //tells the client the id of the game he just created
  MSG_SERVER_JOIN_GAME,       //tells the client the id of the game he just joined or indicates if the client failed to join
  MSG_SERVER_LEAVE_GAME,
  MSG_SERVER_UPDATE_GAME,     //is sent when players join or leave a game or when a game dies
  MSG_SERVER_GAME_PLAYERS,
  MSG_SERVER_PLAYER_STATUS,
  MSG_SERVER_DISCONNECT,

  /* Client to game server messages */
  MSG_MOVE_FORWARD,    //I want to move forward
  MSG_ROTATE_LEFT,     //I want to rotate left.
  MSG_ROTATE_RIGHT,    //I want to rotate right.
  MSG_ROTATE_WEAPON,   //I want to rotate my weapon.
  MSG_FIRE_WEAPON,     //I want to fire my weapon.
  
  /* Game server to client messages */
  MSG_GSERVER_INIT,     //Game server in initing you.
  MSG_GSERVER_DESTROY_OBJ, //Destroy this object.
  MSG_GSERVER_SHIP_YOU, //Message for a ship object.
  MSG_GSERVER_SHIP_OPP, //Message for a ship object.
  MSG_GSERVER_BULLET,   //Message for a bullet object.
  MSG_GSERVER_MINE,     //Message for a mine object.
  MSG_GSERVER_UFO,      //Message for a UFO object.
  MSG_GSERVER_GUNSHIP,  //Message for a Gunship object.
  MSG_GSERVER_MINESHIP, //Message for a mineship object.
  MSG_GSERVER_NUKE,     //Message for a nuke object.
  MSG_GSERVER_EMP,      //Message for a EMP object.
  MSG_GSERVER_HEXAGON,  //Message for a Hexagon object.
  MSG_GSERVER_ROTATION, //Message for a rotation value.
  MSG_PORTAL_ROTATION,  //Message to update the portals rotation value.

  MSG_GCLIENT_QUIT,
  MSG_GCLIENT_KEYDOWN,	//Message for a key press on a client
  MSG_GCLIENT_KEYUP		//Message for a key release on a client
} messages;

#endif //CONSTANTS_HPP
