#include <vector>
#ifndef GAMESTATE_HPP
#define GAMESTATE_HPP

#define DEFAULT_ATTACK 5
#define DEFAULT_ARMOR  0
#define DEFAULT_SPEED  1 
#define DEFAULT_LIFE   100
#define DEFAULT_POSITION_X -0.5f
#define DEFAULT_POSITION_Y  0.0f
#define DEFAULT_ORIENTATION 00.0f
#define DEFAULT_ROTATION 5.0f
#define SHIP_SIZE 100
#define SHIP_RADIUS (float)SHIP_SIZE/Graphics::screen.y/2.0
#define BULLET_SIZE 10
#define MINE_SIZE 40 
#define UFO_SIZE 100
#define GUNSHIP_SIZE 100
#define MINESHIP_SIZE 100
#define NUKE_SIZE 100
#define EMP_SIZE 100
#define MAX_HEALTH 100
#define PORTAL_SIZE 200
#define POWERUP_SIZE 75
#define POWERUP_HEALTH 0
#define HEALTH_INCREASE 25
#define POWERUP_POWER 1
#define POWER_INCREASE 1
#define POWERUP_ARMOR 2
#define ARMOR_INCREASE 1
#define POWERUP_SPEED 3
#define SPEED_INCREASE 1
#define SPEED_DAMPENER 5.0
#define ARMOR_PERCENT 0.05

//Position is the center of the object
struct ship{
  int attack;                    // attack of the ship
  int armor;                     // defense of the ship
  int speed;                     // speed of the ship
  int life;                      // the amount of life remaining
  float position[2];             // position of the ship
  float velocity[2];             // position of the ship
  volatile float orientation;    // orientation of the ship
  int player;                    // Player number of the ship.
  int curr_weapon;               // Current weapon of the player
  int num_weapons[7];
};

struct bullet{
  int id;             // id number of the object
  int attack;         // the strength of the bullet
  float position[2];  // the position of the bullet
  float orientation;  // the orientation of the bullet
  float speed;
  int player;           // the player who shot the bullet
  int screen;         // player whose screen this is on.
  float distance;     // The distance traveled by the bullet
  bool active;		  //Is the AI active
};

struct mine{
  int id;               // id number of the object
  float position[2]; // where the mine is
  int player;        // which player sent it
  int screen;        //player whose screen this is on.
  bool active;		 //Is the AI active
  float orientation;  // the orientation of the mine while moving
  float distance;	//distance the mine floats before stopping
  int life;
};

struct ufo{
  int id;            // id number of the object
  float position[2]; // where the UFO is
  float orientation; // orientation of the UFO
  int player;        // which player sent it
  int screen;        // player whose screen this is on.
  bool active;		   //Is the AI active
  int life;          //hitpoints of the UFO 
};

struct gunship{
  int id;               // id number of the object
  float position[2];       // position of the ship
  float orientation;  	   // orientation of the ship
  int player;              // which player sent it
  int screen;              //player whose screen this is on
  bool active;		 //Is the AI active
  float shotTimer;		//timer till next shot
  int life;
};

struct mineship{
  int id;               // id number of the object
  float position[2];     // position of the ship
  float orientation;     // orientation of the ship
  float mineCD;            // amount of time left until next mine drop
  int player;            // which player sent it
  int screen;              //player whose screen this is on
  bool active;		 //Is the AI active
  int life;
};

struct nuke{
  int id;               // id number of the object
  float position[2];  // position of the nuke
  int timer;          // amount of time remaining on timer
  int player;         // which player sent it
  float orientation;  // orientation of the nuke
  int screen;              //player whose screen this is on
  bool active;		 //Is the AI active
  int life;
  float speed;
};

struct emp{
  int id;               // id number of the object
  float position[2];  // position of the EMP
  int timer;          // amount of time remaining on timer
  int player;         // which player sent it
  float orientation;  // orientation of the EMP
  int screen;              //player whose screen this is on
  bool active;		 //Is the AI active
  int life;
  float speed;
};

struct hexagon{
  int id;               // id number of the object
  float position[2];  // position of the hexagon
  int size;           // how large the hexagon has grown
  int player;         // which player sent it
  float orientation;  // orientation of the hexagon
  int screen;              //player whose screen this is on
  bool active;		 //Is the AI active
};

struct powerup{
  int id;             // id number of the object
  float position[2];  // position of the powerup
  int screen;         //player whose screen this is on
  int type;			  //0=health, 1=power, 2=armor, 3=speed
  float orientation;
};

/**@brief A structure to tell a client to stop tracking an object.
  *
  * id The object id of the object to stop tracking.
  * curr_screen The screen this object should now be on. -1 if dead.
  * prev_screen The old screen that this object should be on.
  */
struct removeItem{
  int id;
  int curr_screen;
};

struct gamestate{
  std::vector <ship> lifeList;          // list of the life percentages for each player.
  std::vector <bullet> bulletList;      // list of all bullets on the screen
  std::vector <mine> mineList;          // list of all mines on the screen
  std::vector <ufo> UFOList;            // list of all UFOs on the screen
  std::vector <gunship> GunshipList;    // list of all Gunships on the screen
  std::vector <mineship> MineshipList;  // list of all Mineships on the screen
  std::vector <nuke> nukeList;          // list of all nukes on the screen
  std::vector <emp> empList;            // list of all emps on the screen
  std::vector <hexagon> hexagonList;    // list of all hexagons on the screen
  std::vector <powerup> powerupList;
  std::vector <removeItem> removeList;      // List of items to remove.
  float portalRotation;		              // Stores the rotation value of the portal.
  unsigned int itemnumber;              // Counter for items in the game.
  float flashScreen[6];					//Flash the client screen. 0=none, 1=nuke, 2=emp
};

#endif //GAMESTATE_HPP
