#include <Basis/Screen.hpp>
#include "client.hpp"
#include <Graphics/Graphics.hpp>
#include <SDL.h>
#include <SDL_opengl.h>
#include <Basis/Log.hpp>

int main(int argc, char **argv) {
  if (argc != 3) {
    printf("Please pass hostname and port\n");
    exit(0);
  }
  if (SDL_Init(SDL_INIT_TIMER) != 0) {
    LOG_ERROR("Could not initialize sdl: " << SDL_GetError());
    return 1;
  }
  
  //ask for a specific screen size:
  Graphics::screen.x = 700;
  Graphics::screen.y = 700;
  Graphics::fullscreen = false;
  
  if (!Graphics::init()) {
    LOG_ERROR("Could not initialize graphics, not continuing.");
    exit(1);
  }
  
  //Some basic GL state:
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glScalef(1.0f / Graphics::aspectf(), 1.0f, 1.0f);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glEnable(GL_LINE_SMOOTH);
  
  //we'll draw our own cursor, thank you:
  //SDL_ShowCursor(SDL_DISABLE);
  //Uncomment to grab cursor. Can be annoying when debugging:
  //SDL_WM_GrabInput(SDL_GRAB_ON);
  
  //Uncomment to get character info along with keypresses
  // (useful for 'type in your name' and so on)
  //SDL_EnableUNICODE(1);
  
  {
    client play(argv[1], argv[2]);
    //This will return when someone does quit_screen(play):
    start_screen(&play);
  }
  
  Graphics::deinit();
  
  SDL_Quit();
  
  return 0;
}
