/*
 * main_server.cpp
 *
 *  Created on: Nov 2, 2009
 *      Author: dportnoy
 */

#include "server.hpp"

#include <time.h>
#include <stdlib.h>
#include <Graphics/Graphics.hpp>

using namespace std;

void kick_player(string msg, player_t *player) {
  char buf[MAXLINE];
  list<player_t *> *players_l;
  
  buf[0] = 0;
  sprintf(buf+1, "%s", player->name);
  
  players_l = players.get_writelock();
  
  players_l->remove(player);
  player_map.erase(player->name);

  list<player_t *>::iterator iter;
  for(iter = players_l->begin(); iter != players_l->end(); iter++) {
    cout << "sending exit message about " << player->name << " to " << (*iter)->name << endl;
    send_message(MSG_SERVER_PLAYER_STATUS, buf, strlen(buf+1)+1, (*iter)->fd);
  }
  
  players.release_lock();

  kick_player_from_game(player);

  send_disconnect_message(msg, player);

  Close(player->fd);
}

int kick_player_from_game(player_t *player) {
  char buf[MAXLINE];
  list<game_t *> *games_l;
  list<player_t *> *players_l;

  if(player->game == NULL)
    return -1;

  int id = player->game->id;

  list<player_t *>::iterator iter;
  players_l = player->game->players.get_writelock();
  for(iter = players_l->begin(); iter != players_l->end(); iter++) {
    if((*iter)->fd == player->fd ) {
      players_l->erase(iter);
      buf[0] = player->game->id;
      buf[1] = players_l->size();
      break;
    }
  }

  int size;	//trying to prevent nested locks

  size = players_l->size();
  player->game->players.release_lock();

  cout << "player is removed from the list of players. # of players left: " << size << endl;

  players_l = players.get_readlock();
  for(iter = players_l->begin(); iter != players_l->end(); iter++) {
    send_message(MSG_SERVER_UPDATE_GAME, buf, 2, (*iter)->fd);
  }
  players.release_lock();

  //remove game if there are no players in it
  if(size == 0) {
    list<game_t *>::iterator game_it;
    games_l = games.get_writelock();
    for(game_it = games_l->begin(); game_it != games_l->end(); game_it++) {
      if((*game_it)->id == player->game->id) {
        games_l->erase(game_it);
        cout << "empty game just got deleted" << endl;
        break;
      }
    }
    games.release_lock();
  }

  game_t *game = player->game;
  player->game = NULL;

  players_l = game->players.get_readlock();
  if(game->started) {
    pthread_rwlock_wrlock(&game->playerl_lock);

    game->numplayers--;
    player->s->life = 0;

    game->players_cache[player->id] = *player;
    game->players_arr[player->id] = &game->players_cache[player->id];
    player = game->players_arr[player->id];
    player->ingame = false;

    pthread_rwlock_unlock(&game->playerl_lock);
  }else
    player->game = NULL;
  game->players.release_lock();

  return id;
}

void send_disconnect_message(string msg, player_t *player) {
  send_message(MSG_SERVER_DISCONNECT, msg.c_str(), msg.size(), player->fd);
}

void *thread(void *vargp) {
  player_t player = *(player_t *)vargp;
  rio_t rio;
  char buf[MAXLINE];
  message_t message;
  list<game_t *> *games_l;
  list<player_t *> *players_l;
  list<player_t *>::iterator iter;
  list<game_t *>::iterator game_it;

  Rio_readinitb(&rio, player.fd);
  
  player.ingame = false;
  player.game = NULL;

  if(read_message(&rio, &message)) {
    if(message.type == MSG_CLIENT_INIT) {
      message.contents[message.length] = '\0';
      sprintf(player.name, "%s", message.contents);
      
      players_l = players.get_writelock();
      if(player_map.find(player.name) == player_map.end()) {
        players_l->push_back(&player);
        player_map.insert( pair<string, player_t *>(player.name, &player) );
        sprintf(buf, "There are currently %d players connected! Also, your name is %s.", (int)players_l->size(), player.name);
        players.release_lock();

        send_message(MSG_SERVER_INFO, buf, strlen(buf), player.fd);
      }else {
        players.release_lock();
        send_disconnect_message("Your username is already taken", &player);
        return (void *)1;
      }
    }else {
      send_disconnect_message("The server received an invalid message from you", &player);
      return (void *)1;
    }
  }else {
    send_disconnect_message("The server did not receive a handshake from you", &player);
    return (void *)1;
  }
  
  printf("Got past the handshake...\n");

  //send info about existing players to the player
  buf[0] = 1;
  players_l = players.get_readlock();
  for(iter = players_l->begin(); iter != players_l->end(); iter++) {
    sprintf(buf+1, "%s", (*iter)->name);
    cout << "sending player status about " << buf+1 << " to " << player.name << endl;
    send_message(MSG_SERVER_PLAYER_STATUS, buf, strlen(buf), player.fd);

    if(*iter != &player) {
      sprintf(buf+1, "%s", player.name);
      cout << "sending player status about " << player.name << " to " << (*iter)->name << endl;
      send_message(MSG_SERVER_PLAYER_STATUS, buf, strlen(buf), (*iter)->fd);
    }	
  }
  players.release_lock();

  //send info about existing games to the player
  games_l = games.get_readlock();
  for(game_it = games_l->begin(); game_it != games_l->end(); game_it++) {
    if((*game_it)->started)
      continue;

    buf[0] = (*game_it)->id;
    strcpy(buf+1, (*game_it)->name);
    send_message(MSG_SERVER_CREATE_GAME, buf, strlen(buf+1)+1, player.fd);
    cout << "sent game creation (" << (*game_it)->id <<") to " << player.name << endl;

    players_l = (*game_it)->players.get_readlock();
    buf[1] = players_l->size();
    (*game_it)->players.release_lock();
    send_message(MSG_SERVER_UPDATE_GAME, buf, 2, player.fd);
    cout << "sent game size (" << players_l->size() << ") of (" << (*game_it)->id <<") to " << player.name << endl;
  }
  games.release_lock();
  
  game_t *game = NULL;
  int name_len;
  char name[NAMELEN];
  bool found;
  pthread_t tid;
  while(read_message(&rio, &message)) {
    switch(message.type) {
    case MSG_CLIENT_CHAT:
      message.contents[message.length] = '\0';
      buf[0] = strlen(player.name);
      sprintf(buf+1, "%s%s", player.name, message.contents);
     
      if(player.game != NULL) {
        cout << "start send" << endl;
        players_l = game->players.get_readlock();
        cout << "got the list" << endl;
        for(iter = players_l->begin(); iter != players_l->end(); iter++) {
          if((*iter)->ingame || !(*iter)->game->started)
            send_message(MSG_SERVER_CHAT, buf, strlen(buf), (*iter)->fd);
        }
        game->players.release_lock();
        cout << "end send" << endl;
      }else {
        cout << "start send 2" << endl;
        players_l = players.get_readlock();
        for(iter = players_l->begin(); iter != players_l->end(); iter++) {
          if((*iter)->game == NULL)
            send_message(MSG_SERVER_CHAT, buf, strlen(buf), (*iter)->fd);
        }
        players.release_lock();
        cout << "end send 2" << endl;
      }
      
      break;
    case MSG_CLIENT_PM:
      message.contents[message.length] = '\0';
      name_len = message.contents[0];
      
      memcpy(name, message.contents+1, name_len);
      name[name_len] = '\0';
      
      players_l = players.get_readlock();	//this should actually only be a lock on the map, but it works for now (in a hackish way)
      if(player_map.find(name) == player_map.end()) {
        sprintf(buf, "Player %s does not exist", name);	
        send_message(MSG_SERVER_INFO, buf, strlen(buf), player.fd);
      }else {
        buf[0] = name_len;
        sprintf(buf+1, "%s%s", name, message.contents+1+name_len);
        send_message(MSG_SERVER_PM_CONFIRM, buf, strlen(buf), player.fd);

        buf[0] = strlen(player.name);
        sprintf(buf+1, "%s%s", player.name, message.contents+1+name_len);
        send_message(MSG_SERVER_PM, buf, strlen(buf), player_map[name]->fd);
      }
      players.release_lock();
      break;
    case MSG_CLIENT_CREATE_GAME:
      game = new game_t;
      game->started = false;
      games_l = games.get_readlock();
      if(games_l->size() == 0)
        game->id = 0;
      else
        game->id = games_l->back()->id+1;
      games.release_lock();

      players_l = game->players.get_writelock();
      players_l->push_back(&player);
      game->players.release_lock();

      message.contents[message.length] = '\0';
      strcpy(game->name, message.contents);

      games_l = games.get_writelock();
      games_l->push_back(game);
      games.release_lock();

      player.game = game;

      buf[0] = game->id;
      strcpy(buf+1, game->name);
      players_l = players.get_readlock();
      for(iter = players_l->begin(); iter != players_l->end(); iter++) {
	  send_message(MSG_SERVER_CREATE_GAME, buf, strlen(buf+1)+1, (*iter)->fd);
      }
      send_message(MSG_SERVER_JOIN_GAME, buf, 1, player.fd);
      buf[1] = 1;
      //should not be necessary once the client keeps track of all game populations
      for(iter = players_l->begin(); iter != players_l->end(); iter++) {
	send_message(MSG_SERVER_UPDATE_GAME, buf, 2, (*iter)->fd);
      }
      players.release_lock();
      buf[0] = 1;
      buf[1] = strlen(player.name);
      sprintf(buf+2, "%s", player.name);
      send_message(MSG_SERVER_GAME_PLAYERS, buf, 2+buf[1], player.fd);
      cout << "game " << game->id << " fully created" << endl;
      break;
    case MSG_CLIENT_JOIN_GAME:
      found = false;

      games_l = games.get_readlock();
      for(game_it = games_l->begin(); game_it != games_l->end(); game_it++) {
	if((*game_it)->id == message.contents[0] && !(*game_it)->started) {
	  players_l = (*game_it)->players.get_writelock();
	  if(players_l->size() < MAX_PLAYERS) {
            players_l->push_back(&player);
            player.game = game = *game_it;
	    found = true;
	  }else
	    cout << "Game is full" << endl;
	  (*game_it)->players.release_lock();
	  break;
	}
      }
      games.release_lock();

      if(!found) {
        cout << "You got rejected from the game you tried to join. (id=" << (int)message.contents[0] << ")" << endl;
        buf[0] = -1;	//tried to join an invalid or full game. maybe want to send different values depending on the case
        send_message(MSG_SERVER_JOIN_GAME, buf, 1, player.fd);
      }else {
        players_l = player.game->players.get_readlock();

        buf[0] = players_l->size();
        int x=1;
        for(iter = players_l->begin(); iter != players_l->end(); iter++) {
          buf[x] = strlen((*iter)->name);
          sprintf(buf+x+1, "%s", (*iter)->name);
          x += buf[x]+1;
        }
        for(iter = players_l->begin(); iter != players_l->end(); iter++) {
          send_message(MSG_SERVER_GAME_PLAYERS, buf, x, (*iter)->fd);
        }

        buf[0] = message.contents[0];	//send back game id for confirmation
        buf[1] = players_l->size();

        (*game_it)->players.release_lock();

        send_message(MSG_SERVER_JOIN_GAME, buf, 1, player.fd);

        players_l = players.get_readlock();
        for(iter = players_l->begin(); iter != players_l->end(); iter++) {
          send_message(MSG_SERVER_UPDATE_GAME, buf, 2, (*iter)->fd);
        }
        players.release_lock();
      }
      break;
    case MSG_CLIENT_LEAVE_GAME:
      cout << "Got MSG_CLIENT_LEAVE_GAME" << endl;
      buf[0] = kick_player_from_game(&player);
      send_message(MSG_SERVER_LEAVE_GAME, buf, 1, player.fd);
      break;
    case MSG_CLIENT_START_GAME:
      cout << "Received command to start a game." << endl;
      buf[0] = player.game->id;
      buf[1] = 0;
      players_l = players.get_readlock();
      for(iter = players_l->begin(); iter != players_l->end(); iter++) {
        send_message(MSG_SERVER_UPDATE_GAME, buf, 2, (*iter)->fd);
      }
      players.release_lock();
      Pthread_create(&tid, NULL, game_thread, (void *)player.game);
      break;
    case MSG_GCLIENT_KEYDOWN:
      update_player(&player, message.contents, MSG_GCLIENT_KEYDOWN);
      break;
    case MSG_GCLIENT_KEYUP: 
      update_player(&player, message.contents, MSG_GCLIENT_KEYUP);
      break;
    default:
      sprintf(buf, "server received an invalid message of type %d", message.type);
      kick_player(buf, &player);
      return (void *)1;
    }
    message.length = 0;
  }
  
  kick_player("client closed the connection", &player);
  return (void *)0;
}

void *game_thread(void *vargp) {
  game_t *game = (game_t *)vargp;
  list<player_t *> *players_l;

  game->globalstate.lifeList.reserve(6);
  game->globalstate.bulletList.reserve(100);
  game->globalstate.mineList.reserve(20);
  game->globalstate.UFOList.reserve(20);
  game->globalstate.GunshipList.reserve(20);
  game->globalstate.MineshipList.reserve(20);
  game->globalstate.nukeList.reserve(20);
  game->globalstate.empList.reserve(20);
  game->globalstate.hexagonList.reserve(20);
  game->globalstate.powerupList.reserve(20);
  game->globalstate.removeList.reserve(20);

  pthread_rwlock_init(&game->bllock,NULL);
  pthread_rwlock_init(&game->lllock,NULL);
  pthread_rwlock_init(&game->itemlock,NULL);
  pthread_rwlock_init(&game->rmlock,NULL);
  pthread_rwlock_init(&game->nlock,NULL);
  pthread_rwlock_init(&game->mlock,NULL);
  pthread_rwlock_init(&game->ulock,NULL);
  pthread_rwlock_init(&game->glock,NULL);
  pthread_rwlock_init(&game->elock,NULL);
  pthread_rwlock_init(&game->hlock,NULL);
  pthread_rwlock_init(&game->plock,NULL);
  pthread_rwlock_init(&game->mslock,NULL);
  pthread_rwlock_init(&game->playerl_lock,NULL);

  game->rotation_counter = 0.0f;
  game->globalstate.portalRotation = 0.0f;
  game->portal_direction = CLOCKWISE;
  
  for(int i = 0; i < 6; i++)
  	game->globalstate.flashScreen[i]=NO_FLASH;

  cout << "Started the game thread and inited some stuff" << endl;

  // BUILD PLAYER ARRAY
  players_l = game->players.get_readlock();

  list<player_t *>::iterator it;
  int x=0;
  for(it = players_l->begin(); it != players_l->end(); it++) {
    game->players_arr[x] = *it;
    (*it)->ingame = true;
    x++;
  }

  game->numplayers = MAX_PLAYERS;

  game->started = true;

  game->players.release_lock();
  // END BUILD

  cout << "Finished building the array" << endl;


  game->num_bullets = 0;

  int i;
  char buf[MAXLINE];
  game->globalstate.itemnumber = 0;

  //Gets rio information for each player.
  for(int x=0; x<MAX_PLAYERS; x++) 
  {
    player_initialize(x, game->players_arr[x], game);
  }

  //Sends the initialization message to each player.
  for(int x=0; x<MAX_PLAYERS; x++) 
  {
    buf[0] = x;
    cout << "sending MSG_GSERVER_INIT" << endl;
    send_message(MSG_GSERVER_INIT, buf, 1, game->players_arr[x]->fd);
  }

  //Send the initial game state.
  for(i=0;i<MAX_PLAYERS; i++)
  {
    send_game_state(i, game->players_arr, game);
  }

  cout << "starting the while loop" << endl;

  //Now, we sleep for a while and then send a message.
  player_t *dead_players[MAX_PLAYERS];
  int players_removed;
  bool gameover;
  while(1) {
    players_removed = 0;
    gameover = false;
    usleep(25000);
    pthread_rwlock_wrlock(&game->playerl_lock);
    if(game->numplayers == 0)
      break;

    for(int i=0; i<MAX_PLAYERS; i++) {
      if(game->players_arr[i]->ingame)
        update_state(game->players_arr[i], 0.025);
      if(game->players_arr[i]->s->life < 0)
        game->players_arr[i]->s->life = 0;
      if(game->players_arr[i]->s->life == 0 && game->players_arr[i]->ingame) {
        dead_players[i] = game->players_arr[i];
        players_removed++;
      }else
        dead_players[i] = NULL;
    }
    if(players_removed+1 == game->numplayers) {
      gameover = true;
      for(int i=0; i<MAX_PLAYERS; i++) {
        if(game->players_arr[i]->ingame)
          dead_players[i] = game->players_arr[i];
      }
    }
    update_object_states(0.05, game);
    for(int i=0; i<MAX_PLAYERS; i++) {
      if(game->players_arr[i]->ingame)
        send_game_state(game->players_arr[i]->id, game->players_arr, game);
    }
    pthread_rwlock_unlock(&game->playerl_lock);
    for(int i=0; i<MAX_PLAYERS; i++) {
      if(dead_players[i] != NULL && (dead_players[i]->s->life == 0 || gameover)) {
        cout << "Kicked player " << i << endl;
        kick_player_from_game(dead_players[i]);
      }
    }
    
    pthread_rwlock_wrlock(&game->rmlock);
    game->globalstate.removeList.clear();
    pthread_rwlock_unlock(&game->rmlock);

    rotate_portal(game);
  }

  cout << "Closing game thread" << endl;
  delete game;

  return (void *)0;
}

int player_initialize(int num, player_t *player, game_t *game) {
  player->s = ship_create(num, 0, game);
  player->U  = MSG_GCLIENT_KEYUP; 
  player->D  = MSG_GCLIENT_KEYUP; 
  player->R  = MSG_GCLIENT_KEYUP; 
  player->L  = MSG_GCLIENT_KEYUP; 
  player->id = num;
  game->globalstate.itemnumber ++;
  player->portal_hitpoints = PORTAL_MAX_HITPOINTS;
  player->reverseTimer = 0;
  return 0;
}

ship* ship_create(int player,int type, game_t *game) {
  struct ship new_ship;
  new_ship.attack      = DEFAULT_ATTACK;
  new_ship.armor       = DEFAULT_ARMOR;
  new_ship.speed       = DEFAULT_SPEED;
  new_ship.life        = DEFAULT_LIFE;
  new_ship.position[0] = DEFAULT_POSITION_X;
  new_ship.position[1] = DEFAULT_POSITION_Y;
  new_ship.orientation = DEFAULT_ORIENTATION;
  new_ship.velocity[0] = 0;
  new_ship.velocity[1] = 0;
  new_ship.player      = player;

  new_ship.curr_weapon      = 0;
#ifdef DEBUG
  new_ship.num_weapons[0]  = (char)100;
  new_ship.num_weapons[1]  = (char)100;
  new_ship.num_weapons[2]  = (char)100;
  new_ship.num_weapons[3]  = (char)100;
  new_ship.num_weapons[4]  = (char)100;
  new_ship.num_weapons[5]  = (char)100;
  new_ship.num_weapons[6]  = (char)100;
#else
  new_ship.num_weapons[0]  = (char)0;
  new_ship.num_weapons[1]  = (char)0;
  new_ship.num_weapons[2]  = (char)0;
  new_ship.num_weapons[3]  = (char)0;
  new_ship.num_weapons[4]  = (char)0;
  new_ship.num_weapons[5]  = (char)0;
  new_ship.num_weapons[6]  = (char)0;
#endif

  pthread_rwlock_wrlock(&game->lllock);
  game->globalstate.lifeList.push_back(new_ship);
  pthread_rwlock_unlock(&game->lllock);

  return &game->globalstate.lifeList.back();
}

void send_game_state(int player, player_t **players, game_t *game)
{
  unsigned int i,size;
  char buf[MAXLINE];
  char *temp = buf;
  int *num_items;
  int j,intsize,floatsize,nweapon;
  player_t *curr;

  intsize   = sizeof(int);
  floatsize = sizeof(float);
  nweapon = 0;

  size = 0;
  temp = buf;

  /***************************************************************
    Sending the ship information
   *************************************************************/

  //Send the players his ship first.....
  curr = players[player];

  pthread_rwlock_rdlock(&game->lllock);
  //Update the attack strength of the ship 
  *((int *)temp) = game->globalstate.lifeList[player].attack;
  temp += intsize; 
  size += intsize;

  //Update the armor strength of the ship 
  *((int *)temp) = game->globalstate.lifeList[player].armor;
  temp += intsize; 
  size += intsize;

  //Update the speed of the ship
  *((int *)temp) = game->globalstate.lifeList[player].speed;
  temp += intsize;
  size += intsize;

  //Update the life of the ship
  *((int *)temp) = game->globalstate.lifeList[player].life;
  temp += intsize;
  size += intsize;

  //Update player's position
  *((float *)temp) = game->globalstate.lifeList[player].position[0];
  temp += floatsize;
  size += floatsize; 
  *((float *)temp) = game->globalstate.lifeList[player].position[1];
  temp += floatsize; 
  size += floatsize;

  //Update player's velocity
  *((float *)temp) = game->globalstate.lifeList[player].velocity[0];
  temp += floatsize;
  size += floatsize;
  *((float *)temp) = game->globalstate.lifeList[player].velocity[1];
  temp += floatsize; 
  size += floatsize; 

  //Update the player's orientation
  *((float *)temp) = game->globalstate.lifeList[player].orientation;
  temp += floatsize;
  size += floatsize; 

  //Sending the current weapon that the user has.
  *((int *)temp) = game->globalstate.lifeList[player].curr_weapon;
  temp += intsize;
  size += intsize;

  for(int x=0; x<7; x++) {
    *((int *)temp) = (int)game->globalstate.lifeList[player].num_weapons[x];      
    temp += intsize;
    size += intsize;
  }

  //Send the state of all of the other players. 
  //All this guy can know is the life of this player. 
  for(j=0; j < (int)(game->globalstate.lifeList.size() ); j++)
  {
    if(j == player)
      continue;

    //Send the player number of this ship...also the id number
    *((int *)temp) = game->globalstate.lifeList[j].player;
    temp += intsize;
    size += intsize;

    //Update the life of the ship
    *((int *)temp) = game->globalstate.lifeList[j].life;
    temp += intsize;
    size += intsize;
  }
  pthread_rwlock_unlock(&game->lllock);



  /***************************************************************
    Sending the bullet information
   *************************************************************/

  //Set up the pointer int storing number of bullets.
  num_items = (int *)temp;
  (*num_items) = 0;
  temp += intsize;
  size += intsize;

  pthread_rwlock_rdlock(&game->bllock);
  //Now, add all of the random weapons that appear on our screen.
  for(i=0;i<game->globalstate.bulletList.size();i++)
  {
    if(game->globalstate.bulletList[i].screen == player)
    {   
      //Copy the id number of the bullet.
      *((int *)temp) = game->globalstate.bulletList[i].id;
      temp += intsize;
      size += intsize;

      //Copy the attack strength of the bullet.
      *((int *)temp) = game->globalstate.bulletList[i].attack;
      temp += intsize;
      size += intsize;

      //Copy the current position of the bullet.
      *((float *)temp) = game->globalstate.bulletList[i].position[0];
      temp += floatsize;
      size += floatsize;
      *((float *)temp) = game->globalstate.bulletList[i].position[1];
      temp += floatsize;
      size += floatsize;

      //Copy the orientation of the bullet.
      *((float *)temp) = game->globalstate.bulletList[i].orientation;
      temp += floatsize;
      size += floatsize;

      //Copy the player number of the bullet
      *((int *)temp) = game->globalstate.bulletList[i].player;
      temp += intsize;
      size += intsize;

      //Copy the screen that this is on. This is redundant.
      *((int *)temp) = game->globalstate.bulletList[i].screen;
      temp += intsize;
      size += intsize;

      //Copy the distance traveled 
      *((float *)temp) = game->globalstate.bulletList[i].distance;
      temp += floatsize;
      size += floatsize;

      (*num_items) ++;
    }
  }
  pthread_rwlock_unlock(&game->bllock);

  //Send the portal rotation value.
  *((float *)temp) = game->globalstate.portalRotation;
  temp += floatsize;
  size += floatsize;



  /***************************************************************
    Sending the mine information
   *************************************************************/

  //Set up the pointer int storing number of bullets.
  num_items = (int *)temp;
  (*num_items) = 0;
  temp += intsize;
  size += intsize;

  pthread_rwlock_rdlock(&game->mlock);
  //Now, add all of the random weapons that appear on our screen.
  for(i=0;i<game->globalstate.mineList.size();i++)
  {
    if(game->globalstate.mineList[i].screen == player)
    {  
      //Copy the id number of the bullet.
      *((int *)temp) = game->globalstate.mineList[i].id;
      temp += intsize;
      size += intsize;

      //Copy the current position of the mine.
      *((float *)temp) = game->globalstate.mineList[i].position[0];
      temp += floatsize;
      size += floatsize;
      *((float *)temp) = game->globalstate.mineList[i].position[1];
      temp += floatsize;
      size += floatsize;

      //Copy the player number of the mine
      *((int *)temp) = game->globalstate.mineList[i].player;
      temp += intsize;
      size += intsize;

      //Copy the screen that this is on. This is redundant.
      *((int *)temp) = game->globalstate.mineList[i].screen;
      temp += intsize;
      size += intsize;

      (*num_items) ++;
    }
  }
  pthread_rwlock_unlock(&game->mlock);


  /***************************************************************
    Sending the ufo information
   *************************************************************/

  //Set up the pointer int storing number of bullets.
  num_items = (int *)temp;
  (*num_items) = 0;
  temp += intsize;
  size += intsize;

  pthread_rwlock_rdlock(&game->ulock);
  //Now, add all of the random weapons that appear on our screen.
  for(i=0;i<game->globalstate.UFOList.size();i++)
  {
    if(game->globalstate.UFOList[i].screen == player)
    {  
      //Copy the id number of the bullet.
      *((int *)temp) = game->globalstate.UFOList[i].id;
      temp += intsize;
      size += intsize;

      //Copy the current position of the mine.
      *((float *)temp) = game->globalstate.UFOList[i].position[0];
      temp += floatsize;
      size += floatsize;
      *((float *)temp) = game->globalstate.UFOList[i].position[1];
      temp += floatsize;
      size += floatsize;

      //Copy the current orientation of the ufo
      *((float *)temp) = game->globalstate.UFOList[i].orientation;
      temp += floatsize;
      size += floatsize;

      //Copy the player number of the mine
      *((int *)temp) = game->globalstate.UFOList[i].player;
      temp += intsize;
      size += intsize;

      //Copy the screen that this is on. This is redundant.
      *((int *)temp) = game->globalstate.UFOList[i].screen;
      temp += intsize;
      size += intsize;

      (*num_items) ++;
    }
  }
  pthread_rwlock_unlock(&game->ulock);

  /***************************************************************
    Sending the gunship information
   *************************************************************/

  //Set up the pointer int storing number of gunships.
  num_items = (int *)temp;
  (*num_items) = 0;
  temp += intsize;
  size += intsize;

  pthread_rwlock_rdlock(&game->glock);
  //Add all the gunships
  for(i=0;i<game->globalstate.GunshipList.size();i++)
  {
    if(game->globalstate.GunshipList[i].screen == player)
    {  
      //Copy the id number of the bullet.
      *((int *)temp) = game->globalstate.GunshipList[i].id;
      temp += intsize;
      size += intsize;

      //Copy the current position of the mine.
      *((float *)temp) = game->globalstate.GunshipList[i].position[0];
      temp += floatsize;
      size += floatsize;
      *((float *)temp) = game->globalstate.GunshipList[i].position[1];
      temp += floatsize;
      size += floatsize;

      //Copy the current orientation of the ufo
      *((float *)temp) = game->globalstate.GunshipList[i].orientation;
      temp += floatsize;
      size += floatsize;

      //Copy the player number of the mine
      *((int *)temp) = game->globalstate.GunshipList[i].player;
      temp += intsize;
      size += intsize;

      //Copy the screen that this is on. This is redundant.
      *((int *)temp) = game->globalstate.GunshipList[i].screen;
      temp += intsize;
      size += intsize;

      (*num_items) ++;
    }
  }
  pthread_rwlock_unlock(&game->glock);


  /***************************************************************
    Sending the mineship information
   *************************************************************/

  //Set up the pointer int storing number of bullets.
  num_items = (int *)temp;
  (*num_items) = 0;
  temp += intsize;
  size += intsize;

  pthread_rwlock_rdlock(&game->mslock);
  //Now, add all of the random weapons that appear on our screen.
  for(i=0;i<game->globalstate.MineshipList.size();i++)
  {
    if(game->globalstate.MineshipList[i].screen == player)
    {  
      //Copy the id number of the bullet.
      *((int *)temp) = game->globalstate.MineshipList[i].id;
      temp += intsize;
      size += intsize;

      //Copy the current position of the mine.
      *((float *)temp) = game->globalstate.MineshipList[i].position[0];
      temp += floatsize;
      size += floatsize;
      *((float *)temp) = game->globalstate.MineshipList[i].position[1];
      temp += floatsize;
      size += floatsize;

      //Copy the mineship next drop time.
      *((float *)temp) = game->globalstate.MineshipList[i].mineCD;
      temp += floatsize;
      size += floatsize;

      //Copy the current orientation of the ufo
      *((float *)temp) = game->globalstate.MineshipList[i].orientation;
      temp += floatsize;
      size += floatsize;

      //Copy the player number of the mine
      *((int *)temp) = game->globalstate.MineshipList[i].player;
      temp += intsize;
      size += intsize;

      //Copy the screen that this is on. This is redundant.
      *((int *)temp) = game->globalstate.MineshipList[i].screen;
      temp += intsize;
      size += intsize;

      (*num_items) ++;
    }
  }
  pthread_rwlock_unlock(&game->mslock);


  /***************************************************************
    Sending the nuke information
   *************************************************************/

  //Set up the pointer int storing number of bullets.
  num_items = (int *)temp;
  (*num_items) = 0;
  temp += intsize;
  size += intsize;

  pthread_rwlock_rdlock(&game->nlock);
  //Now, add all of the random weapons that appear on our screen.
  for(i=0;i<game->globalstate.nukeList.size();i++)
  {
    if(game->globalstate.nukeList[i].screen == player)
    {  
      //Copy the id number of the bullet.
      *((int *)temp) = game->globalstate.nukeList[i].id;
      temp += intsize;
      size += intsize;

      //Copy the current position of the mine.
      *((float *)temp) = game->globalstate.nukeList[i].position[0];
      temp += floatsize;
      size += floatsize;
      *((float *)temp) = game->globalstate.nukeList[i].position[1];
      temp += floatsize;
      size += floatsize;

      //Send the nuke timer.
      *((int *)temp) = game->globalstate.nukeList[i].timer;
      temp += intsize;
      size += intsize;

      //Copy the player number of the mine
      *((int *)temp) = game->globalstate.nukeList[i].player;
      temp += intsize;
      size += intsize;

      //Copy the current orientation of the ufo
      *((float *)temp) = game->globalstate.nukeList[i].orientation;
      temp += floatsize;
      size += floatsize;

      //Copy the screen that this is on. This is redundant.
      *((int *)temp) = game->globalstate.nukeList[i].screen;
      temp += intsize;
      size += intsize;

      (*num_items) ++;
    }
  }
  pthread_rwlock_unlock(&game->nlock);



  /***************************************************************
    Sending the emp  information
   *************************************************************/

  //Set up the pointer int storing number of bullets.
  num_items = (int *)temp;
  (*num_items) = 0;
  temp += intsize;
  size += intsize;

  pthread_rwlock_rdlock(&game->elock);
  //Now, add all of the random weapons that appear on our screen.
  for(i=0;i<game->globalstate.empList.size();i++)
  {
    if(game->globalstate.empList[i].screen == player)
    {  
      //Copy the id number of the bullet.
      *((int *)temp) = game->globalstate.empList[i].id;
      temp += intsize;
      size += intsize;

      //Copy the current position of the mine.
      *((float *)temp) = game->globalstate.empList[i].position[0];
      temp += floatsize;
      size += floatsize;
      *((float *)temp) = game->globalstate.empList[i].position[1];
      temp += floatsize;
      size += floatsize;

      //Send the nuke timer.
      *((int *)temp) = game->globalstate.empList[i].timer;
      temp += intsize;
      size += intsize;

      //Copy the player number of the mine
      *((int *)temp) = game->globalstate.empList[i].player;
      temp += intsize;
      size += intsize;

      //Copy the current orientation of the ufo
      *((float *)temp) = game->globalstate.empList[i].orientation;
      temp += floatsize;
      size += floatsize;


      //Copy the screen that this is on. This is redundant.
      *((int *)temp) = game->globalstate.empList[i].screen;
      temp += intsize;
      size += intsize;

      (*num_items) ++;
    }
  }
  pthread_rwlock_unlock(&game->elock);


  /***************************************************************
    Sending the hexagon information
   *************************************************************/

  //Set up the pointer int storing number of bullets.
  num_items = (int *)temp;
  (*num_items) = 0;
  temp += intsize;
  size += intsize;

  pthread_rwlock_rdlock(&game->hlock);
  //Now, add all of the random weapons that appear on our screen.
  for(i=0;i<game->globalstate.hexagonList.size();i++)
  {
    if(game->globalstate.hexagonList[i].screen == player)
    {  
      //Copy the id number of the bullet.
      *((int *)temp) = game->globalstate.hexagonList[i].id;
      temp += intsize;
      size += intsize;

      //Copy the current position of the mine.
      *((float *)temp) = game->globalstate.hexagonList[i].position[0];
      temp += floatsize;
      size += floatsize;
      *((float *)temp) = game->globalstate.hexagonList[i].position[1];
      temp += floatsize;
      size += floatsize;

      //Copy the player size 
      *((int *)temp) = game->globalstate.hexagonList[i].size;
      temp += intsize;
      size += intsize;

      //Copy the player number of the hexagon
      *((int *)temp) = game->globalstate.hexagonList[i].player;
      temp += intsize;
      size += intsize;

      //Copy the current orientation of the hexagon
      *((float *)temp) = game->globalstate.hexagonList[i].orientation;
      temp += floatsize;
      size += floatsize;


      //Copy the screen that this is on. This is redundant.
      *((int *)temp) = game->globalstate.hexagonList[i].screen;
      temp += intsize;
      size += intsize;

      (*num_items) ++;
    }
  }
  pthread_rwlock_unlock(&game->hlock);
  
  /***************************************************************
    Sending the powerup information
   *************************************************************/

  //Set up the pointer int storing number of powerups.
  num_items = (int *)temp;
  (*num_items) = 0;
  temp += intsize;
  size += intsize;

  
  pthread_rwlock_rdlock(&game->plock);
  for(i=0;i<game->globalstate.powerupList.size();i++)
  {
    if(game->globalstate.powerupList[i].screen == player)
    {  
      //Copy the id number of the bullet.
      *((int *)temp) = game->globalstate.powerupList[i].id;
      temp += intsize;
      size += intsize;

      //Copy the current position of the powerup.
      *((float *)temp) = game->globalstate.powerupList[i].position[0];
      temp += floatsize;
      size += floatsize;
      *((float *)temp) = game->globalstate.powerupList[i].position[1];
      temp += floatsize;
      size += floatsize;

      //Copy the screen
      *((int *)temp) = game->globalstate.powerupList[i].screen;
      temp += intsize;
      size += intsize;

      //Copy the type
      *((int *)temp) = game->globalstate.powerupList[i].type;
      temp += intsize;
      size += intsize;

      (*num_items) ++;
    }
  }
  pthread_rwlock_unlock(&game->plock);

  //Set up the pointer int storing number of items to remove..
  num_items = (int *)temp;
  (*num_items) = 0;
  temp += intsize;
  size += intsize;

  //Send the client items he should remove from the screen.
  //This includes either items that portaled or items that died.
  //Send it the new screen for the item (-1 if the item is dead)
  pthread_rwlock_wrlock(&game->rmlock);
  for(i=0;i<game->globalstate.removeList.size();i++)
  {
    if(game->globalstate.removeList[i].curr_screen == player)
    {   
      //Copy the id number of the bullet.
      *((int *)temp) = game->globalstate.removeList[i].id;
      temp += intsize;
      size += intsize;
      (*num_items) ++;
    }
  }
  pthread_rwlock_unlock(&game->rmlock);

  //Send how the client needs to flash the screen if it needs to
  *((float *)temp) = game->globalstate.flashScreen[player];
  temp += floatsize;
  size += floatsize;
  game->globalstate.flashScreen[player]=NO_FLASH;
  
  send_message(MSG_GSERVER_BULLET,buf,size,players[player]->fd);
}

void rotate_portal(game_t *game)
{
  //Add to the rotation value.
  game->globalstate.portalRotation += (.01f * (float)game->rotation_counter);
  if(game->globalstate.portalRotation >= 360.0f)
    game->globalstate.portalRotation -= 360.0f;
  if(game->globalstate.portalRotation < 0.0f)
    game->globalstate.portalRotation += 360.0f;

  //Check to see if we should be changing directions.
  if(game->rotation_counter == 500)
    game->portal_direction = COUNTERCLOCKWISE;
  else if(game->rotation_counter == -500)
    game->portal_direction = CLOCKWISE;

  //Actually change the portal rotation values.
  if(game->portal_direction == CLOCKWISE)
    game->rotation_counter += 2;
  else 
    game->rotation_counter -= 2;
}

void update_state(player_t *player, float time)
{
  if((player->L == MSG_GCLIENT_KEYDOWN && player->reverseTimer <= 0) || (player->R == MSG_GCLIENT_KEYDOWN && player->reverseTimer > 0))
  {
    ship_rotate(player->s,-1,time);
  }
  if((player->R == MSG_GCLIENT_KEYDOWN && player->reverseTimer <= 0) || (player->L == MSG_GCLIENT_KEYDOWN && player->reverseTimer > 0))
  {
    ship_rotate(player->s, 1,time);
  }
  if(player->U == MSG_GCLIENT_KEYDOWN)
  {
    ship_accelerate(player->s, time);    
  }
  if(player->D == MSG_GCLIENT_KEYDOWN)
  {
    ship_deccelerate(player->s, time);    
  }
  move_ship(player->s, time);
  player->reverseTimer -= time;
}

void spread_mines(int mine, game_t *game)
{
  for(int i = 0; i < MINES_SPAWNED; i++)
  {
    struct mine new_mine;
    pthread_rwlock_wrlock(&game->itemlock);
    new_mine.id = game->globalstate.itemnumber;
    game->globalstate.itemnumber++;
    pthread_rwlock_unlock(&game->itemlock);

    new_mine.active = true;
    new_mine.distance = rand()%100/150.0;
    new_mine.position[0] = game->globalstate.mineList[mine].position[0];
    new_mine.position[1] = game->globalstate.mineList[mine].position[1];
    new_mine.orientation = game->globalstate.mineList[mine].orientation + rand()%40-20;
    new_mine.player = game->globalstate.mineList[mine].player;
    new_mine.screen = game->globalstate.mineList[mine].screen;
    new_mine.life   = MINE_LIFE;
    game->globalstate.mineList.push_back(new_mine);
  }
}



/**@brief Spawns a random powerup on the screen.
 *
 * This should eventually be integrated into one function with 
 * initialize_special_weapon 
 *
 * @param game A pointer to the game structure for this game
 * @param screen The screen to spawn this powerup on.
 *
 * @return Void.
 */
void spawn_random_powerup(game_t * game, int screen, float *position, bool location)
{
  int weapon = rand() % 11;
  float newx, newy, mag, orientation;

  if(location) {
    newx = position[0];
    newy = position[1];
    orientation = ((float)rand() / (float)RAND_MAX) * 360.0f; 
  }else {
    newx = (((float)rand() / (float)RAND_MAX) * 20.0f) - 10.0f;
    newy = (((float)rand() / (float)RAND_MAX) * 20.0f) - 10.0f;

    mag = magnitude(newx,newy); 
    scale(&newx,&newy,0.26f,mag);
    orientation = getAngle(newx,newy);
  }

  powerup new_powerup;
  switch(weapon)
  {
    case 0:
      mine new_mine;
      pthread_rwlock_wrlock(&game->itemlock);
      new_mine.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);

      new_mine.position[0] = newx;
      new_mine.position[1] = newy;
      new_mine.orientation = orientation;

      new_mine.player = -1;
      new_mine.screen = screen;
      new_mine.active = false;
      new_mine.distance = 0;
      new_mine.life   = MINE_LIFE;

      pthread_rwlock_wrlock(&game->mlock);
      game->globalstate.mineList.push_back(new_mine);
      pthread_rwlock_unlock(&game->mlock);

      break;

    case 1:
      ufo new_ufo;
      pthread_rwlock_wrlock(&game->itemlock);
      new_ufo.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);

      new_ufo.position[0] = newx;
      new_ufo.position[1] = newy;
      new_ufo.orientation = orientation;

      new_ufo.player = -1;
      new_ufo.screen = screen;
      new_ufo.active = false;
      new_ufo.life   = UFO_LIFE;

      pthread_rwlock_wrlock(&game->ulock);
      game->globalstate.UFOList.push_back(new_ufo);
      pthread_rwlock_unlock(&game->ulock);

      break;

    case 2:
      gunship new_gunship;
      pthread_rwlock_wrlock(&game->itemlock);
      new_gunship.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++ ;
      pthread_rwlock_unlock(&game->itemlock);

      new_gunship.position[0] = newx;
      new_gunship.position[1] = newy;
      new_gunship.orientation = orientation;

      new_gunship.player = -1;
      new_gunship.screen = screen;
      new_gunship.active = false;
      new_gunship.life   = GUNSHIP_LIFE;

      pthread_rwlock_wrlock(&game->glock);
      game->globalstate.GunshipList.push_back(new_gunship);
      pthread_rwlock_unlock(&game->glock);

      break;

    case 3:
      mineship new_mineship;
      pthread_rwlock_wrlock(&game->itemlock);
      new_mineship.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);

      new_mineship.position[0] = newx;
      new_mineship.position[1] = newy;
      new_mineship.orientation = orientation;

      new_mineship.mineCD = 5; 
      new_mineship.player = -1;
      new_mineship.screen = screen;
      new_mineship.active = false;
      new_mineship.life   = MINESHIP_LIFE;

      pthread_rwlock_wrlock(&game->mslock);
      game->globalstate.MineshipList.push_back(new_mineship);
      pthread_rwlock_unlock(&game->mslock);

      break;

    case 4:

      nuke new_nuke;
      pthread_rwlock_wrlock(&game->itemlock);
      new_nuke.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);

      new_nuke.position[0] = newx;
      new_nuke.position[1] = newy;
      new_nuke.orientation = orientation;

      new_nuke.timer = 10;

      new_nuke.player = -1;
      new_nuke.screen = screen;
      new_nuke.active = false;
      new_nuke.life   = NUKE_LIFE;
      new_nuke.speed  = NUKE_SPEED;

      pthread_rwlock_wrlock(&game->nlock);
      game->globalstate.nukeList.push_back(new_nuke);
      pthread_rwlock_unlock(&game->nlock);

      break;

    case 5:
      emp new_emp;
      pthread_rwlock_wrlock(&game->itemlock);
      new_emp.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);

      new_emp.position[0] = newx;
      new_emp.position[1] = newy;
      new_emp.orientation = orientation;

      new_emp.timer = 10;

      new_emp.screen = screen;
      new_emp.player = -1;
      new_emp.active = false;
      new_emp.life   = EMP_LIFE;
      new_emp.speed  = EMP_SPEED;

      pthread_rwlock_wrlock(&game->elock);
      game->globalstate.empList.push_back(new_emp);
      pthread_rwlock_unlock(&game->elock);

      break;
    case 6:
      hexagon new_hexagon;
      pthread_rwlock_wrlock(&game->itemlock);
      new_hexagon.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);


      new_hexagon.position[0] = newx;
      new_hexagon.position[1] = newy;
      new_hexagon.orientation = orientation;

      new_hexagon.size = 100;  
      new_hexagon.screen = screen;
      new_hexagon.player = -1;
      new_hexagon.active = false;

      pthread_rwlock_wrlock(&game->hlock);
      game->globalstate.hexagonList.push_back(new_hexagon);
      pthread_rwlock_unlock(&game->hlock);

      break;
      
    case 7:
      pthread_rwlock_wrlock(&game->itemlock);
      new_powerup.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);

      new_powerup.position[0] = newx;
      new_powerup.position[1] = newy;
      new_powerup.orientation = orientation;
      
      new_powerup.type = POWERUP_HEALTH;
      new_powerup.screen = screen;

      pthread_rwlock_wrlock(&game->plock);
      game->globalstate.powerupList.push_back(new_powerup);
      pthread_rwlock_unlock(&game->plock);

      break;
      
    case 8:
      pthread_rwlock_wrlock(&game->itemlock);
      new_powerup.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);

      new_powerup.position[0] = newx;
      new_powerup.position[1] = newy;
      new_powerup.orientation = orientation;
      
      new_powerup.type = POWERUP_POWER;
      new_powerup.screen = screen;

      pthread_rwlock_wrlock(&game->plock);
      game->globalstate.powerupList.push_back(new_powerup);
      pthread_rwlock_unlock(&game->plock);

      break;
      
    case 9:
      pthread_rwlock_wrlock(&game->itemlock);
      new_powerup.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);

      new_powerup.position[0] = newx;
      new_powerup.position[1] = newy;
      new_powerup.orientation = orientation;
      
      new_powerup.type = POWERUP_ARMOR;
      new_powerup.screen = screen;

      pthread_rwlock_wrlock(&game->plock);
      game->globalstate.powerupList.push_back(new_powerup);
      pthread_rwlock_unlock(&game->plock);

      break;
      
    case 10:
      pthread_rwlock_wrlock(&game->itemlock);
      new_powerup.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);

      new_powerup.position[0] = newx;
      new_powerup.position[1] = newy;
      new_powerup.orientation = orientation;
      
      new_powerup.type = POWERUP_SPEED;
      new_powerup.screen = screen;

      pthread_rwlock_wrlock(&game->plock);
      game->globalstate.powerupList.push_back(new_powerup);
      pthread_rwlock_unlock(&game->plock);

      break;

  }

}

/**@brief What to do in the case that a bullet hits the portal
 *
 * Right now, this will count the portal hitpoints down. When it gets 
 * to 0, it will reset and then spawn a random powerup on the screen
 * of the player that reduced his portals hitpoints to 0.
 *
 * @param game A pointer to the gamestate.
 * @param b A pointer to the bullet that hit the portal.
 *
 * @return Void.
 */
void portal_attacked(game_t *game,bullet *b)
{
  int player_num = b->player;
  player_t * p = game->players_arr[player_num]; 

  p->portal_hitpoints --;
  if(p->portal_hitpoints == 0)
  {
    spawn_random_powerup(game, p->id, NULL, false);    
    p->portal_hitpoints = PORTAL_MAX_HITPOINTS;
  }
}



void deploy_mine(int mineship, game_t *game)
{
  struct mine new_mine;
  pthread_rwlock_wrlock(&game->itemlock);
  new_mine.id = game->globalstate.itemnumber;
  game->globalstate.itemnumber++;
  pthread_rwlock_unlock(&game->itemlock);

  new_mine.active = true;
  new_mine.distance = 0;
  new_mine.position[0] = game->globalstate.MineshipList[mineship].position[0];
  new_mine.position[1] = game->globalstate.MineshipList[mineship].position[1];
  new_mine.orientation = game->globalstate.MineshipList[mineship].orientation;
  new_mine.player = game->globalstate.MineshipList[mineship].player;
  new_mine.screen = game->globalstate.MineshipList[mineship].screen;
  new_mine.life   = MINE_LIFE;

  pthread_rwlock_wrlock(&game->mlock);
  game->globalstate.mineList.push_back(new_mine);
  pthread_rwlock_unlock(&game->mlock);
}

void gunship_shoot(int gunship, game_t *game)
{
  if(game->globalstate.GunshipList[gunship].screen >= MAX_PLAYERS)
    return;

  bullet new_bullet;
  pthread_rwlock_wrlock(&game->itemlock);
  new_bullet.id          = game->globalstate.itemnumber;
  game->globalstate.itemnumber ++;
  pthread_rwlock_unlock(&game->itemlock);

  new_bullet.attack      = GUNSHIP_ATTACK;
  new_bullet.position[0] = game->globalstate.GunshipList[gunship].position[0];
  new_bullet.position[1] = game->globalstate.GunshipList[gunship].position[1];
  int bestx = -2;
  int besty = -2;
  float best = 10;
  float x1 = game->players_arr[game->globalstate.GunshipList[gunship].screen]->s->position[0];
  float y1 = game->players_arr[game->globalstate.GunshipList[gunship].screen]->s->position[1];
  for (int dx = -2; dx <= 2; dx += 2)
  {
    for (int dy = -2; dy <= 2; dy += 2)
    {
      float dist = magnitude(x1+dx-new_bullet.position[0], y1+dy-new_bullet.position[1]);
      if(dist < best){
        best = dist;
        bestx = dx;
        besty = dy;
      }
    }
  }
  new_bullet.orientation = getAngle(x1+bestx-new_bullet.position[0], y1+besty-new_bullet.position[1]);
  new_bullet.player      = game->globalstate.GunshipList[gunship].player;
  new_bullet.screen      = game->globalstate.GunshipList[gunship].screen;
  new_bullet.distance    = 0.0f;
  new_bullet.active      = true;
  new_bullet.speed       = BULLET_SPEED;

  pthread_rwlock_wrlock(&game->bllock);
  game->globalstate.bulletList.push_back(new_bullet);
  pthread_rwlock_unlock(&game->bllock);
}

void move_mines(float time, game_t *game)
{
  float yrotrad,x,y,x1,y1,inter,curr_rot,lower,upper;
  int curr_screen,new_screen,degree_range,j;
  unsigned int i;
  struct removeItem r;
  degree_range = 360 / NUMBER_OF_PORTALS; 
  curr_rot = game->globalstate.portalRotation;
  /****************************************************************
    Begin moving mines
   *****************************************************************/
  pthread_rwlock_wrlock(&game->mlock);
  for(i=0;i<game->globalstate.mineList.size();i++)
  {
    //Get the state of the mine that we are moving.
    x = game->globalstate.mineList[i].position[0];
    y = game->globalstate.mineList[i].position[1];
    curr_screen = game->globalstate.mineList[i].screen;
    new_screen  = curr_screen;

    //If this mine belongs to a neutral player.......
    if(game->globalstate.mineList[i].player == -1)
    {
      if(curr_screen < MAX_PLAYERS)
      {
        //For testing purposes.
        //Get the location of the ship on the current screen.
        x1 = game->players_arr[curr_screen]->s->position[0];
        y1 = game->players_arr[curr_screen]->s->position[1];

        //If the mine is within range.
        if((distance(x,y,x1,y1) <= SHIP_RADIUS))
        {
          //Add to the number of mines for the certain player and remove the mine from the screen.
          game->players_arr[curr_screen]->s->num_weapons[0] ++;
          r.id = game->globalstate.mineList[i].id;
          r.curr_screen = game->globalstate.mineList[i].screen;
          game->globalstate.removeList.push_back(r);
          game->globalstate.mineList.erase(game->globalstate.mineList.begin()+i);
        }
      }
    }
    //Otherwise, this mine does not belong to a neutral...do your worst.
    else
    {
      if(curr_screen < MAX_PLAYERS && game->globalstate.mineList[i].player != curr_screen)
      {
        x1 = game->players_arr[curr_screen]->s->position[0];
        y1 = game->players_arr[curr_screen]->s->position[1];
        if((distance(x,y,x1,y1) <= SHIP_RADIUS))
        {
          game->players_arr[curr_screen]->s->life -= (MINE_DAMAGE - (int)(MINE_DAMAGE*ARMOR_PERCENT*game->players_arr[curr_screen]->s->armor));
          r.id = game->globalstate.mineList[i].id;
          r.curr_screen = game->globalstate.mineList[i].screen;
          game->globalstate.removeList.push_back(r);
          game->globalstate.mineList.erase(game->globalstate.mineList.begin()+i);
          continue;
        }
      }
    }  

    //If the mine is going into a portal...it also must not be active
    if(magnitude(x,y) < PORTAL_RADIUS && !game->globalstate.mineList[i].active)  
    {
      inter = getAngle(x,y);
      for(j=0;j<NUMBER_OF_PORTALS;j++)
      {
        getPortalAngles(j, &upper, &lower, game);

        //Check to see if the player is neutral again...make sure we don't activate the mine
        if(game->globalstate.mineList[i].player == -1)
        {
          //case where we are overlapping the intersection (lower = 330, upper = 30).
          if(upper < lower && (inter < upper || inter > lower))
          {
            new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
            while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
              new_screen = rand()%MAX_PLAYERS;
            float midline = j*degree_range + game->globalstate.portalRotation;
            repositionObject(curr_screen, new_screen, &game->globalstate.mineList[i].orientation, game->globalstate.mineList[i].position, inter, midline, game);
            game->globalstate.mineList[i].screen = new_screen;
            break;
          }
          else if(upper > lower && inter < upper && inter > lower)
          {
            new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
            while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
              new_screen = rand()%MAX_PLAYERS;
            float midline = j*degree_range + game->globalstate.portalRotation;
            repositionObject(curr_screen, new_screen, &game->globalstate.mineList[i].orientation, game->globalstate.mineList[i].position, inter, midline, game);
            game->globalstate.mineList[i].screen = new_screen;
            break;
          }
        }

        //This is the normal case....a player owned mine floating around.
        else
        {
          getPortalAngles(j, &upper, &lower, game);
          //case where we are overlapping the intersection (lower = 330, upper = 30).
          if(upper < lower && (inter < upper || inter > lower))
          {
            new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
            while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
              new_screen = rand()%MAX_PLAYERS;
            float midline = j*degree_range + game->globalstate.portalRotation;
            repositionObject(curr_screen, new_screen, &game->globalstate.mineList[i].orientation, game->globalstate.mineList[i].position, inter, midline, game);
            game->globalstate.mineList[i].screen = new_screen;
            game->globalstate.mineList[i].active = true;
            game->globalstate.mineList[i].distance = rand()%100/150.0;
            spread_mines(i, game);
            break;
          }
          else if(upper > lower && inter < upper && inter > lower)
          {
            new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
            while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
              new_screen = rand()%MAX_PLAYERS;
            float midline = j*degree_range + game->globalstate.portalRotation;
            repositionObject(curr_screen, new_screen, &game->globalstate.mineList[i].orientation, game->globalstate.mineList[i].position, inter, midline, game);
            game->globalstate.mineList[i].screen = new_screen;
            game->globalstate.mineList[i].active = true;
            game->globalstate.mineList[i].distance = rand()%100/150.0;
            spread_mines(i, game);
            break;
          }
        } 
      }      
    }

    //Now, we calculate the movement of the mine...
    if(!game->globalstate.mineList[i].active || game->globalstate.mineList[i].distance > 0){
      yrotrad = game->globalstate.mineList[i].orientation / 180 * PI;
      x = MINE_SPEED * (float)sin(yrotrad);
      y = MINE_SPEED * (float)cos(yrotrad);
      move_object(game->globalstate.mineList[i].position, x*time, y*time);
      game->globalstate.mineList[i].distance -= magnitude(x*time, y*time);
    }

    //If it has passed through a portal, remove it
    if(curr_screen != new_screen)
    {
      r.id = game->globalstate.mineList[i].id;
      r.curr_screen = curr_screen; 
      game->globalstate.removeList.push_back(r);
    }
  }
  pthread_rwlock_unlock(&game->mlock);

  /****************************************************************
    End moving mines
   ***************************************************************/
}


/**@brief Function to move mineships's
 *
 *
 * @param time The time elapsed since the last update.
 * @param game A pointer to a game structure.
 *
 * @return Void.
 */
void move_mineships(float time, game_t *game)
{
  float yrotrad,x,y,x1,y1,inter,curr_rot,lower,upper;
  int curr_screen,new_screen,degree_range,j;
  unsigned int i;
  struct removeItem r;
  degree_range = 360 / NUMBER_OF_PORTALS; 
  curr_rot = game->globalstate.portalRotation;


  pthread_rwlock_wrlock(&game->mslock);
  for(i=0;i<game->globalstate.MineshipList.size();i++)
  {
    x = game->globalstate.MineshipList[i].position[0];
    y = game->globalstate.MineshipList[i].position[1];
    curr_screen = game->globalstate.MineshipList[i].screen;
    new_screen  = curr_screen;

    //If this mine belongs to a neutral player.......
    if(game->globalstate.MineshipList[i].player == -1)
    {
      if(curr_screen < MAX_PLAYERS)
      {
        //For testing purposes.
        //Get the location of the ship on the current screen.
        x1 = game->players_arr[curr_screen]->s->position[0];
        y1 = game->players_arr[curr_screen]->s->position[1];

        //If the mine is within range.
        if((distance(x,y,x1,y1) <= SHIP_RADIUS))
        {
          //Add to the number of mines for the certain player and remove the mine from the screen.
          game->players_arr[curr_screen]->s->num_weapons[3] ++;
          r.id = game->globalstate.MineshipList[i].id;
          r.curr_screen = game->globalstate.MineshipList[i].screen;
          game->globalstate.removeList.push_back(r);
          game->globalstate.MineshipList.erase(game->globalstate.MineshipList.begin()+i);
        }
      }
    }
    //Otherwise, this mine does not belong to a neutral...do your worst.
    else
    {
      if(curr_screen < MAX_PLAYERS && game->globalstate.MineshipList[i].player != curr_screen)
      {
        x1 = game->players_arr[curr_screen]->s->position[0];
        y1 = game->players_arr[curr_screen]->s->position[1];
        if((distance(x,y,x1,y1) <= SHIP_RADIUS))
        {
          game->players_arr[curr_screen]->s->life -= (COLLISION_DAMAGE - (int)(COLLISION_DAMAGE*ARMOR_PERCENT*game->players_arr[curr_screen]->s->armor));
          r.id = game->globalstate.MineshipList[i].id;
          r.curr_screen = game->globalstate.MineshipList[i].screen;
          game->globalstate.removeList.push_back(r);
          game->globalstate.MineshipList.erase(game->globalstate.MineshipList.begin()+i);
          continue;
        }
      }
    }


    if(magnitude(x,y) < PORTAL_RADIUS)  
    {
      inter = getAngle(x,y);
      for(j=0;j<NUMBER_OF_PORTALS;j++)
      {
        getPortalAngles(j, &upper, &lower, game);

        //Check to see if the player is neutral again...make sure we don't activate the mine
        if(game->globalstate.MineshipList[i].player == -1)
        {
          //case where we are overlapping the intersection (lower = 330, upper = 30).
          if(upper < lower && (inter < upper || inter > lower))
          {
            new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
            while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
              new_screen = rand()%MAX_PLAYERS;
            float midline = j*degree_range + game->globalstate.portalRotation;
            repositionObject(curr_screen, new_screen, &game->globalstate.MineshipList[i].orientation, game->globalstate.MineshipList[i].position, inter, midline, game);
            game->globalstate.MineshipList[i].screen = new_screen;
            break;
          }
          else if(upper > lower && inter < upper && inter > lower)
          {
            new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
            while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
              new_screen = rand()%MAX_PLAYERS;
            float midline = j*degree_range + game->globalstate.portalRotation;
            repositionObject(curr_screen, new_screen, &game->globalstate.MineshipList[i].orientation, game->globalstate.MineshipList[i].position, inter, midline, game);
            game->globalstate.MineshipList[i].screen = new_screen;
            break;
          }
        }
        //This is the normal case....a player owned mine floating around.
        else
        {
          getPortalAngles(j, &upper, &lower, game);
          if((upper < lower && (inter < upper || inter > lower))||(upper > lower && inter < upper && inter > lower))
          {
            if(game->globalstate.MineshipList[i].active)
            {
              game->globalstate.MineshipList[i].orientation += 180;
              float angleDiff = game->globalstate.MineshipList[i].orientation - j*degree_range - game->globalstate.portalRotation;
              game->globalstate.MineshipList[i].orientation -= 2*angleDiff;
              inter = inter/180*PI;
              game->globalstate.MineshipList[i].position[0] = PORTAL_RADIUS*sin(inter);
              game->globalstate.MineshipList[i].position[1] = PORTAL_RADIUS*cos(inter);
              break;
            }
            else
            {
              new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
              while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
                new_screen = rand()%MAX_PLAYERS;
              float midline = j*degree_range + game->globalstate.portalRotation;
              repositionObject(curr_screen, new_screen, &game->globalstate.MineshipList[i].orientation, game->globalstate.MineshipList[i].position, inter, midline, game);
              game->globalstate.MineshipList[i].screen = new_screen;
              game->globalstate.MineshipList[i].active = true;
              break;
            }
          }
        }      
      }
    }

    if(game->globalstate.MineshipList[i].active){
      game->globalstate.MineshipList[i].mineCD -= time;
      if (game->globalstate.MineshipList[i].mineCD <= 0){
        game->globalstate.MineshipList[i].mineCD = MINESHIP_TIME;
        deploy_mine(i, game);
      }
    }
    //calculate the delta movement
    yrotrad = game->globalstate.MineshipList[i].orientation / 180 * PI;
    x = MINESHIP_SPEED * (float)sin(yrotrad);
    y = MINESHIP_SPEED * (float)cos(yrotrad);
    move_object(game->globalstate.MineshipList[i].position, x*time, y*time);
    //If it has passed through a portal, remove it
    if(curr_screen != new_screen)
    {
      r.id = game->globalstate.MineshipList[i].id;
      r.curr_screen = curr_screen; 
      game->globalstate.removeList.push_back(r);
    }
  }
  pthread_rwlock_unlock(&game->mslock);
}
/****************************************************************
  End moving mineships   
 ***************************************************************/

/**@brief Function to move UFO's
 *
 *
 * @param time The time elapsed since the last update.
 * @param game A pointer to a game structure.
 *
 * @return Void.
 */
void move_UFOS(float time, game_t *game)
{
  float yrotrad,x,y,x1,y1,inter,curr_rot,lower,upper;
  int curr_screen,new_screen,degree_range,j;
  unsigned int i;
  struct removeItem r;
  degree_range = 360 / NUMBER_OF_PORTALS; 
  curr_rot = game->globalstate.portalRotation;
  pthread_rwlock_wrlock(&game->ulock);

  //Iterate through all the UFOs in the list.
  for(i=0;i<game->globalstate.UFOList.size();i++)
  {
    x = game->globalstate.UFOList[i].position[0];
    y = game->globalstate.UFOList[i].position[1];
    curr_screen = game->globalstate.UFOList[i].screen;
    new_screen  = curr_screen;

    //If this UFO belongs to a neutral player.......
    if(game->globalstate.UFOList[i].player == -1)
    {
      if(curr_screen < MAX_PLAYERS)
      {
        //For testing purposes.
        //Get the location of the ship on the current screen.
        x1 = game->players_arr[curr_screen]->s->position[0];
        y1 = game->players_arr[curr_screen]->s->position[1];

        //If the ufo is within range.
        if((distance(x,y,x1,y1) <= SHIP_RADIUS))
        {
          //Add to the number of ufos for the certain player and remove the ufo from the screen.
          game->players_arr[curr_screen]->s->num_weapons[1] ++;
          r.id = game->globalstate.UFOList[i].id;
          r.curr_screen = game->globalstate.UFOList[i].screen;
          game->globalstate.removeList.push_back(r);
          game->globalstate.UFOList.erase(game->globalstate.UFOList.begin()+i);
        }
      }
    }
    else if(curr_screen < MAX_PLAYERS && game->globalstate.UFOList[i].player != curr_screen)
    {
      x1 = game->players_arr[curr_screen]->s->position[0];
      y1 = game->players_arr[curr_screen]->s->position[1];
      if((distance(x,y,x1,y1) <= SHIP_RADIUS))
      {
        game->players_arr[curr_screen]->s->life -= (UFO_DAMAGE - (int)(UFO_DAMAGE*ARMOR_PERCENT*game->players_arr[curr_screen]->s->armor));
        r.id = game->globalstate.UFOList[i].id;
        r.curr_screen = game->globalstate.UFOList[i].screen;
        game->globalstate.removeList.push_back(r);
        game->globalstate.UFOList.erase(game->globalstate.UFOList.begin()+i);
        continue;
      }
    }

    if(magnitude(x,y) < PORTAL_RADIUS)  
    {
      inter = getAngle(x,y);
      for(j=0;j<NUMBER_OF_PORTALS;j++)
      {
        getPortalAngles(j, &upper, &lower, game);
        //Check to see if the player is neutral again...make sure we don't activate the ufo
        if(game->globalstate.UFOList[i].player == -1)
        {
          //case where we are overlapping the intersection (lower = 330, upper = 30).
          if(upper < lower && (inter < upper || inter > lower))
          {
            new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
            while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
              new_screen = rand()%MAX_PLAYERS;
            float midline = j*degree_range + game->globalstate.portalRotation;
            repositionObject(curr_screen, new_screen, &game->globalstate.UFOList[i].orientation, game->globalstate.UFOList[i].position, inter, midline, game);
            game->globalstate.UFOList[i].screen = new_screen;
            break;
          }
          else if(upper > lower && inter < upper && inter > lower)
          {
            new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
            while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
              new_screen = rand()%MAX_PLAYERS;
            float midline = j*degree_range + game->globalstate.portalRotation;
            repositionObject(curr_screen, new_screen, &game->globalstate.UFOList[i].orientation, game->globalstate.UFOList[i].position, inter, midline, game);
            game->globalstate.UFOList[i].screen = new_screen;
            break;
          }
        }
        else
        {
          if((upper < lower && (inter < upper || inter > lower))||(upper > lower && inter < upper && inter > lower)){
            if(game->globalstate.UFOList[i].active)
            {
              game->globalstate.UFOList[i].orientation += 180;
              float angleDiff = game->globalstate.UFOList[i].orientation - j*degree_range - game->globalstate.portalRotation;
              game->globalstate.UFOList[i].orientation -= 2*angleDiff;
              inter = inter/180*PI;
              game->globalstate.UFOList[i].position[0] = PORTAL_RADIUS*sin(inter);
              game->globalstate.UFOList[i].position[1] = PORTAL_RADIUS*cos(inter);
              break;
            }
            else
            {
              new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
              while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
                new_screen = rand()%MAX_PLAYERS;
              float midline = j*degree_range + game->globalstate.portalRotation;
              repositionObject(curr_screen, new_screen, &game->globalstate.UFOList[i].orientation, game->globalstate.UFOList[i].position, inter, midline, game);
              game->globalstate.UFOList[i].screen = new_screen;
              game->globalstate.UFOList[i].active = true;
              break;
            }
          }
        }
      }      
    }

    //calculate the delta movement
    if(game->globalstate.UFOList[i].active && curr_screen < MAX_PLAYERS)
    {
      int bestx = -2;
      int besty = -2;
      float best = 10;
      x1 = game->players_arr[curr_screen]->s->position[0];
      y1 = game->players_arr[curr_screen]->s->position[1];
      for (int dx = -2; dx <= 2; dx += 2)
      {
        for (int dy = -2; dy <= 2; dy += 2)
        {
          float dist = magnitude(x1+dx-x, y1+dy-y);
          if(dist < best){
            best = dist;
            bestx = dx;
            besty = dy;
          }
        }
      }
      game->globalstate.UFOList[i].orientation = getAngle(x1+bestx-x, y1+besty-y);
    }
    yrotrad = game->globalstate.UFOList[i].orientation / 180 * PI;
    x = UFO_SPEED * (float)sin(yrotrad);
    y = UFO_SPEED * (float)cos(yrotrad);
    move_object(game->globalstate.UFOList[i].position, x*time, y*time);
    //If it has passed through a portal, remove it
    if(curr_screen != new_screen)
    {
      r.id = game->globalstate.UFOList[i].id;
      r.curr_screen = curr_screen; 
      game->globalstate.removeList.push_back(r);
    }
  }
  pthread_rwlock_unlock(&game->ulock);

  /****************************************************************
    End moving UFOS 
   ***************************************************************/
}

/**@brief Function to move the gun ships
 *
 *
 */
void move_gunships(float time, game_t *game)
{
  float yrotrad,x,y,x1,y1,inter,curr_rot,lower,upper;
  int curr_screen,new_screen,degree_range,j;
  unsigned int i;
  struct removeItem r;
  degree_range = 360 / NUMBER_OF_PORTALS; 
  curr_rot = game->globalstate.portalRotation;
  pthread_rwlock_wrlock(&game->glock);

  //Loop for each GunShip in the global list.
  for(i=0;i<game->globalstate.GunshipList.size();i++)
  {
    x = game->globalstate.GunshipList[i].position[0];
    y = game->globalstate.GunshipList[i].position[1];
    curr_screen = game->globalstate.GunshipList[i].screen;
    new_screen  = curr_screen;

    if(game->globalstate.GunshipList[i].player == -1)
    {
      if(curr_screen < MAX_PLAYERS)
      {
        //For testing purposes.
        //Get the location of the ship on the current screen.
        x1 = game->players_arr[curr_screen]->s->position[0];
        y1 = game->players_arr[curr_screen]->s->position[1];

        //If the ufo is within range.
        if((distance(x,y,x1,y1) <= SHIP_RADIUS))
        {
          //Add to the number of ufos for the certain player and remove the ufo from the screen.
          game->players_arr[curr_screen]->s->num_weapons[2] ++;
          r.id = game->globalstate.GunshipList[i].id;
          r.curr_screen = game->globalstate.GunshipList[i].screen;
          game->globalstate.removeList.push_back(r);
          game->globalstate.GunshipList.erase(game->globalstate.GunshipList.begin()+i);
        }
      }
    }

    else if(curr_screen < MAX_PLAYERS && game->globalstate.GunshipList[i].player != curr_screen)
    {
      x1 = game->players_arr[curr_screen]->s->position[0];
      y1 = game->players_arr[curr_screen]->s->position[1];
      if((distance(x,y,x1,y1) <= SHIP_RADIUS))
      {
        game->players_arr[curr_screen]->s->life -= (COLLISION_DAMAGE - (int)(COLLISION_DAMAGE*ARMOR_PERCENT*game->players_arr[curr_screen]->s->armor));
        r.id = game->globalstate.GunshipList[i].id;
        r.curr_screen = game->globalstate.GunshipList[i].screen;
        game->globalstate.removeList.push_back(r);
        game->globalstate.GunshipList.erase(game->globalstate.GunshipList.begin()+i);
        continue;
      }
    }

    if(magnitude(x,y) < PORTAL_RADIUS)  
    {
      inter = getAngle(x,y);
      for(j=0;j<NUMBER_OF_PORTALS;j++)
      {
        getPortalAngles(j, &upper, &lower, game);
        //Check to see if the player is neutral again...make sure we don't activate the ufo
        if(game->globalstate.GunshipList[i].player == -1)
        {
          //case where we are overlapping the intersection (lower = 330, upper = 30).
          if(upper < lower && (inter < upper || inter > lower))
          {
            new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
            while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
              new_screen = rand()%MAX_PLAYERS;
            float midline = j*degree_range + game->globalstate.portalRotation;
            repositionObject(curr_screen, new_screen, &game->globalstate.GunshipList[i].orientation, game->globalstate.GunshipList[i].position, inter, midline, game);
            game->globalstate.GunshipList[i].screen = new_screen;
            break;
          }
          else if(upper > lower && inter < upper && inter > lower)
          {
            new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
            while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
              new_screen = rand()%MAX_PLAYERS;
            float midline = j*degree_range + game->globalstate.portalRotation;
            repositionObject(curr_screen, new_screen, &game->globalstate.GunshipList[i].orientation, game->globalstate.GunshipList[i].position, inter, midline, game);
            game->globalstate.GunshipList[i].screen = new_screen;
            break;
          }
        }
        else
        {
          getPortalAngles(j, &upper, &lower, game);

          if((upper < lower && (inter < upper || inter > lower))||(upper > lower && inter < upper && inter > lower))
          {
            if(game->globalstate.GunshipList[i].active)
            {
              game->globalstate.GunshipList[i].orientation += 180;
              float angleDiff = game->globalstate.GunshipList[i].orientation - j*degree_range - game->globalstate.portalRotation;
              game->globalstate.GunshipList[i].orientation -= 2*angleDiff;
              inter = inter/180*PI;
              game->globalstate.GunshipList[i].position[0] = PORTAL_RADIUS*sin(inter);
              game->globalstate.GunshipList[i].position[1] = PORTAL_RADIUS*cos(inter);
              break;
            }
            else
            {
              new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
              while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
                new_screen = rand()%MAX_PLAYERS;
              float midline = j*degree_range + game->globalstate.portalRotation;
              repositionObject(curr_screen, new_screen, &game->globalstate.GunshipList[i].orientation, game->globalstate.GunshipList[i].position, inter, midline, game);
              game->globalstate.GunshipList[i].screen = new_screen;
              game->globalstate.GunshipList[i].active = true;
              break;
            }
          }
        }
      }      
    }
    //calculate the delta movement
    if(game->globalstate.GunshipList[i].active)
    {
      game->globalstate.GunshipList[i].shotTimer -= time;
      if(game->globalstate.GunshipList[i].shotTimer <= 0){
        game->globalstate.GunshipList[i].shotTimer = GUNSHIP_SHOT_TIME;
        gunship_shoot(i, game);
      }
    }
    yrotrad = game->globalstate.GunshipList[i].orientation / 180 * PI;
    x = GUNSHIP_SPEED * (float)sin(yrotrad);
    y = GUNSHIP_SPEED * (float)cos(yrotrad);
    move_object(game->globalstate.GunshipList[i].position, x*time, y*time);
    //If it has passed through a portal, remove it
    if(curr_screen != new_screen)
    {
      r.id = game->globalstate.GunshipList[i].id;
      r.curr_screen = curr_screen; 
      game->globalstate.removeList.push_back(r);
    }
  }
  pthread_rwlock_unlock(&game->glock);
}

void move_nukes(float time, game_t *game) {
  float yrotrad,x,y,x1,y1,inter,curr_rot,lower,upper;
  int curr_screen,new_screen,degree_range,j;
  unsigned int i;
  struct removeItem r;
  degree_range = 360 / NUMBER_OF_PORTALS; 
  curr_rot = game->globalstate.portalRotation;


  pthread_rwlock_wrlock(&game->nlock);
  for(i=0;i<game->globalstate.nukeList.size();i++)
  {
    x = game->globalstate.nukeList[i].position[0];
    y = game->globalstate.nukeList[i].position[1];
    curr_screen = game->globalstate.nukeList[i].screen;
    new_screen  = curr_screen;

    if(game->globalstate.nukeList[i].player == -1)
    {
      if(curr_screen < MAX_PLAYERS)
      {
        //For testing purposes.
        //Get the location of the ship on the current screen.
        x1 = game->players_arr[curr_screen]->s->position[0];
        y1 = game->players_arr[curr_screen]->s->position[1];

        //If the ufo is within range.
        if((distance(x,y,x1,y1) <= SHIP_RADIUS))
        {
          //Add to the number of ufos for the certain player and remove the ufo from the screen.
          game->players_arr[curr_screen]->s->num_weapons[4] ++;
          r.id = game->globalstate.nukeList[i].id;
          r.curr_screen = game->globalstate.nukeList[i].screen;
          game->globalstate.removeList.push_back(r);
          game->globalstate.nukeList.erase(game->globalstate.nukeList.begin()+i);
        }
      }
    }
    else if(curr_screen < MAX_PLAYERS && game->globalstate.nukeList[i].player != curr_screen)
    {
      x1 = game->players_arr[curr_screen]->s->position[0];
      y1 = game->players_arr[curr_screen]->s->position[1];
      if((distance(x,y,x1,y1) <= SHIP_RADIUS))
      {
        game->players_arr[curr_screen]->s->life -= (NUKE_DAMAGE - (int)(NUKE_DAMAGE*ARMOR_PERCENT*game->players_arr[curr_screen]->s->armor));
        r.id = game->globalstate.nukeList[i].id;
        r.curr_screen = game->globalstate.nukeList[i].screen;
        game->globalstate.removeList.push_back(r);
        game->globalstate.nukeList.erase(game->globalstate.nukeList.begin()+i);
        game->globalstate.flashScreen[curr_screen]=NUKE_FLASH;
        continue;
      }
    }


    if(magnitude(x,y) < PORTAL_RADIUS)  
    {
      inter = getAngle(x,y);
      for(j=0;j<NUMBER_OF_PORTALS;j++)
      {
        getPortalAngles(j, &upper, &lower, game);
        if((upper < lower && (inter < upper || inter > lower))||(upper > lower && inter < upper && inter > lower))
        {
          new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
          while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
            new_screen = rand()%MAX_PLAYERS;
          float midline = j*degree_range + game->globalstate.portalRotation;
          repositionObject(curr_screen, new_screen, &game->globalstate.nukeList[i].orientation, game->globalstate.nukeList[i].position, inter, midline, game);
          game->globalstate.nukeList[i].screen = new_screen;
          if(game->globalstate.nukeList[i].player != -1)
            game->globalstate.nukeList[i].active = true;
          break;
        }
      }
    }
    //calculate the delta movement
    if(game->globalstate.nukeList[i].active)
      game->globalstate.nukeList[i].speed -= time*NUKE_DECELERATION;
    if(game->globalstate.nukeList[i].speed < 0){
      if(curr_screen < MAX_PLAYERS)
        game->players_arr[curr_screen]->s->life -= (NUKE_DAMAGE - (int)(NUKE_DAMAGE*ARMOR_PERCENT*game->players_arr[curr_screen]->s->armor));
      r.id = game->globalstate.nukeList[i].id;
      r.curr_screen = game->globalstate.nukeList[i].screen;
      game->globalstate.removeList.push_back(r);
      game->globalstate.nukeList.erase(game->globalstate.nukeList.begin()+i);
      game->globalstate.flashScreen[curr_screen]=NUKE_FLASH;
    }
    yrotrad = game->globalstate.nukeList[i].orientation / 180 * PI;
    x = game->globalstate.nukeList[i].speed * (float)sin(yrotrad);
    y = game->globalstate.nukeList[i].speed * (float)cos(yrotrad);
    move_object(game->globalstate.nukeList[i].position, x*time, y*time);
    //If it has passed through a portal, remove it
    if(curr_screen != new_screen)
    {
      r.id = game->globalstate.nukeList[i].id;
      r.curr_screen = curr_screen; 
      game->globalstate.removeList.push_back(r);
    }
  }
  pthread_rwlock_unlock(&game->nlock);
}

void move_hexs(float time, game_t *game) {
  float yrotrad,x,y,x1,y1,inter,curr_rot,lower,upper;
  int curr_screen,new_screen,degree_range,j;
  unsigned int i;
  struct removeItem r;
  degree_range = 360 / NUMBER_OF_PORTALS; 
  curr_rot = game->globalstate.portalRotation;

  pthread_rwlock_wrlock(&game->hlock);
  for(i=0;i<game->globalstate.hexagonList.size();i++)
  {
    x = game->globalstate.hexagonList[i].position[0];
    y = game->globalstate.hexagonList[i].position[1];
    curr_screen = game->globalstate.hexagonList[i].screen;
    new_screen  = curr_screen;
    if(game->globalstate.hexagonList[i].player == -1)
    {
      if(curr_screen < MAX_PLAYERS)
      {
        //For testing purposes.
        //Get the location of the ship on the current screen.
        x1 = game->players_arr[curr_screen]->s->position[0];
        y1 = game->players_arr[curr_screen]->s->position[1];

        //If the ufo is within range.
        if((distance(x,y,x1,y1) <= SHIP_RADIUS))
        {
          //Add to the number of ufos for the certain player and remove the ufo from the screen.
          game->players_arr[curr_screen]->s->num_weapons[6] ++;
          r.id = game->globalstate.hexagonList[i].id;
          r.curr_screen = game->globalstate.hexagonList[i].screen;
          game->globalstate.removeList.push_back(r);
          game->globalstate.hexagonList.erase(game->globalstate.hexagonList.begin()+i);
        }
      }
    }
    else if(curr_screen < MAX_PLAYERS && game->globalstate.hexagonList[i].player != curr_screen)
    {
      x1 = game->players_arr[curr_screen]->s->position[0];
      y1 = game->players_arr[curr_screen]->s->position[1];
      if((distance(x,y,x1,y1) <= (float)game->globalstate.hexagonList[i].size/Graphics::screen.y/2.0))
      {
        game->players_arr[curr_screen]->s->life -= (COLLISION_DAMAGE - (int)(COLLISION_DAMAGE*ARMOR_PERCENT*game->players_arr[curr_screen]->s->armor));
        r.id = game->globalstate.hexagonList[i].id;
        r.curr_screen = game->globalstate.hexagonList[i].screen;
        game->globalstate.removeList.push_back(r);
        game->globalstate.hexagonList.erase(game->globalstate.hexagonList.begin()+i);
        continue;
      }
    }
    if(!game->globalstate.hexagonList[i].active && magnitude(x,y) < PORTAL_RADIUS+((float)game->globalstate.hexagonList[i].size-100)/Graphics::screen.y/2.0)  
    {
      inter = getAngle(x,y);
      for(j=0;j<NUMBER_OF_PORTALS;j++)
      {
        getPortalAngles(j, &upper, &lower, game);
        if((upper < lower && (inter < upper || inter > lower))||(upper > lower && inter < upper && inter > lower))
        {
          new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
          while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
            new_screen = rand()%MAX_PLAYERS;   
          float midline = j*degree_range + game->globalstate.portalRotation;
          repositionObject(curr_screen, new_screen, &game->globalstate.hexagonList[i].orientation, game->globalstate.hexagonList[i].position, inter, midline, game);
          game->globalstate.hexagonList[i].screen = new_screen;
          if(game->globalstate.hexagonList[i].player != -1)
            game->globalstate.hexagonList[i].active = true;
          break;
        } 
      }
    }

    //calculate the delta movement
    if(game->globalstate.hexagonList[i].active)
      game->globalstate.hexagonList[i].size++;
    yrotrad = game->globalstate.hexagonList[i].orientation / 180 * PI;
    x = HEXAGON_SPEED * (float)sin(yrotrad);
    y = HEXAGON_SPEED * (float)cos(yrotrad);
    move_object(game->globalstate.hexagonList[i].position, x*time, y*time);
    //If it has passed through a portal, remove it
    if(curr_screen != new_screen)
    {
      r.id = game->globalstate.hexagonList[i].id;
      r.curr_screen = curr_screen; 
      game->globalstate.removeList.push_back(r);
    }
  }
  pthread_rwlock_unlock(&game->hlock);
}

void move_powerups(float time, game_t *game){
  float yrotrad,x,y,x1,y1,inter,curr_rot,lower,upper;
  int curr_screen,new_screen,degree_range,j;
  unsigned int i;
  struct removeItem r;
  degree_range = 360 / NUMBER_OF_PORTALS; 
  curr_rot = game->globalstate.portalRotation;

  //check collisions on powerups
  pthread_rwlock_wrlock(&game->plock);
  for(i=0;i<game->globalstate.powerupList.size();i++)
  {
    x = game->globalstate.powerupList[i].position[0];
    y = game->globalstate.powerupList[i].position[1];
    curr_screen = game->globalstate.powerupList[i].screen;
    new_screen = curr_screen;
    if(curr_screen < MAX_PLAYERS)
    {
      //For testing purposes.
      //Get the location of the ship on the current screen.
      x1 = game->players_arr[curr_screen]->s->position[0];
      y1 = game->players_arr[curr_screen]->s->position[1];

      //If the powerup is within range.
      if((distance(x,y,x1,y1) <= SHIP_RADIUS))
      {
        if(game->globalstate.powerupList[i].type == POWERUP_HEALTH){
          game->players_arr[curr_screen]->s->life += HEALTH_INCREASE;
          if(game->players_arr[curr_screen]->s->life > DEFAULT_LIFE)
          	game->players_arr[curr_screen]->s->life = DEFAULT_LIFE;
        }
        else if(game->globalstate.powerupList[i].type == POWERUP_POWER){
          game->players_arr[curr_screen]->s->attack += POWER_INCREASE;
          if(game->players_arr[curr_screen]->s->attack > MAX_POWER)
          	game->players_arr[curr_screen]->s->attack = MAX_POWER;
        }
        else if(game->globalstate.powerupList[i].type == POWERUP_ARMOR){
          game->players_arr[curr_screen]->s->armor += ARMOR_INCREASE;
          if(game->players_arr[curr_screen]->s->armor > MAX_ARMOR)
          	game->players_arr[curr_screen]->s->armor = MAX_ARMOR;
        }
        else if(game->globalstate.powerupList[i].type == POWERUP_SPEED){
          game->players_arr[curr_screen]->s->speed += SPEED_INCREASE;
          if(game->players_arr[curr_screen]->s->speed > MAX_SPEED)
            game->players_arr[curr_screen]->s->speed = MAX_SPEED;
        }
        r.id = game->globalstate.powerupList[i].id;
        r.curr_screen = game->globalstate.powerupList[i].screen;
        game->globalstate.removeList.push_back(r);
        game->globalstate.powerupList.erase(game->globalstate.powerupList.begin()+i);
      }
    }
    if(magnitude(x,y) < PORTAL_RADIUS)  
    {
      inter = getAngle(x,y);
      for(j=0;j<NUMBER_OF_PORTALS;j++)
      {
        getPortalAngles(j, &upper, &lower, game);
        if((upper < lower && (inter < upper || inter > lower))||(upper > lower && inter < upper && inter > lower))
        {
          new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS;
          while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
            new_screen = rand()%MAX_PLAYERS;   
          float midline = j*degree_range + game->globalstate.portalRotation;
          repositionObject(curr_screen, new_screen, &game->globalstate.powerupList[i].orientation, game->globalstate.powerupList[i].position, inter, midline, game);
          game->globalstate.powerupList[i].screen = new_screen;
          break;
        } 
      }
    }
    //calculate the delta movement
    yrotrad = game->globalstate.powerupList[i].orientation / 180 * PI;
    x = LOCAL_POWERUP_SPEED * (float)sin(yrotrad);
    y = LOCAL_POWERUP_SPEED * (float)cos(yrotrad);
    move_object(game->globalstate.powerupList[i].position, x*time, y*time);
    //If it has passed through a portal, remove it
    if(curr_screen != new_screen)
    {
      r.id = game->globalstate.powerupList[i].id;
      r.curr_screen = curr_screen; 
      game->globalstate.removeList.push_back(r);
    }
  }
}

void move_emps(float time, game_t *game) {
  float yrotrad,x,y,x1,y1,inter,curr_rot,lower,upper;
  int curr_screen,new_screen,degree_range,j;
  unsigned int i;
  struct removeItem r;
  degree_range = 360 / NUMBER_OF_PORTALS; 
  curr_rot = game->globalstate.portalRotation;

  pthread_rwlock_wrlock(&game->elock);
  for(i=0;i<game->globalstate.empList.size();i++)
  {
    x = game->globalstate.empList[i].position[0];
    y = game->globalstate.empList[i].position[1];
    curr_screen = game->globalstate.empList[i].screen;
    new_screen  = curr_screen;

    if(game->globalstate.empList[i].player == -1)
    {
      if(curr_screen < MAX_PLAYERS)
      {
        //For testing purposes.
        //Get the location of the ship on the current screen.
        x1 = game->players_arr[curr_screen]->s->position[0];
        y1 = game->players_arr[curr_screen]->s->position[1];

        //If the ufo is within range.
        if((distance(x,y,x1,y1) <= SHIP_RADIUS))
        {
          //Add to the number of ufos for the certain player and remove the ufo from the screen.
          game->players_arr[curr_screen]->s->num_weapons[5] ++;
          r.id = game->globalstate.empList[i].id;
          r.curr_screen = game->globalstate.empList[i].screen;
          game->globalstate.removeList.push_back(r);
          game->globalstate.empList.erase(game->globalstate.empList.begin()+i);
        }
      }
    }
    else if(curr_screen < MAX_PLAYERS && game->globalstate.empList[i].player != curr_screen)
    {
      x1 = game->players_arr[curr_screen]->s->position[0];
      y1 = game->players_arr[curr_screen]->s->position[1];
      if((distance(x,y,x1,y1) <= SHIP_RADIUS))
      {
        game->players_arr[curr_screen]->s->life -= (COLLISION_DAMAGE - (int)(COLLISION_DAMAGE*ARMOR_PERCENT*game->players_arr[curr_screen]->s->armor));
        r.id = game->globalstate.empList[i].id;
        r.curr_screen = game->globalstate.empList[i].screen;
        game->globalstate.removeList.push_back(r);
        game->globalstate.empList.erase(game->globalstate.empList.begin()+i);
        game->players_arr[curr_screen]->reverseTimer = EMP_TIME;
        game->globalstate.flashScreen[curr_screen]=EMP_FLASH;
        continue;
      }
    }


    if(magnitude(x,y) < PORTAL_RADIUS)  
    {
      inter = getAngle(x,y);
      for(j=0;j<NUMBER_OF_PORTALS;j++)
      {
        getPortalAngles(j, &upper, &lower, game);
        if((upper < lower && (inter < upper || inter > lower))||(upper > lower && inter < upper && inter > lower))
        {
          new_screen = curr_screen > j ? j : (j + 1)%MAX_PLAYERS; 
          while(!game->players_arr[new_screen]->ingame || new_screen == curr_screen)
            new_screen = rand()%MAX_PLAYERS;     
          float midline = j*degree_range + game->globalstate.portalRotation;
          repositionObject(curr_screen, new_screen, &game->globalstate.empList[i].orientation, game->globalstate.empList[i].position, inter, midline, game);
          game->globalstate.empList[i].screen = new_screen;
          if(game->globalstate.empList[i].player != -1)
            game->globalstate.empList[i].active = true;
          break;
        } 
      }
    }
    //calculate the delta movement
    if(game->globalstate.empList[i].active)
      game->globalstate.empList[i].speed -= time*EMP_DECELERATION;
    if(game->globalstate.empList[i].speed < 0)
    {
      r.id = game->globalstate.empList[i].id;
      r.curr_screen = game->globalstate.empList[i].screen;
      game->globalstate.removeList.push_back(r);
      game->globalstate.empList.erase(game->globalstate.empList.begin()+i);
      if(curr_screen < MAX_PLAYERS){
        game->players_arr[curr_screen]->reverseTimer = EMP_TIME;
        game->globalstate.flashScreen[curr_screen]=EMP_FLASH;
      }
    }
    yrotrad = game->globalstate.empList[i].orientation / 180 * PI;
    x = game->globalstate.empList[i].speed * (float)sin(yrotrad);
    y = game->globalstate.empList[i].speed * (float)cos(yrotrad);
    move_object(game->globalstate.empList[i].position, x*time, y*time);
    //If it has passed through a portal, remove it
    if(curr_screen != new_screen)
    {
      r.id = game->globalstate.empList[i].id;
      r.curr_screen = curr_screen; 
      game->globalstate.removeList.push_back(r);
    }
  }
  pthread_rwlock_unlock(&game->elock);
}

void update_object_states(float time, game_t *game) {
  unsigned int i;
  float yrotrad,x,y,curr_rot;
  int curr_screen,degree_range;
  struct removeItem r;
  degree_range = 360 / NUMBER_OF_PORTALS; 
  curr_rot = game->globalstate.portalRotation;

  /**************************************************************
    Begin moving the bullets.
   **************************************************************/  

  pthread_rwlock_wrlock(&game->bllock);
  for(i=0;i<game->globalstate.bulletList.size();i++)
  {
    x = game->globalstate.bulletList[i].position[0];
    y = game->globalstate.bulletList[i].position[1];
    curr_screen = game->globalstate.bulletList[i].screen;

    //If we hit the portal...
    if(magnitude(x,y) < PORTAL_RADIUS)  
    {
      portal_attacked(game,&(game->globalstate.bulletList[i]));
      r.id = game->globalstate.bulletList[i].id;
      r.curr_screen = game->globalstate.bulletList[i].screen;
      game->globalstate.removeList.push_back(r);
      game->globalstate.bulletList.erase(game->globalstate.bulletList.begin()+i);
    }
    //otherwise we have not hit the portal
    else
    {
      //If we have moved our maximum distance.
      game->globalstate.bulletList[i].active = true;
      bool hit = false;
      for(unsigned int j = 0; j < game->globalstate.mineList.size(); j++)
      {
        if(game->globalstate.mineList[j].screen == game->globalstate.bulletList[i].screen && game->globalstate.mineList[j].player != game->globalstate.bulletList[i].player)
        {
          if(magnitude(x-game->globalstate.mineList[j].position[0],y-game->globalstate.mineList[j].position[1]) < (float)MINE_SIZE/Graphics::screen.y/2.0)
          {
            pthread_rwlock_wrlock(&game->mlock);
            game->globalstate.mineList[j].life -= (game->globalstate.bulletList[i].attack - (int)(game->globalstate.bulletList[i].attack*ARMOR_PERCENT*game->players_arr[curr_screen]->s->armor));
            if(game->globalstate.mineList[j].life <= 0)
            {
              r.id = game->globalstate.mineList[j].id;
              r.curr_screen = game->globalstate.mineList[j].screen;
              game->globalstate.removeList.push_back(r);
              mine o = game->globalstate.mineList[j];
              game->globalstate.mineList.erase(game->globalstate.mineList.begin()+j);
              pthread_rwlock_unlock(&game->mlock);
              if(o.player != -1 && ((float)rand() / (float)RAND_MAX) < MINE_SPAWN_CHANCE)
                spawn_random_powerup(game, o.screen, o.position, true);
            }else
              pthread_rwlock_unlock(&game->mlock);
            hit = true;
            break;
          }
        }
      }
      if(!hit)
      {
        for(unsigned int j = 0; j < game->globalstate.UFOList.size(); j++)
        {
          if(game->globalstate.UFOList[j].screen == game->globalstate.bulletList[i].screen && game->globalstate.UFOList[j].player != game->globalstate.bulletList[i].player)
          {
            if(magnitude(x-game->globalstate.UFOList[j].position[0],y-game->globalstate.UFOList[j].position[1]) < (float)UFO_SIZE/Graphics::screen.y/2.0)
            {
              pthread_rwlock_wrlock(&game->ulock);
              game->globalstate.UFOList[j].life -= (game->globalstate.bulletList[i].attack - (int)(game->globalstate.bulletList[i].attack*ARMOR_PERCENT*game->players_arr[curr_screen]->s->armor));
              if(game->globalstate.UFOList[j].life <= 0)
              {
                r.id = game->globalstate.UFOList[j].id;
                r.curr_screen = game->globalstate.UFOList[j].screen;
                game->globalstate.removeList.push_back(r);
                ufo o = game->globalstate.UFOList[j];
                game->globalstate.UFOList.erase(game->globalstate.UFOList.begin()+j);
                pthread_rwlock_unlock(&game->ulock);
                if(o.player != -1 && ((float)rand() / (float)RAND_MAX) < POWERUP_SPAWN_CHANCE)
                  spawn_random_powerup(game, o.screen, o.position, true);
              }else
                pthread_rwlock_unlock(&game->ulock);
              hit = true;
              break;
            }
          }
        }
      }
      if(!hit)
      {
        for(unsigned int j = 0; j < game->globalstate.GunshipList.size(); j++)
        {
          if(game->globalstate.GunshipList[j].screen == game->globalstate.bulletList[i].screen && game->globalstate.GunshipList[j].player != game->globalstate.bulletList[i].player)
          {
            if(magnitude(x-game->globalstate.GunshipList[j].position[0],y-game->globalstate.GunshipList[j].position[1]) < (float)GUNSHIP_SIZE/Graphics::screen.y/2.0)
            {
              pthread_rwlock_wrlock(&game->glock);
              game->globalstate.GunshipList[j].life -= (game->globalstate.bulletList[i].attack - (int)(game->globalstate.bulletList[i].attack*ARMOR_PERCENT*game->players_arr[curr_screen]->s->armor));
              if(game->globalstate.GunshipList[j].life <= 0)
              {
                r.id = game->globalstate.GunshipList[j].id;
                r.curr_screen = game->globalstate.GunshipList[j].screen;
                game->globalstate.removeList.push_back(r);
                gunship o = game->globalstate.GunshipList[j];
                game->globalstate.GunshipList.erase(game->globalstate.GunshipList.begin()+j);
                pthread_rwlock_unlock(&game->glock);
                if(o.player != -1 && ((float)rand() / (float)RAND_MAX) < POWERUP_SPAWN_CHANCE)
                  spawn_random_powerup(game, o.screen, o.position, true);
              }else
                pthread_rwlock_unlock(&game->glock);
              hit = true;
              break;
            }
          }
        }
      }
      if(!hit)
      {
        for(unsigned int j = 0; j < game->globalstate.MineshipList.size(); j++)
        {
          if(game->globalstate.MineshipList[j].screen == game->globalstate.bulletList[i].screen && game->globalstate.MineshipList[j].player != game->globalstate.bulletList[i].player)
          {
            if(magnitude(x-game->globalstate.MineshipList[j].position[0],y-game->globalstate.MineshipList[j].position[1]) < (float)MINESHIP_SIZE/Graphics::screen.y/2.0)
            {
              pthread_rwlock_wrlock(&game->mslock);
              game->globalstate.MineshipList[j].life -= (game->globalstate.bulletList[i].attack - (int)(game->globalstate.bulletList[i].attack*ARMOR_PERCENT*game->players_arr[curr_screen]->s->armor));
              if(game->globalstate.MineshipList[j].life <= 0)
              {
                r.id = game->globalstate.MineshipList[j].id;
                r.curr_screen = game->globalstate.MineshipList[j].screen;
                game->globalstate.removeList.push_back(r);
                mineship o = game->globalstate.MineshipList[j];
                game->globalstate.MineshipList.erase(game->globalstate.MineshipList.begin()+j);
                pthread_rwlock_unlock(&game->mslock);
                if(o.player != -1 && ((float)rand() / (float)RAND_MAX) < POWERUP_SPAWN_CHANCE)
                  spawn_random_powerup(game, o.screen, o.position, true);
              }else
                pthread_rwlock_unlock(&game->mslock);
              hit = true;
              break;
            }
          }
        }
      }
      if(!hit)
      {
        for(unsigned int j = 0; j < game->globalstate.nukeList.size(); j++)
        {
          if(game->globalstate.nukeList[j].screen == game->globalstate.bulletList[i].screen && game->globalstate.nukeList[j].active)
          {
            if(magnitude(x-game->globalstate.nukeList[j].position[0],y-game->globalstate.nukeList[j].position[1]) < (float)NUKE_SIZE/Graphics::screen.y/2.0)
            {
              pthread_rwlock_wrlock(&game->nlock);
              float nx = game->globalstate.nukeList[j].speed*sin(game->globalstate.nukeList[j].orientation/180.0*PI);
              float ny = game->globalstate.nukeList[j].speed*cos(game->globalstate.nukeList[j].orientation/180.0*PI);
              float bx = NUKE_SPEED*sin(game->globalstate.bulletList[i].orientation/180.0*PI);
              float by = NUKE_SPEED*cos(game->globalstate.bulletList[i].orientation/180.0*PI);
              nx += bx*BULLET_EFFECT;
              ny += by*BULLET_EFFECT;
              game->globalstate.nukeList[j].speed=magnitude(nx, ny);
              game->globalstate.nukeList[j].orientation = atan(nx/ny)/PI*180;
              if(by < 0)
                game->globalstate.nukeList[j].orientation += 180;
              pthread_rwlock_unlock(&game->nlock);
              hit = true;
              break;
            }
          }
        }
      }
      if(!hit)
      {
        for(unsigned int j = 0; j < game->globalstate.empList.size(); j++)
        {
          if(game->globalstate.empList[j].screen == game->globalstate.bulletList[i].screen && game->globalstate.empList[j].active)
          {
            if(magnitude(x-game->globalstate.empList[j].position[0],y-game->globalstate.empList[j].position[1]) < (float)EMP_SIZE/Graphics::screen.y/2.0)
            {
              pthread_rwlock_wrlock(&game->elock);
              float ex = game->globalstate.empList[j].speed*sin(game->globalstate.empList[j].orientation/180.0*PI);
              float ey = game->globalstate.empList[j].speed*cos(game->globalstate.empList[j].orientation/180.0*PI);
              float bx = EMP_SPEED*sin(game->globalstate.bulletList[i].orientation/180.0*PI);
              float by = EMP_SPEED*cos(game->globalstate.bulletList[i].orientation/180.0*PI);
              ex += bx*BULLET_EFFECT;
              ey += by*BULLET_EFFECT;
              game->globalstate.empList[j].speed=magnitude(ex, ey);
              game->globalstate.empList[j].orientation = atan(ex/ey)/PI*180;
              if(by < 0)
                game->globalstate.empList[j].orientation += 180;
              pthread_rwlock_unlock(&game->elock);
              hit = true;
              break;
            }
          }
        }
      }
      if(!hit)
      {
        for(unsigned int j = 0; j < game->globalstate.hexagonList.size(); j++)
        {
          if(game->globalstate.hexagonList[j].screen == game->globalstate.bulletList[i].screen && game->globalstate.hexagonList[j].player != game->globalstate.bulletList[i].player)
          {
            if(magnitude(x-game->globalstate.hexagonList[j].position[0],y-game->globalstate.hexagonList[j].position[1]) < (float)game->globalstate.hexagonList[j].size/Graphics::screen.y/2.0)
            {
              pthread_rwlock_wrlock(&game->hlock);
              game->globalstate.hexagonList[j].size -= 10*game->globalstate.bulletList[i].attack;
              if(game->globalstate.hexagonList[j].size <= 100)
              {
                r.id = game->globalstate.hexagonList[j].id;
                r.curr_screen = game->globalstate.hexagonList[j].screen;
                game->globalstate.removeList.push_back(r);
                hexagon o = game->globalstate.hexagonList[j];
                game->globalstate.hexagonList.erase(game->globalstate.hexagonList.begin()+j);
                pthread_rwlock_unlock(&game->hlock);
                if(o.player != -1 && ((float)rand() / (float)RAND_MAX) < POWERUP_SPAWN_CHANCE)
                  spawn_random_powerup(game, o.screen, o.position, true);
              }else
                pthread_rwlock_unlock(&game->hlock);
              hit = true;
              break;
            }
          }
        }
      }
      if(!hit)
      {
        for(unsigned int j = 0; j < game->globalstate.powerupList.size(); j++)
        {
          if(game->globalstate.powerupList[j].screen == game->globalstate.bulletList[i].screen)
          {
            if(magnitude(x-game->globalstate.powerupList[j].position[0],y-game->globalstate.powerupList[j].position[1]) < (float)POWERUP_SIZE/Graphics::screen.y/2.0)
            {
              pthread_rwlock_wrlock(&game->plock);
              r.id = game->globalstate.powerupList[j].id;
              r.curr_screen = game->globalstate.powerupList[j].screen;
              game->globalstate.removeList.push_back(r);
              game->globalstate.powerupList.erase(game->globalstate.powerupList.begin()+j);
              pthread_rwlock_unlock(&game->plock);
              hit = true;
              break;
            }
          }
        }
      }
      if(!hit)
      {
        for(unsigned int j = 0; j < game->globalstate.lifeList.size(); j++)
        {
          if(game->globalstate.lifeList[j].player == game->globalstate.bulletList[i].screen && game->globalstate.lifeList[j].player != game->globalstate.bulletList[i].player)
          {
            if(magnitude(x-game->globalstate.lifeList[j].position[0],y-game->globalstate.lifeList[j].position[1]) < (float)SHIP_SIZE/Graphics::screen.y/2.0)
            {
              pthread_rwlock_wrlock(&game->lllock);
              game->globalstate.lifeList[j].life -= (game->globalstate.bulletList[i].attack - (int)(game->globalstate.bulletList[i].attack*ARMOR_PERCENT*game->players_arr[curr_screen]->s->armor));
              pthread_rwlock_unlock(&game->lllock);
              hit = true;
              break;
            }
          }
        }
      }
      
      //Only delete it from the server if it has gone too far
      if(hit || game->globalstate.bulletList[i].distance > game->globalstate.bulletList[i].speed*8)
      {
        r.id = game->globalstate.bulletList[i].id;
        r.curr_screen = game->globalstate.bulletList[i].screen;
        game->globalstate.removeList.push_back(r);
        game->globalstate.bulletList.erase(game->globalstate.bulletList.begin()+i);
      }
      //otherwise keep the bullet moving.
      else
      {
        yrotrad = game->globalstate.bulletList[i].orientation / 180 * PI;
        x = game->globalstate.bulletList[i].speed * (float)sin(yrotrad);
        y = game->globalstate.bulletList[i].speed * (float)cos(yrotrad);
        move_object(game->globalstate.bulletList[i].position, x*time, y*time);
        game->globalstate.bulletList[i].distance += magnitude(x*time,y*time);
      }
    }
  }
  pthread_rwlock_unlock(&game->bllock);
  /**************************************************************
    End moving the bullets.
   **************************************************************/  

  move_nukes(time,game);
  move_mines(time,game);
  move_mineships(time,game);
  move_UFOS(time,game);
  move_gunships(time,game);
  move_emps(time,game);
  move_hexs(time,game);
  move_powerups(time,game);
}

float distance(float x1,float y1, float x2, float y2)
{
  float x = (x1 - x2);
  float y = (y1 - y2);
  return sqrt((x * x) + (y * y));
}

void ship_rotate(ship *s, int dir, float time)
{
  s->orientation += ROTATION_RATE*time*dir*(1+(s->speed/SPEED_DAMPENER));
  while(s->orientation >= 360)
    s->orientation -= 360;
  while(s->orientation < 0)
    s->orientation += 360;
}

void ship_accelerate(ship *s, float time){
  s->velocity[0] += ACCELERATION*time*sin(s->orientation*PI/180)*(1+(s->speed/SPEED_DAMPENER));
  s->velocity[1] += ACCELERATION*time*cos(s->orientation*PI/180)*(1+(s->speed/SPEED_DAMPENER));

  float cv = magnitude(s->velocity[0],s->velocity[1]);
  if(cv > MAX_SHIP_VELOCITY)
    scale(&(s->velocity[0]),&(s->velocity[1]),MAX_SHIP_VELOCITY,cv);
}

void ship_deccelerate(ship *s, float time) {
  if(s->velocity[0] > 0){
    s->velocity[0] -= ACCELERATION*time*(1+(s->speed/SPEED_DAMPENER));
    if(s->velocity[0] < 0)
      s->velocity[0] = 0;
  }
  else if (s->velocity[0] < 0){
    s->velocity[0] += ACCELERATION*time*(1+(s->speed/SPEED_DAMPENER));
    if(s->velocity[0] > 0)
      s->velocity[0] = 0;
  }
  if(s->velocity[1] > 0){
    s->velocity[1] -= ACCELERATION*time*(1+(s->speed/SPEED_DAMPENER));
    if(s->velocity[1] < 0)
      s->velocity[1] = 0;
  }
  else if (s->velocity[1] < 0){
    s->velocity[1] += ACCELERATION*time*(1+(s->speed/SPEED_DAMPENER));
    if(s->velocity[1] > 0)
      s->velocity[1] = 0;
  }
}

void move_ship(ship *s, float time)
{
  float mag;
  float perpx = s->position[0];
  float perpy = s->position[1];
  float velx  = s->velocity[0];
  float vely  = s->velocity[1];

  if((mag = magnitude(perpx,perpy)) < PORTAL_RADIUS)  
  {
    float x = perpx / mag;
    float y = perpy / mag;
    float ndotl2 = ((x * velx) + (y * vely));
    s->velocity[0] = velx - (ndotl2 * x * 2.0f); 
    s->velocity[1] = vely - (ndotl2 * y * 2.0f);
  }

  s->position[0]+=s->velocity[0]*time;
  s->position[1]+=s->velocity[1]*time;
  move_object(s->position, s->velocity[0]*time, s->velocity[1]*time);
}

void move_object(float *position, float x, float y){
  position[0] += x;
  position[1] += y;
  while(position[0] > 1)
    position[0] -= 2;
  while(position[0] < -1)
    position[0] += 2;
  while(position[1] > 1)
    position[1] -= 2;
  while(position[1] < -1)
    position[1] += 2;
}

float magnitude(float x, float y)
{
  return sqrt((x * x) + (y * y));
}

float getAngle(float x, float y){
  float angle = 0;
  if(x != 0)
    angle = atan((y/x)) * 180 / PI;     

  if(x == 0.0f && y >  0.0f)
    angle = 0.0f;
  else if(x >  0.0f && y >  0.0f)
    angle = 90.0f - angle;
  else if(x >  0.0f && y == 0.0f)
    angle = 90.0f;
  else if(x >  0.0f && y <  0.0f)
    angle = 90.0f - angle;
  else if(x == 0.0f && y <  0.0f)
    angle = 180.0f;
  else if(x <  0.0f && y <  0.0f)
    angle = 270.0f - angle;
  else if(x <  0.0f && y == 0.0f)
    angle = 270.0f;
  else if(x <  0.0f && y >  0.0f)
    angle = 270.0f - angle;
  return angle;
}

void getPortalAngles(int portal, float *upper, float *lower, game_t *game){
  float degree_range = 360 / NUMBER_OF_PORTALS;
  *upper =  (degree_range / 2.0) + (degree_range * portal) + game->globalstate.portalRotation;
  *lower = -(degree_range / 2.0) + (degree_range * portal) + game->globalstate.portalRotation;
  if(*upper >= 360.0f)
    *upper -= 360.0f;
  if(*upper < 0.0f)
    *upper += 360.0f;
  if(*lower >= 360.0f)
    *lower -= 360.0f;
  if(*lower < 0.0f)
    *lower += 360.0f;
}

float clamp(float value){
  while(value < 0)
  	value += 360;
  while(value >= 360)
  	value -= 360;
  return value;
}

void repositionObject(int old_screen, int new_screen, float *orientation, float *position, float angle, float midline, game_t *game){
  float degree_range = 360 / NUMBER_OF_PORTALS;
  int portal = new_screen >= old_screen ? old_screen : old_screen - 1;
  float newmidline = portal*degree_range + game->globalstate.portalRotation;
  midline = clamp(midline);
  newmidline = clamp(newmidline);
  *orientation = newmidline;//360 - *orientation - midline + newmidline;
  angle = newmidline/180.0*PI;
  *orientation = clamp(*orientation);
  angle = clamp(angle);
  position[0] = PORTAL_RADIUS*sin(angle);
  position[1] = PORTAL_RADIUS*cos(angle);
}

void scale(float *x,float *y,float s,float cv)
{
  float tempx = ((*x)/cv) * s;
  float tempy = ((*y)/cv) * s;

  *x = tempx;
  *y = tempy;
}

void update_player(player_t *player, char *contents, char action)
{
  game_t *game = player->game;

  switch(contents[0])
  {
    case 'U':
      player->U = action;
      break;
    case 'D':
      player->D = action;
      break;
    case 'R':
      player->R = action;
      break;
    case 'L':
      player->L = action;
      break;
    case 'S':
      bullet_initialize(player);
      break;
    case 'T':
      pthread_rwlock_wrlock(&game->lllock);
      if(contents[1] < 7)
        game->globalstate.lifeList[player->id].curr_weapon = contents[1];
      pthread_rwlock_unlock(&game->lllock);

      break;
    case 'A':
      initialize_special_weapon(player->id, player->id, game->globalstate.lifeList[player->id].position, false, game);
      fire_special_weapon();
      break;
    default: 
      break;
  }
}

void bullet_initialize(player_t * player)
{
  printf("%d bullets\n", player->game->num_bullets);
  bullet_create(player); 
  player->game->num_bullets++;
}

void initialize_special_weapon(int player, int screen, float *position, bool random, game_t *game)
{
  int weapon = 0;
  float orientation;

  if(random) {
    weapon = rand() % 7;
    orientation = rand() % 360;
  }else {
    weapon = game->globalstate.lifeList[player].curr_weapon;
    orientation = game->globalstate.lifeList[player].orientation;
    char num = game->globalstate.lifeList[player].num_weapons[weapon];
    if(num == (char)0)
      return;
    else
      game->globalstate.lifeList[player].num_weapons[weapon] = (char)((int)num - 1);
  }

  switch(weapon)
  {
    case 0:

      mine new_mine;
      pthread_rwlock_wrlock(&game->itemlock);
      new_mine.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);

      pthread_rwlock_wrlock(&game->lllock);
      new_mine.position[0] = position[0];
      new_mine.position[1] = position[1];
      new_mine.orientation = orientation;
      pthread_rwlock_unlock(&game->lllock);
      new_mine.player = player;
      new_mine.screen = screen;
      new_mine.active = false;
      new_mine.distance = 0;
      new_mine.life   = MINE_LIFE;

      pthread_rwlock_wrlock(&game->mlock);
      game->globalstate.mineList.push_back(new_mine);
      pthread_rwlock_unlock(&game->mlock);
      break;

    case 1:
      ufo new_ufo;
      pthread_rwlock_wrlock(&game->itemlock);
      new_ufo.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);
      pthread_rwlock_wrlock(&game->lllock);
      new_ufo.position[0] = position[0];
      new_ufo.position[1] = position[1];
      new_ufo.orientation = orientation;
      pthread_rwlock_unlock(&game->lllock);

      new_ufo.player = player;
      new_ufo.screen = player;
      new_ufo.active = false;
      new_ufo.life   = UFO_LIFE;

      pthread_rwlock_wrlock(&game->ulock);
      game->globalstate.UFOList.push_back(new_ufo);
      pthread_rwlock_unlock(&game->ulock);

      break;

    case 2:
      gunship new_gunship;
      pthread_rwlock_wrlock(&game->itemlock);
      new_gunship.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++ ;
      pthread_rwlock_unlock(&game->itemlock);

      pthread_rwlock_wrlock(&game->lllock);
      new_gunship.position[0] = position[0];
      new_gunship.position[1] = position[1];
      new_gunship.orientation = orientation;
      pthread_rwlock_unlock(&game->lllock);
      new_gunship.player = player;
      new_gunship.screen = player;
      new_gunship.active = false;
      new_gunship.shotTimer = GUNSHIP_SHOT_TIME;
      new_gunship.life   = GUNSHIP_LIFE;

      pthread_rwlock_wrlock(&game->glock);
      game->globalstate.GunshipList.push_back(new_gunship);
      pthread_rwlock_unlock(&game->glock);

      break;

    case 3:
      mineship new_mineship;
      pthread_rwlock_wrlock(&game->itemlock);
      new_mineship.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);

      pthread_rwlock_wrlock(&game->lllock);
      new_mineship.position[0] = position[0];
      new_mineship.position[1] = position[1];
      new_mineship.orientation = orientation;
      pthread_rwlock_unlock(&game->lllock);
      new_mineship.mineCD = MINESHIP_TIME; 
      new_mineship.player = player;
      new_mineship.screen = player;
      new_mineship.active = false;
      new_mineship.life   = MINESHIP_LIFE;

      pthread_rwlock_wrlock(&game->mslock);
      game->globalstate.MineshipList.push_back(new_mineship);
      pthread_rwlock_unlock(&game->mslock);

      break;

    case 4:
      nuke new_nuke;
      pthread_rwlock_wrlock(&game->itemlock);
      new_nuke.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);
      pthread_rwlock_wrlock(&game->lllock);
      new_nuke.position[0] = position[0];
      new_nuke.position[1] = position[1];
      new_nuke.orientation = orientation;
      pthread_rwlock_unlock(&game->lllock);

      new_nuke.timer = 10;

      new_nuke.player = player;
      new_nuke.screen = player;
      new_nuke.active = false;
      new_nuke.life   = NUKE_LIFE;
      new_nuke.speed  = NUKE_SPEED;

      pthread_rwlock_wrlock(&game->nlock);
      game->globalstate.nukeList.push_back(new_nuke);
      pthread_rwlock_unlock(&game->nlock);

      break;

    case 5:
      emp new_emp;
      pthread_rwlock_wrlock(&game->itemlock);
      new_emp.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);
      pthread_rwlock_wrlock(&game->lllock);
      new_emp.position[0] = position[0];
      new_emp.position[1] = position[1];
      new_emp.orientation = orientation;
      pthread_rwlock_unlock(&game->lllock);

      new_emp.timer = 10;

      new_emp.screen = player;
      new_emp.player = player;
      new_emp.active = false;
      new_emp.life   = EMP_LIFE;
      new_emp.speed  = EMP_SPEED;

      pthread_rwlock_wrlock(&game->elock);
      game->globalstate.empList.push_back(new_emp);
      pthread_rwlock_unlock(&game->elock);

      break;
    case 6:
      hexagon new_hexagon;
      pthread_rwlock_wrlock(&game->itemlock);
      new_hexagon.id = game->globalstate.itemnumber;
      game->globalstate.itemnumber++;
      pthread_rwlock_unlock(&game->itemlock);
      pthread_rwlock_wrlock(&game->lllock);
      new_hexagon.position[0] = position[0];
      new_hexagon.position[1] = position[1];
      new_hexagon.orientation = orientation;
      pthread_rwlock_unlock(&game->lllock);

      new_hexagon.size = 100;  
      new_hexagon.screen = player;
      new_hexagon.player = player;
      new_hexagon.active = false;

      pthread_rwlock_wrlock(&game->hlock);
      game->globalstate.hexagonList.push_back(new_hexagon);
      pthread_rwlock_unlock(&game->hlock);

      break;
  }
}

void bullet_create(player_t *player)
{
  game_t *game = player->game;

  bullet new_bullet;

  pthread_rwlock_wrlock(&game->itemlock);
  new_bullet.id          = game->globalstate.itemnumber;
  game->globalstate.itemnumber ++;
  pthread_rwlock_unlock(&game->itemlock);

  pthread_rwlock_wrlock(&game->lllock);
  new_bullet.attack      = game->globalstate.lifeList[player->id].attack;
  new_bullet.position[0] = game->globalstate.lifeList[player->id].position[0];
  new_bullet.position[1] = game->globalstate.lifeList[player->id].position[1];
  new_bullet.orientation = game->globalstate.lifeList[player->id].orientation;
  new_bullet.player      = game->globalstate.lifeList[player->id].player;
  new_bullet.screen      = game->globalstate.lifeList[player->id].player;
  float x = game->globalstate.lifeList[player->id].velocity[0];
  float y = game->globalstate.lifeList[player->id].velocity[1];
  new_bullet.speed		 = sqrt(x*x+y*y)+ BULLET_SPEED;
  pthread_rwlock_unlock(&game->lllock);
  new_bullet.distance    = 0.0f;
  new_bullet.active      = false;

  pthread_rwlock_wrlock(&game->bllock);
  game->globalstate.bulletList.push_back(new_bullet);
  pthread_rwlock_unlock(&game->bllock);
}

void fire_special_weapon() { }

int main(int argc, char **argv) {
  int listenfd, port;
  socklen_t clientlen = sizeof(struct sockaddr_in);
  pthread_t tid;
  srand(time(NULL));
  if(argc != 2) {
    fprintf(stderr, "usage: %s <port>\n", argv[0]);
    exit(0);
  }
  port = atoi(argv[1]);

  struct sigaction sa;
  memset(&sa, 0, sizeof(sa));
  sa.sa_handler = SIG_IGN;
  sigaction(SIGPIPE, &sa, NULL);

  listenfd = Open_listenfd(port);
  while (1) {
    player_t player;
    player.game = NULL;
    player.fd = Accept(listenfd, (SA *) &(player.addr), &clientlen);
    cout << "A client connected" << endl;

    Pthread_create(&tid, NULL, thread, (void *)(&player));
  }
}
